@@ -21,6 +21,7 @@ import itertools
 import os
 import re
 import shutil
+import signal
 import time
 
 import netaddr
@@ -46,6 +47,7 @@ from neutron.ipam import utils as ipam_utils
 from neutron.privileged.agent.linux import dhcp as priv_dhcp
 
 LOG = logging.getLogger(__name__)
+SIGTERM_TIMEOUT = 5
 
 DNS_PORT = 53
 WIN2k3_STATIC_DNS = 249
@@ -183,19 +185,20 @@ class NetModel(DictModel):
 class DhcpBase(object, metaclass=abc.ABCMeta):
 
     def __init__(self, conf, network, process_monitor,
-                 version=None, plugin=None):
+                 version=None, plugin=None, segment=None):
         self.conf = conf
         self.network = network
         self.process_monitor = process_monitor
         self.device_manager = DeviceManager(self.conf, plugin)
         self.version = version
+        self.segment = segment
 
     @abc.abstractmethod
     def enable(self):
         """Enables DHCP for this network."""
 
     @abc.abstractmethod
-    def disable(self, retain_port=False, block=False):
+    def disable(self, retain_port=False, block=False, **kwargs):
         """Disable dhcp for this network."""
 
     def restart(self):
@@ -234,18 +237,56 @@ class DhcpBase(object, metaclass=abc.ABCMeta):
         """True if the metadata-proxy should be enabled for the network."""
         raise NotImplementedError()
 
+    @abc.abstractmethod
+    def clean_devices(self):
+        """Request to clean unnecessary devices for the network"""
+
 
 class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
     PORTS = []
 
+    # Track running interfaces, indexed by network ID, for example,
+    # {net-id-1: set(intf_1, intf_2), net-id-2: set(intf_3, intf_4), ...}
+    _interfaces = collections.defaultdict(set)
+
     def __init__(self, conf, network, process_monitor, version=None,
-                 plugin=None):
+                 plugin=None, segment=None):
         super(DhcpLocalProcess, self).__init__(conf, network, process_monitor,
-                                               version, plugin)
+                                               version, plugin, segment)
         self.confs_dir = self.get_confs_dir(conf)
-        self.network_conf_dir = os.path.join(self.confs_dir, network.id)
+        if self.segment:
+            # In case of multi-segments support we want a dns process per vlan.
+            self.network_conf_dir = os.path.join(
+                # NOTE(sahid): Path of dhcp conf will be /<segid>/<netid>. We
+                # don't do the opposite so we can clean /<netid>* when calling
+                # disable of the legacy port that is not taking care of
+                # segmentation.
+                self.confs_dir, str(self.segment.segmentation_id), network.id)
+        else:
+            self.network_conf_dir = os.path.join(self.confs_dir, network.id)
+
         fileutils.ensure_tree(self.network_conf_dir, mode=0o755)
 
+    @classmethod
+    def _add_running_interface(cls, interface, network_id):
+        """Safe method that adds a given interface"""
+        cls._interfaces[network_id].add(interface)
+
+    @classmethod
+    def _del_running_interface(cls, interface, network_id):
+        """Safe method that removes a given interface"""
+        if cls._interfaces.get(network_id):
+            if interface in cls._interfaces[network_id]:
+                cls._interfaces[network_id].remove(interface)
+            # no entries, cleanup
+            if not cls._interfaces[network_id]:
+                del cls._interfaces[network_id]
+
+    @classmethod
+    def _has_running_interfaces(cls, network_id):
+        """Safe method that checks for interfaces"""
+        bool(cls._interfaces.get(network_id))
+
     @staticmethod
     def get_confs_dir(conf):
         return os.path.abspath(os.path.normpath(conf.dhcp_confs))
@@ -254,6 +295,14 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
         """Returns the file name for a given kind of config file."""
         return os.path.join(self.network_conf_dir, kind)
 
+    def get_process_uuid(self):
+        if self.segment:
+            # NOTE(sahid): Keep the order to match directory path. This is used
+            # by external_process.ProcessManager to check whether the process
+            # is active.
+            return "%s/%s" % (self.segment.segmentation_id, self.network.id)
+        return self.network.id
+
     def _remove_config_files(self):
         shutil.rmtree(self.network_conf_dir, ignore_errors=True)
 
@@ -284,9 +333,12 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
 
             if self._enable_dhcp():
                 fileutils.ensure_tree(self.network_conf_dir, mode=0o755)
-                interface_name = self.device_manager.setup(self.network)
+                interface_name = self.device_manager.setup(
+                    self.network, self.segment)
                 self.interface_name = interface_name
                 self.spawn_process()
+                self._add_running_interface(self.interface_name,
+                                            self.network.id)
             return True
         except exceptions.ProcessExecutionError as error:
             LOG.debug("Spawning DHCP process for network %s failed; "
@@ -296,35 +348,48 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
     def _get_process_manager(self, cmd_callback=None):
         return external_process.ProcessManager(
             conf=self.conf,
-            uuid=self.network.id,
+            uuid=self.get_process_uuid(),
             namespace=self.network.namespace,
             service=DNSMASQ_SERVICE_NAME,
             default_cmd_callback=cmd_callback,
             pid_file=self.get_conf_file_name('pid'),
             run_as_root=True)
 
-    def disable(self, retain_port=False, block=False):
+    def disable(self, retain_port=False, block=False, **kwargs):
         """Disable DHCP for this network by killing the local process."""
-        self.process_monitor.unregister(self.network.id, DNSMASQ_SERVICE_NAME)
-        self._get_process_manager().disable()
+        pm = self._get_process_manager()
+        self.process_monitor.unregister(pm.uuid, DNSMASQ_SERVICE_NAME)
+        pm.disable(sig=str(int(signal.SIGTERM)))
         if block:
-            common_utils.wait_until_true(lambda: not self.active)
+            try:
+                common_utils.wait_until_true(lambda: not self.active,
+                                             timeout=SIGTERM_TIMEOUT)
+            except common_utils.WaitTimeout:
+                LOG.warning('dnsmasq process %s did not finish after SIGTERM '
+                            'signal in %s seconds, sending SIGKILL signal',
+                            pm.pid, SIGTERM_TIMEOUT)
+                pm.disable(sig=str(int(signal.SIGKILL)))
+                common_utils.wait_until_true(lambda: not self.active)
+        self._del_running_interface(self.interface_name, self.network.id)
         if not retain_port:
             self._destroy_namespace_and_port()
             self._remove_config_files()
 
     def _destroy_namespace_and_port(self):
         try:
-            self.device_manager.destroy(self.network, self.interface_name)
+            self.device_manager.destroy(
+                self.network, self.interface_name, self.segment)
         except RuntimeError:
             LOG.warning('Failed trying to delete interface: %s',
                         self.interface_name)
-
-        try:
-            ip_lib.delete_network_namespace(self.network.namespace)
-        except RuntimeError:
-            LOG.warning('Failed trying to delete namespace: %s',
-                        self.network.namespace)
+        # Delete namespace only if there are no running interfaces in it,
+        # which covers the case where a network has multiple segmentation ids.
+        if not self._has_running_interfaces(self.network.id):
+            try:
+                ip_lib.delete_network_namespace(self.network.namespace)
+            except RuntimeError:
+                LOG.warning('Failed trying to delete namespace: %s',
+                            self.network.namespace)
 
     def _get_value_from_conf_file(self, kind, converter=None):
         """A helper function to read a value from one of the state files."""
@@ -333,10 +398,9 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
             with open(file_name, 'r') as f:
                 return converter(f.read()) if converter else f.read()
         except ValueError:
-            msg = "Unable to convert value in %s"
+            LOG.debug("Unable to convert value in %s", file_name)
         except IOError:
-            msg = "Unable to access %s"
-        LOG.debug(msg, file_name)
+            LOG.debug("Unable to access %s", file_name)
         return None
 
     @property
@@ -359,6 +423,10 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
     def spawn_process(self):
         pass
 
+    def clean_devices(self):
+        return self.device_manager.cleanup_stale_devices(
+            self.network, dhcp_port=None)
+
 
 class Dnsmasq(DhcpLocalProcess):
     # The ports that need to be opened when security policies are active
@@ -533,7 +601,7 @@ class Dnsmasq(DhcpLocalProcess):
 
         pm.enable(reload_cfg=reload_with_HUP, ensure_active=True)
 
-        self.process_monitor.register(uuid=self.network.id,
+        self.process_monitor.register(uuid=pm.uuid,
                                       service_name=DNSMASQ_SERVICE_NAME,
                                       monitored_process=pm)
 
@@ -660,7 +728,7 @@ class Dnsmasq(DhcpLocalProcess):
         for fip in fixed_ips:
             if (fip.subnet_id in v6_nets and
                     v6_nets[fip.subnet_id].ipv6_address_mode == (
-                            constants.DHCPV6_STATEFUL)):
+                        constants.DHCPV6_STATEFUL)):
                 if fip.subnet_id not in by_subnet:
                     by_subnet.update({fip.subnet_id: []})
                 by_subnet[fip.subnet_id].append(fip.ip_address)
@@ -1141,11 +1209,15 @@ class Dnsmasq(DhcpLocalProcess):
         return name
 
     def _get_ovn_metadata_port_ip(self, subnet):
-        m_ports = [port for port in self.network.ports if
-                   self._is_ovn_metadata_port(port, self.network.id)]
-        if m_ports:
-            port = self.device_manager.plugin.get_dhcp_port(m_ports[0].id)
-            for fixed_ip in port.fixed_ips:
+        """Check if provided subnet contains OVN metadata port"""
+        ports_result = self.device_manager.plugin.get_ports(
+            port_filters={
+                'device_owner': [constants.DEVICE_OWNER_DISTRIBUTED],
+                'device_id': ['ovnmeta-' + self.network.id]
+            },
+        )
+        if ports_result:
+            for fixed_ip in ports_result[0].get('fixed_ips', []):
                 if fixed_ip.subnet_id == subnet.id:
                     return fixed_ip.ip_address
 
@@ -1159,8 +1231,8 @@ class Dnsmasq(DhcpLocalProcess):
             addr_mode = getattr(subnet, 'ipv6_address_mode', None)
             segment_id = getattr(subnet, 'segment_id', None)
             if (not subnet.enable_dhcp or
-                (subnet.ip_version == 6 and
-                 addr_mode == constants.IPV6_SLAAC)):
+                    (subnet.ip_version == 6 and
+                     addr_mode == constants.IPV6_SLAAC)):
                 continue
             if subnet.dns_nameservers:
                 if common_utils.is_dns_servers_any_address(
@@ -1452,12 +1524,14 @@ class DeviceManager(object):
         """Return interface(device) name for use by the DHCP process."""
         return self.driver.get_device_name(port)
 
-    def get_device_id(self, network):
+    def get_device_id(self, network, segment=None):
         """Return a unique DHCP device ID for this host on the network."""
         # There could be more than one dhcp server per network, so create
         # a device id that combines host and network ids
+        segmentation_id = segment.segmentation_id if segment else None
         return common_utils.get_dhcp_agent_device_id(network.id,
-                                                     self.conf.host)
+                                                     self.conf.host,
+                                                     segmentation_id)
 
     def _set_default_route_ip_version(self, network, device_name, ip_version):
         device = ip_lib.IPDevice(device_name, namespace=network.namespace)
@@ -1498,7 +1572,7 @@ class DeviceManager(object):
                 # gateway being replaced, if it is outside the subnet
                 is_old_gateway_not_in_subnet = (gateway and
                                                 not ipam_utils.check_subnet_ip(
-                                                        subnet.cidr, gateway))
+                                                    subnet.cidr, gateway))
                 if is_old_gateway_not_in_subnet:
                     onlink = device.route.list_onlink_routes(ip_version)
                     existing_onlink_routes = set(r['cidr'] for r in onlink)
@@ -1507,8 +1581,8 @@ class DeviceManager(object):
 
                 is_new_gateway_not_in_subnet = (subnet.gateway_ip and
                                                 not ipam_utils.check_subnet_ip(
-                                                        subnet.cidr,
-                                                        subnet.gateway_ip))
+                                                    subnet.cidr,
+                                                    subnet.gateway_ip))
                 if is_new_gateway_not_in_subnet:
                     device.route.add_route(subnet.gateway_ip, scope='link')
                 device.route.add_gateway(subnet.gateway_ip)
@@ -1658,11 +1732,11 @@ class DeviceManager(object):
                   {'dhcp_port': dhcp_port,
                    'updated_dhcp_port': updated_dhcp_port})
 
-    def setup_dhcp_port(self, network):
+    def setup_dhcp_port(self, network, segment=None):
         """Create/update DHCP port for the host if needed and return port."""
 
         # The ID that the DHCP port will have (or already has).
-        device_id = self.get_device_id(network)
+        device_id = self.get_device_id(network, segment)
 
         # Get the set of DHCP-enabled local subnets on this network.
         dhcp_subnets = {subnet.id: subnet for subnet in network.subnets
@@ -1708,20 +1782,24 @@ class DeviceManager(object):
         else:
             network.ports.append(port)
 
-    def _cleanup_stale_devices(self, network, dhcp_port):
-        """Unplug any devices found in the namespace except for dhcp_port."""
+    def cleanup_stale_devices(self, network, dhcp_port):
+        """Unplug unrelated or stale devices found in the namespace."""
         LOG.debug("Cleaning stale devices for network %s", network.id)
         skip_dev_name = (self.driver.get_device_name(dhcp_port)
                          if dhcp_port else None)
         ns_ip = ip_lib.IPWrapper(namespace=network.namespace)
         if not ns_ip.netns.exists(network.namespace):
             return
-        for d in ns_ip.get_devices():
-            # delete all devices except current active DHCP port device
-            if d.name != skip_dev_name:
-                LOG.debug("Found stale device %s, deleting", d.name)
+
+        db_ports = {self.get_interface_name(network, port)
+                    for port in network.ports}
+        hw_ports = {d.name for d in ns_ip.get_devices()}
+
+        for dev_name in hw_ports - db_ports:
+            if dev_name != skip_dev_name:
+                LOG.debug("Found stale device %s, deleting", dev_name)
                 try:
-                    self.unplug(d.name, network)
+                    self.unplug(dev_name, network)
                 except Exception:
                     LOG.exception("Exception during stale "
                                   "dhcp device cleanup")
@@ -1735,15 +1813,15 @@ class DeviceManager(object):
                          namespace=network.namespace,
                          mtu=network.get('mtu'))
 
-    def setup(self, network):
+    def setup(self, network, segment=None):
         """Create and initialize a device for network's DHCP on this host."""
         try:
-            port = self.setup_dhcp_port(network)
+            port = self.setup_dhcp_port(network, segment)
         except Exception:
             with excutils.save_and_reraise_exception():
                 # clear everything out so we don't leave dangling interfaces
                 # if setup never succeeds in the future.
-                self._cleanup_stale_devices(network, dhcp_port=None)
+                self.cleanup_stale_devices(network, dhcp_port=None)
         self._update_dhcp_port(network, port)
         interface_name = self.get_interface_name(network, port)
 
@@ -1811,7 +1889,7 @@ class DeviceManager(object):
                             namespace=network.namespace)
 
         self._set_default_route(network, interface_name)
-        self._cleanup_stale_devices(network, port)
+        self.cleanup_stale_devices(network, port)
 
         return interface_name
 
@@ -1823,7 +1901,7 @@ class DeviceManager(object):
         """Unplug device settings for the network's DHCP on this host."""
         self.driver.unplug(device_name, namespace=network.namespace)
 
-    def destroy(self, network, device_name):
+    def destroy(self, network, device_name, segment=None):
         """Destroy the device used for the network's DHCP on this host."""
         if device_name:
             self.unplug(device_name, network)
@@ -1831,7 +1909,7 @@ class DeviceManager(object):
             LOG.debug('No interface exists for network %s', network.id)
 
         self.plugin.release_dhcp_port(network.id,
-                                      self.get_device_id(network))
+                                      self.get_device_id(network, segment))
 
     def fill_dhcp_udp_checksums(self, namespace):
         """Ensure DHCP reply packets always have correct UDP checksums."""
