@@ -18,14 +18,15 @@ import time
 
 import netaddr
 from neutron_lib.agent.linux import interface
+from neutron_lib.api import converters
 from neutron_lib import constants
 from neutron_lib import exceptions
 from neutron_lib.plugins.ml2 import ovs_constants as ovs_const
 from oslo_log import log as logging
 from oslo_utils import excutils
-from pyroute2.netlink import exceptions \
-    as pyroute2_exc  # pylint: disable=no-name-in-module
+from pyroute2.netlink import exceptions as pyroute2_exc
 
+from neutron._i18n import _
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import ip_lib
 from neutron.common import utils
@@ -227,7 +228,7 @@ class LinuxInterfaceDriver(interface.LinuxInterfaceDriver,
 
     def get_ipv6_llas(self, device_name, namespace):
         kwargs = {'family': utils.get_socket_address_family(
-                                constants.IP_VERSION_6),
+            constants.IP_VERSION_6),
                   'scope': 'link'}
         return ip_lib.get_devices_with_ip(namespace, name=device_name,
                                           **kwargs)
@@ -336,26 +337,35 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         ovs.replace_port(device_name, *attrs)
 
     def _set_device_address(self, device, mac_address):
+        device.link.set_address(mac_address)
+        current_mac = converters.convert_to_sanitized_mac_address(
+            device.link.address)
+        if current_mac != mac_address:
+            msg = _("Failed to set mac address to: %s; "
+                    "Current mac: %s") % (mac_address, current_mac)
+            raise RuntimeError(msg)
+
+    def _ensure_device_address(self, device, mac_address):
+        mac_address = converters.convert_to_sanitized_mac_address(mac_address)
         for i in range(9):
             # workaround for the OVS shy port syndrome. ports sometimes
             # hide for a bit right after they are first created.
             # see bug/1618987
             try:
-                device.link.set_address(mac_address)
-                break
+                self._set_device_address(device, mac_address)
+                return
             except RuntimeError as e:
                 LOG.warning("Got error trying to set mac, retrying: %s",
                             str(e))
                 time.sleep(1)
-        else:
-            # didn't break, we give it one last shot without catching
-            device.link.set_address(mac_address)
+        # didn't break, we give it one last shot without catching
+        self._set_device_address(device, mac_address)
 
     def _add_device_to_namespace(self, ip_wrapper, device, namespace):
         namespace_obj = ip_wrapper.ensure_namespace(namespace)
         for i in range(9):
             try:
-                namespace_obj.add_device_to_namespace(device)
+                namespace_obj.add_device_to_namespace(device, is_ovs_port=True)
                 break
             except ip_lib.NetworkInterfaceNotFound:
                 # NOTE(slaweq): if the exception was NetworkInterfaceNotFound
@@ -364,11 +374,6 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
                 LOG.warning("Failed to set interface %s into namespace %s. "
                             "Interface not found, attempt: %s, retrying.",
                             device, namespace, i + 1)
-                # NOTE(slaweq) In such case it's required to reset device's
-                # namespace as it was already set to the "namespace"
-                # and after retry neutron will look for it in that namespace
-                # which is wrong
-                device.namespace = None
                 time.sleep(1)
             except utils.WaitTimeout:
                 # NOTE(slaweq): if the exception was WaitTimeout then it means
@@ -401,12 +406,15 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
             root_dev.disable_ipv6()
         else:
             ns_dev = ip.device(device_name)
+        if not ns_dev:
+            LOG.warning("Device %s is not ready in namespace %s!",
+                        device_name, namespace)
 
         internal = not self.conf.ovs_use_veth
         self._ovs_add_port(bridge, tap_name, port_id, mac_address,
                            internal=internal)
         try:
-            self._set_device_address(ns_dev, mac_address)
+            self._ensure_device_address(ns_dev, mac_address)
         except Exception:
             LOG.warning("Failed to set mac for interface %s", ns_dev)
             with excutils.save_and_reraise_exception():
@@ -429,6 +437,13 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
                 with excutils.save_and_reraise_exception():
                     ovs = ovs_lib.OVSBridge(bridge)
                     ovs.delete_port(tap_name)
+            except Exception as exc:
+                LOG.warning("Failed to plug interface %s to bridge %s in "
+                            "namespace %s due to unknown reason: %s",
+                            device_name, bridge, namespace, str(exc))
+                with excutils.save_and_reraise_exception():
+                    ovs = ovs_lib.OVSBridge(bridge)
+                    ovs.delete_port(tap_name)
 
         # NOTE(ihrachys): the order here is significant: we must set MTU after
         # the device is moved into a namespace, otherwise OVS bridge does not
