@@ -25,17 +25,17 @@ from neutron_lib import constants
 from neutron_lib import exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import excutils
 from oslo_utils import netutils
-from pyroute2.netlink import exceptions \
-    as netlink_exceptions  # pylint: disable=no-name-in-module
-from pyroute2.netlink import rtnl  # pylint: disable=no-name-in-module
-from pyroute2.netlink.rtnl import \
-    ifaddrmsg  # pylint: disable=no-name-in-module
-from pyroute2.netlink.rtnl import ifinfmsg  # pylint: disable=no-name-in-module
-from pyroute2 import netns  # pylint: disable=no-name-in-module
+from pyroute2.netlink import exceptions as netlink_exceptions
+from pyroute2.netlink import rtnl
+from pyroute2.netlink.rtnl import ifaddrmsg
+from pyroute2.netlink.rtnl import ifinfmsg
+from pyroute2 import netns
 
 from neutron._i18n import _
 from neutron.agent.common import utils
+from neutron.agent.linux import utils as linux_utils
 from neutron.common import utils as common_utils
 from neutron.privileged.agent.linux import ip_lib as privileged
 from neutron.privileged.agent.linux import utils as priv_utils
@@ -285,9 +285,9 @@ class IPWrapper(SubProcessBase):
                 return True
         return False
 
-    def add_device_to_namespace(self, device):
+    def add_device_to_namespace(self, device, is_ovs_port=False):
         if self.namespace:
-            device.link.set_netns(self.namespace)
+            device.link.set_netns(self.namespace, is_ovs_port=is_ovs_port)
 
     def add_vlan(self, name, physical_interface, vlan_id):
         privileged.create_interface(name,
@@ -477,12 +477,17 @@ class IpLinkCommand(IpDeviceCommandBase):
         privileged.set_link_attribute(
             self.name, self._parent.namespace, state='down')
 
-    def set_netns(self, namespace):
-        privileged.set_link_attribute(
-            self.name, self._parent.namespace, net_ns_fd=namespace)
-        self._parent.namespace = namespace
-        common_utils.wait_until_true(lambda: self.exists, timeout=5,
-                                     sleep=0.5)
+    def set_netns(self, namespace, is_ovs_port=False):
+        old_namespace = self._parent.namespace
+        try:
+            privileged.set_link_attribute(
+                self.name, self._parent.namespace, net_ns_fd=namespace)
+            self._parent.namespace = namespace
+            common_utils.wait_until_true(lambda: self.exists, timeout=3,
+                                         sleep=0.5)
+        except common_utils.WaitTimeout:
+            with excutils.save_and_reraise_exception():
+                self._parent.namespace = old_namespace
 
     def set_name(self, name):
         privileged.set_link_attribute(
@@ -555,9 +560,16 @@ class IpAddrCommand(IpDeviceCommandBase):
         add_ip_address(cidr, self.name, self._parent.namespace, scope,
                        add_broadcast)
 
+    def add_multiple(self, cidrs, scope='global', add_broadcast=True):
+        add_ip_addresses(cidrs, self.name, self._parent.namespace, scope,
+                         add_broadcast)
+
     def delete(self, cidr):
         delete_ip_address(cidr, self.name, self._parent.namespace)
 
+    def delete_multiple(self, cidrs):
+        delete_ip_addresses(cidrs, self.name, self._parent.namespace)
+
     def flush(self, ip_version):
         flush_ip_addresses(ip_version, self.name, self._parent.namespace)
 
@@ -836,15 +848,27 @@ def add_ip_address(cidr, device, namespace=None, scope='global',
     """
     net = netaddr.IPNetwork(cidr)
     broadcast = None
-    if add_broadcast and net.version == 4:
-        # NOTE(slaweq): in case if cidr is /32 net.broadcast is None so
-        # same IP address as cidr should be set as broadcast
-        broadcast = str(net.broadcast or net.ip)
+    if add_broadcast:
+        broadcast = common_utils.cidr_broadcast_address_alternative(cidr)
     privileged.add_ip_address(
         net.version, str(net.ip), net.prefixlen,
         device, namespace, scope, broadcast)
 
 
+def add_ip_addresses(cidrs, device, namespace=None, scope='global',
+                     add_broadcast=True):
+    """Add multiple IP addresses.
+
+    :param cidrs: A list of IP addresses to add, in CIDR notation
+    :param device: Device name to use in adding address
+    :param namespace: The name of the namespace in which to add the address
+    :param scope: scope of address being added
+    :param add_broadcast: should broadcast address be added
+    """
+    privileged.add_ip_addresses(
+        cidrs, device, namespace, scope, add_broadcast)
+
+
 def delete_ip_address(cidr, device, namespace=None):
     """Delete an IP address.
 
@@ -857,6 +881,16 @@ def delete_ip_address(cidr, device, namespace=None):
         net.version, str(net.ip), net.prefixlen, device, namespace)
 
 
+def delete_ip_addresses(cidrs, device, namespace=None):
+    """Delete multiple IP address.
+
+    :param cidrs: A list of IP addresses to delete, in CIDR notation
+    :param device: Device name to use in deleting address
+    :param namespace: The name of the namespace in which to delete the address
+    """
+    privileged.delete_ip_addresses(cidrs, device, namespace)
+
+
 def flush_ip_addresses(ip_version, device, namespace=None):
     """Flush all IP addresses.
 
@@ -1362,19 +1396,12 @@ def delete_ip_rule(namespace, ip, iif=None, table=None, priority=None,
     privileged.delete_ip_rule(namespace, **cmd_args)
 
 
-def get_attr(pyroute2_obj, attr_name):
-    """Get an attribute from a PyRoute2 object"""
-    rule_attrs = pyroute2_obj.get('attrs', [])
-    for attr in (attr for attr in rule_attrs if attr[0] == attr_name):
-        return attr[1]
-
-
 def _parse_ip_address(pyroute2_address, device_name):
-    ip = get_attr(pyroute2_address, 'IFA_ADDRESS')
+    ip = linux_utils.get_attr(pyroute2_address, 'IFA_ADDRESS')
     ip_length = pyroute2_address['prefixlen']
     event = IP_ADDRESS_EVENTS.get(pyroute2_address.get('event'))
     cidr = common_utils.ip_to_cidr(ip, prefix=ip_length)
-    flags = get_attr(pyroute2_address, 'IFA_FLAGS')
+    flags = linux_utils.get_attr(pyroute2_address, 'IFA_FLAGS')
     dynamic = not bool(flags & ifaddrmsg.IFA_F_PERMANENT)
     tentative = bool(flags & ifaddrmsg.IFA_F_TENTATIVE)
     dadfailed = bool(flags & ifaddrmsg.IFA_F_DADFAILED)
@@ -1382,7 +1409,8 @@ def _parse_ip_address(pyroute2_address, device_name):
     return {'name': device_name,
             'cidr': cidr,
             'scope': scope,
-            'broadcast': get_attr(pyroute2_address, 'IFA_BROADCAST'),
+            'broadcast': linux_utils.get_attr(pyroute2_address,
+                                              'IFA_BROADCAST'),
             'dynamic': dynamic,
             'tentative': tentative,
             'dadfailed': dadfailed,
@@ -1410,7 +1438,8 @@ def get_devices_with_ip(namespace, name=None, **kwargs):
     devices = {}  # {device index: name}
     for ip_address in ip_addresses:
         index = ip_address['index']
-        name = get_attr(ip_address, 'IFA_LABEL') or devices.get(index)
+        name = (linux_utils.get_attr(ip_address, 'IFA_LABEL') or
+                devices.get(index))
         if not name:
             device = get_devices_info(namespace, index=index)
             if not device:
@@ -1428,31 +1457,35 @@ def get_devices_info(namespace, **kwargs):
     retval = {}
     for device in devices:
         ret = {'index': device['index'],
-               'name': get_attr(device, 'IFLA_IFNAME'),
-               'operstate': get_attr(device, 'IFLA_OPERSTATE'),
-               'linkmode': get_attr(device, 'IFLA_LINKMODE'),
-               'mtu': get_attr(device, 'IFLA_MTU'),
-               'promiscuity': get_attr(device, 'IFLA_PROMISCUITY'),
-               'mac': get_attr(device, 'IFLA_ADDRESS'),
-               'broadcast': get_attr(device, 'IFLA_BROADCAST')}
-        ifla_link = get_attr(device, 'IFLA_LINK')
+               'name': linux_utils.get_attr(device, 'IFLA_IFNAME'),
+               'operstate': linux_utils.get_attr(device, 'IFLA_OPERSTATE'),
+               'linkmode': linux_utils.get_attr(device, 'IFLA_LINKMODE'),
+               'mtu': linux_utils.get_attr(device, 'IFLA_MTU'),
+               'promiscuity': linux_utils.get_attr(device, 'IFLA_PROMISCUITY'),
+               'mac': linux_utils.get_attr(device, 'IFLA_ADDRESS'),
+               'broadcast': linux_utils.get_attr(device, 'IFLA_BROADCAST')}
+        ifla_link = linux_utils.get_attr(device, 'IFLA_LINK')
         if ifla_link:
             ret['parent_index'] = ifla_link
-        ifla_linkinfo = get_attr(device, 'IFLA_LINKINFO')
+        ifla_linkinfo = linux_utils.get_attr(device, 'IFLA_LINKINFO')
         if ifla_linkinfo:
-            ret['kind'] = get_attr(ifla_linkinfo, 'IFLA_INFO_KIND')
-            ifla_data = get_attr(ifla_linkinfo, 'IFLA_INFO_DATA')
+            ret['kind'] = linux_utils.get_attr(ifla_linkinfo, 'IFLA_INFO_KIND')
+            ifla_data = linux_utils.get_attr(ifla_linkinfo, 'IFLA_INFO_DATA')
             if ret['kind'] == 'vxlan':
-                ret['vxlan_id'] = get_attr(ifla_data, 'IFLA_VXLAN_ID')
-                ret['vxlan_group'] = get_attr(ifla_data, 'IFLA_VXLAN_GROUP')
-                ret['vxlan_link_index'] = get_attr(ifla_data,
-                                                   'IFLA_VXLAN_LINK')
+                ret['vxlan_id'] = linux_utils.get_attr(ifla_data,
+                                                       'IFLA_VXLAN_ID')
+                ret['vxlan_group'] = linux_utils.get_attr(ifla_data,
+                                                          'IFLA_VXLAN_GROUP')
+                ret['vxlan_link_index'] = linux_utils.get_attr(
+                    ifla_data, 'IFLA_VXLAN_LINK')
             elif ret['kind'] == 'vlan':
-                ret['vlan_id'] = get_attr(ifla_data, 'IFLA_VLAN_ID')
+                ret['vlan_id'] = linux_utils.get_attr(ifla_data,
+                                                      'IFLA_VLAN_ID')
             elif ret['kind'] == 'bridge':
-                ret['stp'] = get_attr(ifla_data, 'IFLA_BR_STP_STATE')
-                ret['forward_delay'] = get_attr(ifla_data,
-                                                'IFLA_BR_FORWARD_DELAY')
+                ret['stp'] = linux_utils.get_attr(ifla_data,
+                                                  'IFLA_BR_STP_STATE')
+                ret['forward_delay'] = linux_utils.get_attr(
+                    ifla_data, 'IFLA_BR_FORWARD_DELAY')
         retval[device['index']] = ret
 
     for device in retval.values():
@@ -1512,8 +1545,8 @@ def ip_monitor(namespace, queue, event_stop, event_started):
         cache_devices = {}
         with privileged.get_iproute(namespace) as ip:
             for device in ip.get_links():
-                cache_devices[device['index']] = get_attr(device,
-                                                          'IFLA_IFNAME')
+                cache_devices[device['index']] = linux_utils.get_attr(
+                    device, 'IFLA_IFNAME')
         _ip = privileged.get_iproute(namespace)
         ip_updates_thread = threading.Thread(target=read_ip_updates,
                                              args=(_ip, _queue))
@@ -1563,7 +1596,7 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
     """List IP routes"""
     def get_device(index, devices):
         for device in (d for d in devices if d['index'] == index):
-            return get_attr(device, 'IFLA_IFNAME')
+            return linux_utils.get_attr(device, 'IFLA_IFNAME')
 
     def get_proto(proto_number):
         if isinstance(proto_number, int) and proto_number in rtnl.rt_proto:
@@ -1580,37 +1613,38 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
     devices = privileged.get_link_devices(namespace)
     ret = []
     for route in routes:
-        cidr = get_attr(route, 'RTA_DST')
+        cidr = linux_utils.get_attr(route, 'RTA_DST')
         if cidr:
             cidr = '%s/%s' % (cidr, route['dst_len'])
         else:
             cidr = constants.IP_ANY[ip_version]
-        table = int(get_attr(route, 'RTA_TABLE'))
-        metric = (get_attr(route, 'RTA_PRIORITY') or
+        table = int(linux_utils.get_attr(route, 'RTA_TABLE'))
+        metric = (linux_utils.get_attr(route, 'RTA_PRIORITY') or
                   IP_ROUTE_METRIC_DEFAULT[ip_version])
         proto = get_proto(route['proto'])
         value = {
             'table': IP_RULE_TABLES_NAMES.get(table, table),
-            'source_prefix': get_attr(route, 'RTA_PREFSRC'),
+            'source_prefix': linux_utils.get_attr(route, 'RTA_PREFSRC'),
             'cidr': cidr,
             'scope': IP_ADDRESS_SCOPE[int(route['scope'])],
             'metric': metric,
             'proto': proto,
         }
 
-        multipath = get_attr(route, 'RTA_MULTIPATH')
+        multipath = linux_utils.get_attr(route, 'RTA_MULTIPATH')
         if multipath:
             value['device'] = None
             mp_via = []
             for mp in multipath:
                 mp_via.append({'device': get_device(int(mp['oif']), devices),
-                               'via': get_attr(mp, 'RTA_GATEWAY'),
+                               'via': linux_utils.get_attr(mp, 'RTA_GATEWAY'),
                                'weight': int(mp['hops']) + 1})
             value['via'] = mp_via
         else:
-            value['device'] = get_device(int(get_attr(route, 'RTA_OIF')),
-                                         devices)
-            value['via'] = get_attr(route, 'RTA_GATEWAY')
+            value['device'] = get_device(
+                int(linux_utils.get_attr(route, 'RTA_OIF')),
+                devices)
+            value['via'] = linux_utils.get_attr(route, 'RTA_GATEWAY')
 
         ret.append(value)
 
