@@ -40,6 +40,7 @@ from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import exceptions
 from neutron.agent.linux.openvswitch_firewall import iptables
 from neutron.agent.linux.openvswitch_firewall import rules
+from neutron.common import utils as n_utils
 
 LOG = logging.getLogger(__name__)
 CONJ_ID_REGEX = re.compile(r"conj_id=(\d+),")
@@ -170,7 +171,7 @@ class OFPort(object):
         self.lla_address = str(netutils.get_ipv6_addr_by_EUI64(
             lib_const.IPv6_LLA_PREFIX, self.mac))
         self.ofport = ovs_port.ofport
-        self.sec_groups = list()
+        self.sec_groups = []
         self.fixed_ips = port_dict.get('fixed_ips', [])
         self.neutron_port_dict = port_dict.copy()
         self.allowed_pairs_v4 = self._get_allowed_pairs(port_dict, version=4)
@@ -306,7 +307,7 @@ class ConjIdMap(object):
                   for table in ovs_consts.OVS_FIREWALL_TABLES])
             conj_ids = CONJ_ID_REGEX.findall(" | ".join(flows_iter))
             try:
-                conj_id_max = max([int(conj_id) for conj_id in conj_ids])
+                conj_id_max = max(int(conj_id) for conj_id in conj_ids)
             except ValueError:
                 conj_id_max = 0
 
@@ -423,7 +424,7 @@ class ConjIPFlowManager(object):
         # NOTE(hangyang): Handle add/delete overlapped IPs among
         # remote security groups and remote address groups
         removed_ips = set([str(netaddr.IPNetwork(addr[0]).cidr) for addr in (
-                set(flow_state.keys()) - set(addr_to_conj.keys()))])
+            set(flow_state.keys()) - set(addr_to_conj.keys()))])
         ip_to_conj = collections.defaultdict(set)
         for addr, conj_ids in addr_to_conj.items():
             # Addresses from remote security groups have mac addresses,
@@ -543,7 +544,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                                    applied
 
         """
-        self.permitted_ethertypes = cfg.CONF.SECURITYGROUP.permitted_ethertypes
+        self.permitted_ethertypes = n_utils.parse_permitted_ethertypes(
+            cfg.CONF.SECURITYGROUP.permitted_ethertypes)
         self.int_br = self.initialize_bridge(integration_bridge)
         self._initialize_sg()
         self._update_cookie = None
@@ -644,6 +646,14 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                     'resubmit(,%d)' % ovs_consts.BASE_EGRESS_TABLE,
         )
 
+        if cfg.CONF.AGENT.explicitly_egress_direct:
+            self._add_flow(
+                table=ovs_consts.TRANSIENT_TABLE,
+                priority=2,
+                actions='resubmit(,%d)' % (
+                    ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
+            )
+
     def _initialize_third_party_tables(self):
         self.int_br.br.add_flow(
             table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
@@ -695,6 +705,22 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         port_id = port['device']
         return self.sg_port_map.ports.get(port_id)
 
+    def _create_of_port(self, port, ovs_port):
+        # Should always try to get the local vlan tag from
+        # the OVSDB Port other_config, since the ovs-agent's
+        # LocalVlanManager always allocated/updated it and then
+        # set_db_attribute to Port other_config before this.
+        port_vlan_id = self._get_port_vlan_tag(ovs_port.port_name)
+        segment_id = self._get_port_segmentation_id(
+            ovs_port.port_name)
+        network_type = self._get_port_network_type(
+            ovs_port.port_name)
+        physical_network = self._get_port_physical_network(
+            ovs_port.port_name)
+        return OFPort(port, ovs_port, port_vlan_id,
+                      segment_id,
+                      network_type, physical_network)
+
     def get_or_create_ofport(self, port):
         """Get ofport specified by port['device'], checking and reflecting
         ofport changes.
@@ -705,22 +731,12 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         try:
             of_port = self.sg_port_map.ports[port_id]
         except KeyError:
-            port_vlan_id = self._get_port_vlan_tag(ovs_port.port_name)
-            segment_id = self._get_port_segmentation_id(
-                ovs_port.port_name)
-            network_type = self._get_port_network_type(
-                ovs_port.port_name)
-            physical_network = self._get_port_physical_network(
-                ovs_port.port_name)
-            of_port = OFPort(port, ovs_port, port_vlan_id,
-                             segment_id,
-                             network_type, physical_network)
+            of_port = self._create_of_port(port, ovs_port)
             self.sg_port_map.create_port(of_port, port)
         else:
             if of_port.ofport != ovs_port.ofport:
                 self.sg_port_map.remove_port(of_port)
-                of_port = OFPort(port, ovs_port, of_port.vlan_tag,
-                                 of_port.segment_id)
+                of_port = self._create_of_port(port, ovs_port)
                 self.sg_port_map.create_port(of_port, port)
             else:
                 self.sg_port_map.update_port(of_port, port)
@@ -897,7 +913,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                        ovsfw_consts.REG_NET)
         if network_type == lib_const.TYPE_VLAN:
             actions += 'strip_vlan,resubmit(,{:d})'.format(
-                    ovs_consts.BASE_INGRESS_TABLE)
+                ovs_consts.BASE_INGRESS_TABLE)
             self._add_flow(
                 flow_group_id=ofport,
                 table=ovs_consts.TRANSIENT_TABLE,
@@ -909,7 +925,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             # If the port belong to flat network, we need match vlan_tci and
             # needn't pop vlan
             actions += 'resubmit(,{:d})'.format(
-                    ovs_consts.BASE_INGRESS_TABLE)
+                ovs_consts.BASE_INGRESS_TABLE)
             self._add_flow(
                 flow_group_id=ofport,
                 table=ovs_consts.TRANSIENT_TABLE,
@@ -1178,24 +1194,14 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
         # Allow custom ethertypes
         for permitted_ethertype in self.permitted_ethertypes:
-            if permitted_ethertype[:2] == '0x':
-                try:
-                    hex_ethertype = hex(int(permitted_ethertype, base=16))
-                    action = ('resubmit(,%d)' %
-                        ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
-                    self._add_flow(
-                        table=ovs_consts.BASE_EGRESS_TABLE,
-                        priority=95,
-                        dl_type=hex_ethertype,
-                        reg_port=port.ofport,
-                        actions=action
-                    )
-                    continue
-                except ValueError:
-                    pass
-            LOG.warning("Custom ethertype %(permitted_ethertype)s is not "
-                        "a hexadecimal number.",
-                        {'permitted_ethertype': permitted_ethertype})
+            action = ('resubmit(,%d)' %
+                      ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
+            self._add_flow(
+                table=ovs_consts.BASE_EGRESS_TABLE,
+                priority=95,
+                dl_type=permitted_ethertype,
+                reg_port=port.ofport,
+                actions=action)
 
         # Drop all remaining egress connections
         self._add_flow(
@@ -1253,6 +1259,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             return
 
         # Prevent flood for accepted egress traffic
+        # For packets from internal ports or VM ports.
         self._add_flow(
             flow_group_id=dst_port,
             table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
@@ -1261,6 +1268,15 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             reg_net=vlan_tag,
             actions='output:{:d}'.format(dst_port)
         )
+        # For packets from patch ports.
+        self._add_flow(
+            flow_group_id=dst_port,
+            table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
+            priority=12,
+            dl_dst=mac,
+            dl_vlan=vlan_tag,
+            actions='strip_vlan,output:{:d}'.format(dst_port)
+        )
 
         # The former flow may not match, that means the destination port is
         # not in this host. So, we direct the packet to mapped bridge(s).
@@ -1309,6 +1325,12 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             dl_src=mac,
             reg_net=vlan_tag)
 
+        self._delete_flows(
+            table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
+            dl_dst=mac,
+            dl_vlan=vlan_tag
+        )
+
     def _initialize_tracked_egress(self, port):
         # Drop invalid packets
         self._add_flow(
@@ -1328,8 +1350,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         )
 
         for state in (
-            ovsfw_consts.OF_STATE_ESTABLISHED_REPLY,
-            ovsfw_consts.OF_STATE_RELATED,
+                ovsfw_consts.OF_STATE_ESTABLISHED_REPLY,
+                ovsfw_consts.OF_STATE_RELATED,
         ):
             self._add_flow(
                 table=ovs_consts.RULES_EGRESS_TABLE,
@@ -1388,6 +1410,15 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             actions='output:{:d}'.format(port.ofport)
         )
 
+        # Allow custom ethertypes
+        for permitted_ethertype in self.permitted_ethertypes:
+            self._add_flow(
+                table=ovs_consts.BASE_INGRESS_TABLE,
+                priority=100,
+                dl_type=permitted_ethertype,
+                reg_port=port.ofport,
+                actions='output:{:d}'.format(port.ofport))
+
         self._initialize_ingress_ipv6_icmp(port)
 
         # DHCP offers
