@@ -16,24 +16,23 @@
 import collections
 import copy
 
-import netaddr
+from neutron_lib.api import attributes
+from neutron_lib.api import faults
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib import constants
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions
+from neutron_lib import rpc as n_rpc
+from neutron_lib.services import constants as service_const
 from oslo_log import log as logging
 from oslo_policy import policy as oslo_policy
 from oslo_utils import excutils
-import six
 import webob.exc
 
-from neutron._i18n import _, _LE, _LI
+from neutron._i18n import _
 from neutron.api import api_common
-from neutron.api.v2 import attributes
 from neutron.api.v2 import resource as wsgi_resource
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.common import constants as n_const
-from neutron.common import exceptions as n_exc
-from neutron.common import rpc as n_rpc
-from neutron.db import api as db_api
 from neutron import policy
 from neutron import quota
 from neutron.quota import resource_registry
@@ -41,16 +40,6 @@ from neutron.quota import resource_registry
 
 LOG = logging.getLogger(__name__)
 
-FAULT_MAP = {exceptions.NotFound: webob.exc.HTTPNotFound,
-             exceptions.Conflict: webob.exc.HTTPConflict,
-             exceptions.InUse: webob.exc.HTTPConflict,
-             exceptions.BadRequest: webob.exc.HTTPBadRequest,
-             exceptions.ServiceUnavailable: webob.exc.HTTPServiceUnavailable,
-             exceptions.NotAuthorized: webob.exc.HTTPForbidden,
-             netaddr.AddrFormatError: webob.exc.HTTPBadRequest,
-             oslo_policy.PolicyNotAuthorized: webob.exc.HTTPForbidden
-             }
-
 
 class Controller(object):
     LIST = 'list'
@@ -75,6 +64,14 @@ class Controller(object):
     def member_actions(self):
         return self._member_actions
 
+    @property
+    def allow_pagination(self):
+        return self._allow_pagination
+
+    @property
+    def allow_sorting(self):
+        return self._allow_sorting
+
     def _init_policy_attrs(self):
         """Create the list of attributes required by policy.
 
@@ -107,6 +104,7 @@ class Controller(object):
         self._native_bulk = self._is_native_bulk_supported()
         self._native_pagination = self._is_native_pagination_supported()
         self._native_sorting = self._is_native_sorting_supported()
+        self._filter_validation = self._is_filter_validation_supported()
         self._policy_attrs = self._init_policy_attrs()
         self._notifier = n_rpc.get_notifier('network')
         self._member_actions = member_actions
@@ -118,8 +116,8 @@ class Controller(object):
                     _("Native pagination depend on native sorting")
                 )
             if not self._allow_sorting:
-                LOG.info(_LI("Allow sorting is enabled because native "
-                             "pagination requires native sorting"))
+                LOG.info("Allow sorting is enabled because native "
+                         "pagination requires native sorting")
                 self._allow_sorting = True
         self.parent = parent
         if parent:
@@ -137,7 +135,7 @@ class Controller(object):
                                                          self._resource)
 
     def _get_primary_key(self, default_primary_key='id'):
-        for key, value in six.iteritems(self._attr_info):
+        for key, value in self._attr_info.items():
             if value.get('primary_key', False):
                 return key
         return default_primary_key
@@ -153,6 +151,9 @@ class Controller(object):
     def _is_native_sorting_supported(self):
         return api_common.is_native_sorting_supported(self._plugin)
 
+    def _is_filter_validation_supported(self):
+        return api_common.is_filter_validation_supported(self._plugin)
+
     def _exclude_attributes_by_policy(self, context, data):
         """Identifies attributes to exclude according to authZ policies.
 
@@ -172,11 +173,12 @@ class Controller(object):
             attr_data = self._attr_info.get(attr_name)
             if attr_data and attr_data['is_visible']:
                 if policy.check(
-                    context,
-                    '%s:%s' % (self._plugin_handlers[self.SHOW], attr_name),
-                    data,
-                    might_not_exist=True,
-                    pluralized=self._collection):
+                        context,
+                        '%s:%s' % (self._plugin_handlers[self.SHOW],
+                                   attr_name),
+                        data,
+                        might_not_exist=True,
+                        pluralized=self._collection):
                     # this attribute is visible, check next one
                     continue
             # if the code reaches this point then either the policy check
@@ -202,12 +204,12 @@ class Controller(object):
         """
         fields_to_strip = ((fields_to_strip or []) +
                            self._exclude_attributes_by_policy(context, data))
-        return self._filter_attributes(context, data, fields_to_strip)
+        return self._filter_attributes(data, fields_to_strip)
 
-    def _filter_attributes(self, context, data, fields_to_strip=None):
+    def _filter_attributes(self, data, fields_to_strip=None):
         if not fields_to_strip:
             return data
-        return dict(item for item in six.iteritems(data)
+        return dict(item for item in data.items()
                     if (item[0] not in fields_to_strip))
 
     def _do_field_list(self, original_fields):
@@ -224,8 +226,6 @@ class Controller(object):
             @db_api.retry_db_errors
             def _handle_action(request, id, **kwargs):
                 arg_list = [request.context, id]
-                # Ensure policy engine is initialized
-                policy.init()
                 # Fetch the resource and verify if the user can access it
                 try:
                     parent_id = kwargs.get(self._parent_id_name)
@@ -234,7 +234,8 @@ class Controller(object):
                                           do_authz=True,
                                           field_list=None,
                                           parent_id=parent_id)
-                except oslo_policy.PolicyNotAuthorized:
+                except (oslo_policy.PolicyNotAuthorized,
+                        oslo_policy.InvalidScope):
                     msg = _('The resource could not be found.')
                     raise webob.exc.HTTPNotFound(msg)
                 body = kwargs.pop('body', None)
@@ -283,9 +284,11 @@ class Controller(object):
         # plugin before returning.
         original_fields, fields_to_add = self._do_field_list(
             api_common.list_args(request, 'fields'))
-        filters = api_common.get_filters(request, self._attr_info,
-                                         ['fields', 'sort_key', 'sort_dir',
-                                          'limit', 'marker', 'page_reverse'])
+        filters = api_common.get_filters(
+            request, self._attr_info,
+            ['fields', 'sort_key', 'sort_dir',
+             'limit', 'marker', 'page_reverse'],
+            is_filter_validation_supported=self._filter_validation)
         kwargs = {'filters': filters,
                   'fields': original_fields}
         sorting_helper = self._get_sorting_helper(request)
@@ -305,12 +308,15 @@ class Controller(object):
             # FIXME(salvatore-orlando): obj_getter might return references to
             # other resources. Must check authZ on them too.
             # Omit items from list that should not be visible
-            obj_list = [obj for obj in obj_list
-                        if policy.check(request.context,
-                                        self._plugin_handlers[self.SHOW],
-                                        obj,
-                                        plugin=self._plugin,
-                                        pluralized=self._collection)]
+            tmp_list = []
+            for obj in obj_list:
+                self._set_parent_id_into_ext_resources_request(
+                    request, obj, parent_id, is_get=True)
+                if policy.check(
+                        request.context, self._plugin_handlers[self.SHOW],
+                        obj, plugin=self._plugin, pluralized=self._collection):
+                    tmp_list.append(obj)
+            obj_list = tmp_list
         # Use the first element in the list for discriminating which attributes
         # should be filtered out because of authZ policies
         # fields_to_add contains a list of attributes added for request policy
@@ -322,8 +328,7 @@ class Controller(object):
                 request.context, obj_list[0])
         collection = {self._collection:
                       [self._filter_attributes(
-                          request.context, obj,
-                          fields_to_strip=fields_to_strip)
+                          obj, fields_to_strip=fields_to_strip)
                        for obj in obj_list]}
         pagination_links = pagination_helper.get_links(obj_list)
         if pagination_links:
@@ -342,6 +347,8 @@ class Controller(object):
             kwargs[self._parent_id_name] = parent_id
         obj_getter = getattr(self._plugin, action)
         obj = obj_getter(request.context, id, **kwargs)
+        self._set_parent_id_into_ext_resources_request(
+            request, obj, parent_id, is_get=True)
         # Check authz
         # FIXME(salvatore-orlando): obj_getter might return references to
         # other resources. Must check authZ on them too.
@@ -356,8 +363,6 @@ class Controller(object):
     def index(self, request, **kwargs):
         """Returns a list of the requested entity."""
         parent_id = kwargs.get(self._parent_id_name)
-        # Ensure policy engine is initialized
-        policy.init()
         return self._items(request, True, parent_id)
 
     @db_api.retry_db_errors
@@ -370,8 +375,6 @@ class Controller(object):
             field_list, added_fields = self._do_field_list(
                 api_common.list_args(request, "fields"))
             parent_id = kwargs.get(self._parent_id_name)
-            # Ensure policy engine is initialized
-            policy.init()
             return {self._resource:
                     self._view(request.context,
                                self._item(request,
@@ -380,7 +383,7 @@ class Controller(object):
                                           field_list=field_list,
                                           parent_id=parent_id),
                                fields_to_strip=added_fields)}
-        except oslo_policy.PolicyNotAuthorized:
+        except (oslo_policy.PolicyNotAuthorized, oslo_policy.InvalidScope):
             # To avoid giving away information, pretend that it
             # doesn't exist
             msg = _('The resource could not be found.')
@@ -396,7 +399,6 @@ class Controller(object):
                 fields_to_strip = self._exclude_attributes_by_policy(
                     request.context, item)
                 objs.append(self._filter_attributes(
-                    request.context,
                     obj_creator(request.context, **kwargs),
                     fields_to_strip=fields_to_strip))
             return objs
@@ -414,8 +416,8 @@ class Controller(object):
                     except Exception:
                         # broad catch as our only purpose is to log the
                         # exception
-                        LOG.exception(_LE("Unable to undo add for "
-                                          "%(resource)s %(id)s"),
+                        LOG.exception("Unable to undo add for "
+                                      "%(resource)s %(id)s",
                                       {'resource': self._resource,
                                        'id': obj['id']})
                 # TODO(salvatore-orlando): The object being processed when the
@@ -433,10 +435,15 @@ class Controller(object):
     def _create(self, request, body, **kwargs):
         """Creates a new instance of the requested entity."""
         parent_id = kwargs.get(self._parent_id_name)
-        body = Controller.prepare_request_body(request.context,
-                                               body, True,
-                                               self._resource, self._attr_info,
-                                               allow_bulk=self._allow_bulk)
+        try:
+            body = Controller.prepare_request_body(
+                request.context, body, True, self._resource, self._attr_info,
+                allow_bulk=self._allow_bulk)
+        except Exception as e:
+            LOG.warning("An exception happened while processing the request "
+                        "body. The exception message is [%s].", e)
+            raise e
+
         action = self._plugin_handlers[self.CREATE]
         # Check authz
         if self._collection in body:
@@ -444,8 +451,6 @@ class Controller(object):
             items = body[self._collection]
         else:
             items = [body]
-        # Ensure policy engine is initialized
-        policy.init()
         # Store requested resource amounts grouping them by tenant
         # This won't work with multiple resources. However because of the
         # current structure of this controller there will hardly be more than
@@ -454,6 +459,11 @@ class Controller(object):
         for item in items:
             self._validate_network_tenant_ownership(request,
                                                     item[self._resource])
+            # For ext resources policy check, we support two types, such as
+            # parent_id is in request body, another type is parent_id is in
+            # request url, which we can get from kwargs.
+            self._set_parent_id_into_ext_resources_request(
+                request, item[self._resource], parent_id)
             policy.enforce(request.context,
                            action,
                            item[self._resource],
@@ -472,30 +482,31 @@ class Controller(object):
                     tenant,
                     {self._resource: delta},
                     self._plugin)
-                reservations.append(reservation)
-        except n_exc.QuotaResourceUnknown as e:
-                # We don't want to quota this resource
-                LOG.debug(e)
+                if reservation:
+                    reservations.append(reservation)
+        except exceptions.QuotaResourceUnknown as e:
+            # We don't want to quota this resource
+            LOG.debug(e)
 
         def notify(create_result):
             # Ensure usage trackers for all resources affected by this API
             # operation are marked as dirty
-            with request.context.session.begin():
-                # Commit the reservation(s)
-                for reservation in reservations:
-                    quota.QUOTAS.commit_reservation(
-                        request.context, reservation.reservation_id)
-                resource_registry.set_resources_dirty(request.context)
+            # Commit the reservation(s)
+            for reservation in reservations:
+                quota.QUOTAS.commit_reservation(
+                    request.context, reservation.reservation_id)
+            resource_registry.set_resources_dirty(request.context)
 
             notifier_method = self._resource + '.create.end'
             self._notifier.info(request.context,
                                 notifier_method,
                                 create_result)
-            registry.notify(self._resource, events.BEFORE_RESPONSE, self,
-                            context=request.context, data=create_result,
-                            method_name=notifier_method,
-                            collection=self._collection,
-                            action=action, original={})
+            registry.publish(self._resource, events.BEFORE_RESPONSE, self,
+                             payload=events.APIEventPayload(
+                                 request.context, notifier_method, action,
+                                 request_body=body,
+                                 states=({}, create_result,),
+                                 collection_name=self._collection))
             return create_result
 
         def do_create(body, bulk=False, emulated=False):
@@ -531,9 +542,10 @@ class Controller(object):
             # should be removed because of authZ policies
             fields_to_strip = self._exclude_attributes_by_policy(
                 request.context, objs[0])
-            return notify({self._collection: [self._filter_attributes(
-                request.context, obj, fields_to_strip=fields_to_strip)
-                for obj in objs]})
+            return notify({self._collection:
+                           [self._filter_attributes(
+                               obj, fields_to_strip=fields_to_strip)
+                            for obj in objs]})
         else:
             if self._collection in body:
                 # Emulate atomic bulk behavior
@@ -559,7 +571,6 @@ class Controller(object):
         action = self._plugin_handlers[self.DELETE]
 
         # Check authz
-        policy.init()
         parent_id = kwargs.get(self._parent_id_name)
         obj = self._item(request, id, parent_id=parent_id)
         try:
@@ -567,9 +578,15 @@ class Controller(object):
                            action,
                            obj,
                            pluralized=self._collection)
-        except oslo_policy.PolicyNotAuthorized:
+        except (oslo_policy.PolicyNotAuthorized, oslo_policy.InvalidScope):
             # To avoid giving away information, pretend that it
-            # doesn't exist
+            # doesn't exist if policy does not authorize SHOW
+            with excutils.save_and_reraise_exception() as ctxt:
+                if not policy.check(request.context,
+                                    self._plugin_handlers[self.SHOW],
+                                    obj,
+                                    pluralized=self._collection):
+                    ctxt.reraise = False
             msg = _('The resource could not be found.')
             raise webob.exc.HTTPNotFound(msg)
 
@@ -585,10 +602,12 @@ class Controller(object):
         self._notifier.info(request.context,
                             notifier_method,
                             notifier_payload)
-        registry.notify(self._resource, events.BEFORE_RESPONSE, self,
-                        context=request.context, data=result,
-                        method_name=notifier_method, action=action,
-                        original={})
+
+        registry.publish(self._resource, events.BEFORE_RESPONSE, self,
+                         payload=events.APIEventPayload(
+                             request.context, notifier_method, action,
+                             states=({}, obj, result,),
+                             collection_name=self._collection))
 
     def update(self, request, id, body=None, **kwargs):
         """Updates the specified entity's attributes."""
@@ -605,21 +624,28 @@ class Controller(object):
 
     @db_api.retry_db_errors
     def _update(self, request, id, body, **kwargs):
-        body = Controller.prepare_request_body(request.context,
-                                               body, False,
-                                               self._resource, self._attr_info,
-                                               allow_bulk=self._allow_bulk)
+        try:
+            body = Controller.prepare_request_body(
+                request.context, body, False, self._resource, self._attr_info,
+                allow_bulk=self._allow_bulk)
+        except Exception as e:
+            LOG.warning("An exception happened while processing the request "
+                        "body. The exception message is [%s].", e)
+            raise e
+
         action = self._plugin_handlers[self.UPDATE]
         # Load object to check authz
         # but pass only attributes in the original body and required
         # by the policy engine to the policy 'brain'
-        field_list = [name for (name, value) in six.iteritems(self._attr_info)
+        field_list = [name for (name, value) in self._attr_info.items()
                       if (value.get('required_by_policy') or
                           value.get('primary_key') or
                           'default' not in value)]
-        # Ensure policy engine is initialized
-        policy.init()
         parent_id = kwargs.get(self._parent_id_name)
+        # If the parent_id exist, we should get orig_obj with
+        # self._parent_id_name field.
+        if parent_id and self._parent_id_name not in field_list:
+            field_list.append(self._parent_id_name)
         orig_obj = self._item(request, id, field_list=field_list,
                               parent_id=parent_id)
         orig_object_copy = copy.copy(orig_obj)
@@ -627,25 +653,33 @@ class Controller(object):
         # Make a list of attributes to be updated to inform the policy engine
         # which attributes are set explicitly so that it can distinguish them
         # from the ones that are set to their default values.
-        orig_obj[n_const.ATTRIBUTES_TO_UPDATE] = body[self._resource].keys()
+        orig_obj[constants.ATTRIBUTES_TO_UPDATE] = body[self._resource].keys()
+        # Then get the ext_parent_id, format to ext_parent_parent_resource_id
+        if self._parent_id_name in orig_obj:
+            self._set_parent_id_into_ext_resources_request(
+                request, orig_obj, parent_id)
         try:
             policy.enforce(request.context,
                            action,
                            orig_obj,
                            pluralized=self._collection)
-        except oslo_policy.PolicyNotAuthorized:
+        except (oslo_policy.PolicyNotAuthorized, oslo_policy.InvalidScope):
+            # To avoid giving away information, pretend that it
+            # doesn't exist if policy does not authorize SHOW
             with excutils.save_and_reraise_exception() as ctxt:
-                # If a tenant is modifying its own object, it's safe to return
-                # a 403. Otherwise, pretend that it doesn't exist to avoid
-                # giving away information.
-                orig_obj_tenant_id = orig_obj.get("tenant_id")
-                if (request.context.tenant_id != orig_obj_tenant_id or
-                    orig_obj_tenant_id is None):
+                if not policy.check(request.context,
+                                    self._plugin_handlers[self.SHOW],
+                                    orig_obj,
+                                    pluralized=self._collection):
                     ctxt.reraise = False
             msg = _('The resource could not be found.')
             raise webob.exc.HTTPNotFound(msg)
 
-        obj_updater = getattr(self._plugin, action)
+        if self._native_bulk and hasattr(self._plugin, "%s_bulk" % action):
+            obj_updater = getattr(self._plugin, "%s_bulk" % action)
+        else:
+            obj_updater = getattr(self._plugin, action)
+
         kwargs = {self._resource: body}
         if parent_id:
             kwargs[self._parent_id_name] = parent_id
@@ -659,10 +693,12 @@ class Controller(object):
         result = {self._resource: self._view(request.context, obj)}
         notifier_method = self._resource + '.update.end'
         self._notifier.info(request.context, notifier_method, result)
-        registry.notify(self._resource, events.BEFORE_RESPONSE, self,
-                        context=request.context, data=result,
-                        method_name=notifier_method, action=action,
-                        original=orig_object_copy)
+        registry.publish(self._resource, events.BEFORE_RESPONSE, self,
+                         payload=events.APIEventPayload(
+                             request.context, notifier_method, action,
+                             request_body=body,
+                             states=(orig_object_copy, result,),
+                             collection_name=self._collection))
         return result
 
     @staticmethod
@@ -689,13 +725,20 @@ class Controller(object):
                                                      "not supported"))
                 if not body[collection]:
                     raise webob.exc.HTTPBadRequest(_("Resources required"))
-                bulk_body = [
-                    Controller.prepare_request_body(
-                        context, item if resource in item
-                        else {resource: item}, is_create, resource, attr_info,
-                        allow_bulk) for item in body[collection]
-                ]
-                return {collection: bulk_body}
+                try:
+                    bulk_body = [
+                        Controller.prepare_request_body(
+                            context, item if resource in item
+                            else {resource: item}, is_create, resource,
+                            attr_info, allow_bulk) for item in body[collection]
+                    ]
+                    return {collection: bulk_body}
+                except Exception as e:
+                    LOG.warning(
+                        "An exception happened while processing the request "
+                        "body. The exception message is [%s].", e)
+                    raise e
+
             res_dict = body.get(resource)
         except (AttributeError, TypeError):
             msg = _("Body contains invalid data")
@@ -703,20 +746,25 @@ class Controller(object):
         if res_dict is None:
             msg = _("Unable to find '%s' in request body") % resource
             raise webob.exc.HTTPBadRequest(msg)
+        if not isinstance(res_dict, dict):
+            msg = _("Object '%s' contains invalid data") % resource
+            raise webob.exc.HTTPBadRequest(msg)
 
-        attributes.populate_tenant_id(context, res_dict, attr_info, is_create)
-        attributes.verify_attributes(res_dict, attr_info)
+        attr_ops = attributes.AttributeInfo(attr_info)
+        attr_ops.populate_project_id(context, res_dict, is_create)
+        attributes.populate_project_info(attr_info)
+        attr_ops.verify_attributes(res_dict)
 
         if is_create:  # POST
-            attributes.fill_default_value(attr_info, res_dict,
-                                          webob.exc.HTTPBadRequest)
+            attr_ops.fill_post_defaults(
+                res_dict, exc_cls=webob.exc.HTTPBadRequest)
         else:  # PUT
-            for attr, attr_vals in six.iteritems(attr_info):
+            for attr, attr_vals in attr_info.items():
                 if attr in res_dict and not attr_vals['allow_put']:
                     msg = _("Cannot update read-only attribute %s") % attr
                     raise webob.exc.HTTPBadRequest(msg)
 
-        attributes.convert_value(attr_info, res_dict, webob.exc.HTTPBadRequest)
+        attr_ops.convert_values(res_dict, exc_cls=webob.exc.HTTPBadRequest)
         return body
 
     def _validate_network_tenant_ownership(self, request, resource_item):
@@ -740,6 +788,37 @@ class Controller(object):
             msg = _('The resource could not be found.')
             raise webob.exc.HTTPNotFound(msg)
 
+    def _set_parent_id_into_ext_resources_request(
+            self, request, resource_item, parent_id, is_get=False):
+        if not parent_id:
+            return
+
+        # This will pass most create/update/delete cases
+        if not is_get and (request.context.is_admin or
+                           request.context.is_advsvc or
+                           self.parent['member_name'] not in
+                           service_const.EXT_PARENT_RESOURCE_MAPPING or
+                           resource_item.get(self._parent_id_name)):
+            return
+
+        # Then we arrive here, that means the request or get obj contains
+        # ext_parent. If this func is called by list/get, and it contains
+        # _parent_id_name. We need to re-add the ex_parent prefix to policy.
+        if is_get:
+            if (not request.context.is_admin or
+                    not request.context.is_advsvc and
+                    self.parent['member_name'] in
+                    service_const.EXT_PARENT_RESOURCE_MAPPING):
+                resource_item.setdefault(
+                    "%s_%s" % (constants.EXT_PARENT_PREFIX,
+                               self._parent_id_name),
+                    parent_id)
+        # If this func is called by create/update/delete, we just add.
+        else:
+            resource_item.setdefault(
+                "%s_%s" % (constants.EXT_PARENT_PREFIX, self._parent_id_name),
+                parent_id)
+
 
 def create_resource(collection, resource, plugin, params, allow_bulk=False,
                     member_actions=None, parent=None, allow_pagination=False,
@@ -749,4 +828,4 @@ def create_resource(collection, resource, plugin, params, allow_bulk=False,
                             allow_pagination=allow_pagination,
                             allow_sorting=allow_sorting)
 
-    return wsgi_resource.Resource(controller, FAULT_MAP)
+    return wsgi_resource.Resource(controller, faults.FAULT_MAP)
