@@ -17,10 +17,15 @@ from neutron_lib.api.definitions import agent as agent_def
 from neutron_lib.api.definitions import allowedaddresspairs
 from neutron_lib.api.definitions import auto_allocated_topology
 from neutron_lib.api.definitions import availability_zone as az_def
+from neutron_lib.api.definitions import bgp
+from neutron_lib.api.definitions import bgp_4byte_asn
+from neutron_lib.api.definitions import bgp_dragentscheduler
 from neutron_lib.api.definitions import default_subnetpools
+from neutron_lib.api.definitions import dhcpagentscheduler
 from neutron_lib.api.definitions import dns
 from neutron_lib.api.definitions import dns_domain_keywords
 from neutron_lib.api.definitions import dns_domain_ports
+from neutron_lib.api.definitions import empty_string_filtering
 from neutron_lib.api.definitions import expose_port_forwarding_in_fip
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import extra_dhcp_opt
@@ -37,6 +42,7 @@ from neutron_lib.api.definitions import l3_ext_gw_mode
 from neutron_lib.api.definitions import logging
 from neutron_lib.api.definitions import multiprovidernet
 from neutron_lib.api.definitions import network_availability_zone
+from neutron_lib.api.definitions import network_ha
 from neutron_lib.api.definitions import network_ip_availability
 from neutron_lib.api.definitions import network_mtu
 from neutron_lib.api.definitions import network_mtu_writable
@@ -69,13 +75,16 @@ from neutron_lib.api.definitions import sorting
 from neutron_lib.api.definitions import stateful_security_group
 from neutron_lib.api.definitions import subnet_dns_publish_fixed_ip
 from neutron_lib.api.definitions import subnet_service_types
+from neutron_lib.api.definitions import subnetpool_prefix_ops
 from neutron_lib.api.definitions import trunk
+from neutron_lib.api.definitions import uplink_status_propagation
 from neutron_lib.api.definitions import vlantransparent
 from neutron_lib.api.definitions import vpn
 from neutron_lib.api.definitions import vpn_endpoint_groups
 from neutron_lib import constants
 
 from neutron.extensions import quotasv2_detail
+from neutron.extensions import security_groups_default_rules
 
 # NOTE(russellb) This remains in its own file (vs constants.py) because we want
 # to be able to easily import it and export the info without any dependencies
@@ -113,11 +122,14 @@ ML2_SUPPORTED_API_EXTENSIONS = [
     portbindings.ALIAS,
     pbe_ext.ALIAS,
     default_subnetpools.ALIAS,
+    dhcpagentscheduler.ALIAS,
     dns.ALIAS,
+    empty_string_filtering.ALIAS,
     external_net.ALIAS,
     extra_dhcp_opt.ALIAS,
     filter_validation.ALIAS,
     multiprovidernet.ALIAS,
+    network_ha.ALIAS,
     network_mtu.ALIAS,
     network_mtu_writable.ALIAS,
     network_availability_zone.ALIAS,
@@ -142,6 +154,7 @@ ML2_SUPPORTED_API_EXTENSIONS = [
     rbac_security_groups.ALIAS,
     'standard-attr-revisions',
     'security-group',
+    security_groups_default_rules.ALIAS,
     security_groups_normalized_cidr.ALIAS,
     security_groups_remote_address_group.ALIAS,
     security_groups_shared_filtering.ALIAS,
@@ -150,6 +163,7 @@ ML2_SUPPORTED_API_EXTENSIONS = [
     constants.SUBNET_ALLOCATION_EXT_ALIAS,
     'standard-attr-tag',
     'standard-attr-timestamp',
+    subnetpool_prefix_ops.ALIAS,
     subnet_service_types.ALIAS,
     trunk.ALIAS,
     seg_def.ALIAS,
@@ -161,4 +175,8 @@ ML2_SUPPORTED_API_EXTENSIONS = [
     logging.ALIAS,
     vpn.ALIAS,
     vpn_endpoint_groups.ALIAS,
+    bgp.ALIAS,
+    bgp_4byte_asn.ALIAS,
+    bgp_dragentscheduler.ALIAS,
+    uplink_status_propagation.ALIAS,
 ]
