@@ -10,29 +10,10 @@
 #  License for the specific language governing permissions and limitations
 #  under the License.
 
+from neutron_lib import policy as neutron_policy
 from oslo_policy import policy
 
 
-def policy_and(*args):
-    return ' and '.join(args)
-
-
-def policy_or(*args):
-    return ' or '.join(args)
-
-
-# TODO(amotoki): Define these in neutron-lib once what constants are required
-# from stadium and 3rd party projects.
-# As of now, the following are candidates.
-RULE_ADMIN_OR_OWNER = 'rule:admin_or_owner'
-RULE_ADMIN_ONLY = 'rule:admin_only'
-RULE_ANY = 'rule:regular_user'
-RULE_ADVSVC = 'rule:context_is_advsvc'
-RULE_ADMIN_OR_NET_OWNER = 'rule:admin_or_network_owner'
-RULE_ADMIN_OR_NET_OWNER_OR_ADVSVC = policy_or(RULE_ADMIN_OR_NET_OWNER,
-                                              RULE_ADVSVC)
-RULE_ADMIN_OR_PARENT_OWNER = 'rule:admin_or_ext_parent_owner'
-
 # For completion of the phase 1
 # https://governance.openstack.org/tc/goals/selected/consistent-and-secure-rbac.html#phase-1
 # there is now ADMIN role
@@ -75,6 +56,32 @@ ADMIN_OR_PARENT_OWNER_MEMBER = (
 ADMIN_OR_PARENT_OWNER_READER = (
     '(' + ADMIN + ') or (' + PARENT_OWNER_READER + ')')
 
+# Those rules related to the network owner are very similar (almost the same)
+# as parent owner defined above. The only reason why they are kept here is that
+# in case of some resources like ports or subnets neutron have got policies
+# related to the "network owner" and network isn't really parent of the subnet
+# or port. Because of that, using parent owner in those cases may be
+# missleading for users so it's better to keep also "network owner" rules.
+NET_OWNER_MEMBER = 'role:member and ' + RULE_NET_OWNER
+NET_OWNER_READER = 'role:reader and ' + RULE_NET_OWNER
+ADMIN_OR_NET_OWNER_MEMBER = (
+    '(' + ADMIN + ') or (' + NET_OWNER_MEMBER + ')')
+ADMIN_OR_NET_OWNER_READER = (
+    '(' + ADMIN + ') or (' + NET_OWNER_READER + ')')
+
+# Those rules for the SG owner are needed for the policies related to the
+# Security Group rules and are very similar to the parent owner rules defined
+# above. We should probably deprecate SG_OWNER rules and use PARENT_OWNER
+# instead but this can be done later
+# TODO(slaweq): Deprecate SG_OWNER rules and replace them with PARENT_OWNER
+# rules but for that, 'ext_parent_owner:tenant_id' needs to be added to the SG
+# rule target dict
+SG_OWNER_MEMBER = 'role:member and ' + RULE_SG_OWNER
+SG_OWNER_READER = 'role:reader and ' + RULE_SG_OWNER
+ADMIN_OR_SG_OWNER_MEMBER = (
+    '(' + ADMIN + ') or (' + SG_OWNER_MEMBER + ')')
+ADMIN_OR_SG_OWNER_READER = (
+    '(' + ADMIN + ') or (' + SG_OWNER_READER + ')')
 
 rules = [
     policy.RuleDefault(
@@ -87,8 +94,8 @@ rules = [
         description='Rule for resource owner access'),
     policy.RuleDefault(
         'admin_or_owner',
-        policy_or('rule:context_is_admin',
-                  'rule:owner'),
+        neutron_policy.policy_or('rule:context_is_admin',
+                                 'rule:owner'),
         description='Rule for admin or owner access'),
     policy.RuleDefault(
         'context_is_advsvc',
@@ -96,13 +103,13 @@ rules = [
         description='Rule for advsvc role access'),
     policy.RuleDefault(
         'admin_or_network_owner',
-        policy_or('rule:context_is_admin',
-                  'tenant_id:%(network:tenant_id)s'),
+        neutron_policy.policy_or('rule:context_is_admin',
+                                 'tenant_id:%(network:tenant_id)s'),
         description='Rule for admin or network owner access'),
     policy.RuleDefault(
         'admin_owner_or_network_owner',
-        policy_or('rule:owner',
-                  RULE_ADMIN_OR_NET_OWNER),
+        neutron_policy.policy_or('rule:owner',
+                                 neutron_policy.RULE_ADMIN_OR_NET_OWNER),
         description=('Rule for resource owner, '
                      'admin or network owner access')),
     policy.RuleDefault(
@@ -124,12 +131,12 @@ rules = [
         description='Rule of shared network'),
     policy.RuleDefault(
         'default',
-        RULE_ADMIN_OR_OWNER,
+        neutron_policy.RULE_ADMIN_OR_OWNER,
         description='Default access rule'),
     policy.RuleDefault(
         'admin_or_ext_parent_owner',
-        policy_or('rule:context_is_admin',
-                  'tenant_id:%(ext_parent:tenant_id)s'),
+        neutron_policy.policy_or('rule:context_is_admin',
+                                 'tenant_id:%(ext_parent:tenant_id)s'),
         description='Rule for common parent owner check'),
     policy.RuleDefault(
         'ext_parent_owner',
