@@ -33,11 +33,13 @@ from neutron_lib.exceptions import agent as agent_exc
 from neutron_lib.exceptions import availability_zone as az_exc
 from neutron_lib.plugins import directory
 from oslo_config import cfg
+from oslo_db import exception as db_exc
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_serialization import jsonutils
 from oslo_utils import importutils
 from oslo_utils import timeutils
+from sqlalchemy.sql import expression
 
 from neutron.agent.common import utils
 from neutron.api.rpc.callbacks import version_manager
@@ -76,10 +78,8 @@ AZ_ATTRIBUTE_MAP = {
 def get_availability_zones_by_agent_type(context, agent_type,
                                          availability_zones):
     """Get list of availability zones based on agent type"""
-
-    agents = agent_obj.Agent.get_agents_by_availability_zones_and_agent_type(
+    return agent_obj.Agent.get_availability_zones_by_agent_type(
         context, agent_type=agent_type, availability_zones=availability_zones)
-    return set(agent.availability_zone for agent in agents)
 
 
 class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
@@ -146,7 +146,7 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
                      'name': k[0], 'resource': k[1],
                      'tenant_id': context.tenant_id}
                     for k, v in self._list_availability_zones(
-                                               context, filters).items()
+                        context, filters).items()
                     if not filter_states or v in filter_states]
         else:
             # NOTE(hichihara): 'tenant_id' is dummy for policy check.
@@ -155,7 +155,7 @@ class AgentAvailabilityZoneMixin(az_ext.AvailabilityZonePluginBase):
                      'name': k[0], 'resource': k[1],
                      'tenant_id': context.tenant_id}
                     for k, v in self._list_availability_zones(
-                                               context, filters).items()]
+                        context, filters).items()]
 
     @db_api.retry_if_session_inactive()
     def validate_availability_zones(self, context, resource_type,
@@ -472,9 +472,10 @@ class AgentExtRpcCallback(object):
         1.0 - Initial version.
         1.1 - report_state now returns agent state.
         1.2 - add method has_alive_neutron_server.
+        1.3 - has_alive_neutron_server tests db connection.
     """
 
-    target = oslo_messaging.Target(version='1.2',
+    target = oslo_messaging.Target(version='1.3',
                                    namespace=constants.RPC_NAMESPACE_STATE)
     START_TIME = timeutils.utcnow()
 
@@ -488,8 +489,23 @@ class AgentExtRpcCallback(object):
         # Initialize RPC api directed to other neutron-servers
         self.server_versions_rpc = resources_rpc.ResourcesPushToServersRpcApi()
 
+    @db_api.CONTEXT_READER
     def has_alive_neutron_server(self, context, **kwargs):
-        return True
+        """Give basic server status to agents.
+
+        Method for agents to check basic server status. In version 1.2 returned
+        always True so MQ connectivity could be checked. From version 1.3
+        return True or False according to a DB connection liveness check, so
+        both MQ and DB connectivity can be checked from the agent.
+        """
+        try:
+            context.session.scalar(expression.select(1))
+            return True
+        except db_exc.DBConnectionError:
+            return False
+        except Exception:
+            LOG.exception('Unexpected exception')
+            return False
 
     @db_api.retry_if_session_inactive()
     def report_state(self, context, **kwargs):
