@@ -37,6 +37,8 @@ WALLABY = 'wallaby'
 XENA = 'xena'
 YOGA = 'yoga'
 ZED = 'zed'
+RELEASE_2023_1 = '2023.1'
+RELEASE_2023_2 = '2023.2'
 
 NEUTRON_MILESTONES = [
     # earlier milestones were not tagged
@@ -54,6 +56,8 @@ NEUTRON_MILESTONES = [
     WALLABY,
     XENA,
     YOGA,
+    ZED,
+    RELEASE_2023_1,
     # Do not add the milestone until the end of the release
 ]
 
