@@ -39,10 +39,13 @@ from neutron.common import _constants as const
 from neutron.db import address_group_db as ag_db
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import rbac_db_mixin as rbac_mixin
+from neutron.extensions import security_groups_default_rules as \
+    ext_sg_default_rules
 from neutron.extensions import securitygroup as ext_sg
 from neutron.objects import base as base_obj
 from neutron.objects import ports as port_obj
 from neutron.objects import securitygroup as sg_obj
+from neutron.objects import securitygroup_default_rules as sg_default_rules_obj
 from neutron import quota
 
 
@@ -53,8 +56,10 @@ DEFAULT_SG_DESCRIPTION = _('Default security group')
 
 @resource_extend.has_resource_extenders
 @registry.has_registry_receivers
-class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
-                           rbac_mixin.RbacPluginMixin):
+class SecurityGroupDbMixin(
+        ext_sg.SecurityGroupPluginBase,
+        ext_sg_default_rules.SecurityGroupDefaultRulesPluginBase,
+        rbac_mixin.RbacPluginMixin):
     """Mixin class to add security group to db_base_plugin_v2."""
 
     __native_bulk_support = True
@@ -104,8 +109,12 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 return self.get_security_group(context, existing_def_sg_id)
 
         with db_api.CONTEXT_WRITER.using(context):
-            delta = len(ext_sg.sg_supported_ethertypes)
-            delta = delta * 2 if default_sg else delta
+            if default_sg:
+                delta = sg_default_rules_obj.SecurityGroupDefaultRule.count(
+                    context, used_in_default_sg=True)
+            else:
+                delta = sg_default_rules_obj.SecurityGroupDefaultRule.count(
+                    context, used_in_non_default_sg=True)
             quota.QUOTAS.quota_limit_check(context, tenant_id,
                                            security_group_rule=delta)
 
@@ -115,24 +124,8 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 name=s['name'], is_default=default_sg, stateful=stateful)
             sg.create()
 
-            for ethertype in ext_sg.sg_supported_ethertypes:
-                if default_sg:
-                    # Allow intercommunication
-                    ingress_rule = sg_obj.SecurityGroupRule(
-                        context, id=uuidutils.generate_uuid(),
-                        project_id=tenant_id, security_group_id=sg.id,
-                        direction='ingress', ethertype=ethertype,
-                        remote_group_id=sg.id)
-                    ingress_rule.create()
-                    sg.rules.append(ingress_rule)
-
-                egress_rule = sg_obj.SecurityGroupRule(
-                    context, id=uuidutils.generate_uuid(),
-                    project_id=tenant_id, security_group_id=sg.id,
-                    direction='egress', ethertype=ethertype)
-                egress_rule.create()
-                sg.rules.append(egress_rule)
-            sg.obj_reset_changes(['rules'])
+            self._create_rules_from_template(
+                context, tenant_id, sg, default_sg)
 
             # fetch sg from db to load the sg rules with sg model.
             # NOTE(slaweq): With new system/project scopes it may happen that
@@ -207,11 +200,9 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         try:
             with db_api.CONTEXT_READER.using(context):
                 ret = self._make_security_group_dict(self._get_security_group(
-                                                     context, id,
-                                                     fields=fields),
-                                                     fields)
+                    context, id, fields=fields), fields)
                 if (fields is None or len(fields) == 0 or
-                   'security_group_rules' in fields):
+                        'security_group_rules' in fields):
                     rules = self.get_security_group_rules(
                         context_lib.get_admin_context(),
                         {'security_group_id': [id]})
@@ -311,13 +302,13 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             sg.update()
             sg_dict = self._make_security_group_dict(sg)
             self._registry_publish(
-                    resources.SECURITY_GROUP,
-                    events.PRECOMMIT_UPDATE,
-                    exc_cls=ext_sg.SecurityGroupConflict,
-                    payload=events.DBEventPayload(
-                        context, request_body=s,
-                        states=(original_security_group,),
-                        resource_id=id, desired_state=sg_dict))
+                resources.SECURITY_GROUP,
+                events.PRECOMMIT_UPDATE,
+                exc_cls=ext_sg.SecurityGroupConflict,
+                payload=events.DBEventPayload(
+                    context, request_body=s,
+                    states=(original_security_group,),
+                    resource_id=id, desired_state=sg_dict))
         registry.publish(resources.SECURITY_GROUP, events.AFTER_UPDATE, self,
                          payload=events.DBEventPayload(
                              context, request_body=s,
@@ -332,12 +323,12 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                'stateful': security_group['stateful'],
                'tenant_id': security_group['tenant_id'],
                'description': security_group['description'],
-               'standard_attr_id': security_group.db_obj.standard_attr.id,
+               'standard_attr_id': security_group.db_obj.standard_attr_id,
                'shared': security_group['shared'],
                }
         if security_group.rules:
             res['security_group_rules'] = [
-                self._make_security_group_rule_dict(r.db_obj)
+                self._make_security_group_rule_dict(r)
                 for r in security_group.rules
             ]
         else:
@@ -411,9 +402,9 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         res = self._create_security_group_rule(context, security_group_rule)
         registry.publish(resources.SECURITY_GROUP_RULE, events.AFTER_CREATE,
                          self, payload=events.DBEventPayload(
-                                   context,
-                                   resource_id=res['id'],
-                                   states=(res,)))
+                             context,
+                             resource_id=res['id'],
+                             states=(res,)))
 
         return res
 
@@ -473,7 +464,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             # otherwise a DetachedInstanceError can occur for model extensions
             sg_rule = sg_obj.SecurityGroupRule.get_object(context,
                                                           id=sg_rule.id)
-            res_rule_dict = self._make_security_group_rule_dict(sg_rule.db_obj)
+            res_rule_dict = self._make_security_group_rule_dict(sg_rule)
             self._registry_publish(
                 resources.SECURITY_GROUP_RULE,
                 events.PRECOMMIT_CREATE,
@@ -484,6 +475,216 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
 
         return res_rule_dict
 
+    def _validate_multiple_remote_entites(self, rule):
+        remote = None
+        for key in ['remote_ip_prefix', 'remote_group_id',
+                    'remote_address_group_id']:
+            if remote and rule.get(key):
+                raise ext_sg.SecurityGroupMultipleRemoteEntites()
+            remote = rule.get(key) or remote
+
+    def _validate_default_security_group_rule(self, rule):
+        self._validate_base_security_group_rule_attributes(rule)
+
+    def _make_default_security_group_rule_dict(self, rule_obj, fields=None):
+        res = {
+            'id': rule_obj['id'],
+            'ethertype': rule_obj['ethertype'],
+            'direction': rule_obj['direction'],
+            'protocol': rule_obj['protocol'],
+            'port_range_min': rule_obj['port_range_min'],
+            'port_range_max': rule_obj['port_range_max'],
+            'remote_ip_prefix': rule_obj['remote_ip_prefix'],
+            'remote_address_group_id': rule_obj[
+                'remote_address_group_id'],
+            'remote_group_id': rule_obj['remote_group_id'],
+            'standard_attr_id': rule_obj.db_obj.standard_attr_id,
+            'description': rule_obj['description'],
+            'used_in_default_sg': rule_obj['used_in_default_sg'],
+            'used_in_non_default_sg': rule_obj['used_in_non_default_sg']
+        }
+        return db_utils.resource_fields(res, fields)
+
+    def _get_default_security_group_rule(self, context, rule_id):
+        rule_obj = sg_default_rules_obj.SecurityGroupDefaultRule.get_object(
+            context, id=rule_id)
+        if rule_obj is None:
+            raise ext_sg_default_rules.DefaultSecurityGroupRuleNotFound(
+                id=rule_id)
+        return rule_obj
+
+    def _check_for_duplicate_default_rules(self, context, new_rules):
+        # We need to divide rules for those used in default security groups for
+        # projects and for those which are used only for custom security groups
+        self._check_for_duplicate_default_rules_in_template(
+            context,
+            rules=[rule['default_security_group_rule'] for rule in new_rules
+                   if rule.get('used_in_default_sg', False)],
+            filters={'used_in_default_sg': True})
+        self._check_for_duplicate_default_rules_in_template(
+            context,
+            rules=[rule['default_security_group_rule'] for rule in new_rules
+                   if rule.get('used_in_non_default_sg', True)],
+            filters={'used_in_non_default_sg': True})
+
+    def _check_for_duplicate_default_rules_in_template(self, context,
+                                                       rules, filters):
+        new_rules_set = set()
+        for i in rules:
+            rule_key = self._rule_to_key(rule=i)
+            if rule_key in new_rules_set:
+                raise ext_sg_default_rules.DuplicateDefaultSgRuleInPost(rule=i)
+            new_rules_set.add(rule_key)
+
+        # Now, let's make sure none of the new rules conflict with
+        # existing rules; note that we do *not* store the db rules
+        # in the set, as we assume they were already checked,
+        # when added.
+        template_sg_rules = self.get_default_security_group_rules(
+            context, filters=filters) or []
+        for i in template_sg_rules:
+            rule_key = self._rule_to_key(i)
+            if rule_key in new_rules_set:
+                raise ext_sg_default_rules.DefaultSecurityGroupRuleExists(
+                    rule_id=i.get('id'))
+
+    def create_default_security_group_rule(self, context,
+                                           default_security_group_rule):
+        """Create a default security rule template.
+
+        :param context: neutron api request context
+        :type context: neutron.context.Context
+        :param default_security_group_rule: security group rule template data
+                                            to be applied
+        :type sg_rule_template: dict
+
+        :returns: a SecurityGroupDefaultRule object
+        """
+        self._validate_default_security_group_rule(
+            default_security_group_rule['default_security_group_rule'])
+        self._check_for_duplicate_default_rules(context,
+                                                [default_security_group_rule])
+        rule_dict = default_security_group_rule['default_security_group_rule']
+        remote_ip_prefix = rule_dict.get('remote_ip_prefix')
+        if remote_ip_prefix:
+            remote_ip_prefix = net.AuthenticIPNetwork(remote_ip_prefix)
+
+        protocol = rule_dict.get('protocol')
+        if protocol:
+            # object expects strings only
+            protocol = str(protocol)
+
+        args = {
+            'id': (rule_dict.get('id') or
+                   uuidutils.generate_uuid()),
+            'direction': rule_dict.get('direction'),
+            'remote_group_id': rule_dict.get('remote_group_id'),
+            'remote_address_group_id': rule_dict.get(
+                'remote_address_group_id'),
+            'ethertype': rule_dict.get('ethertype'),
+            'protocol': protocol,
+            'remote_ip_prefix': remote_ip_prefix,
+            'description': rule_dict.get('description'),
+            'used_in_default_sg': rule_dict.get('used_in_default_sg'),
+            'used_in_non_default_sg': rule_dict.get('used_in_non_default_sg')
+        }
+
+        port_range_min = self._safe_int(rule_dict.get('port_range_min'))
+        if port_range_min is not None:
+            args['port_range_min'] = port_range_min
+
+        port_range_max = self._safe_int(rule_dict.get('port_range_max'))
+        if port_range_max is not None:
+            args['port_range_max'] = port_range_max
+
+        with db_api.CONTEXT_WRITER.using(context):
+            default_sg_rule_obj = (
+                sg_default_rules_obj.SecurityGroupDefaultRule(context, **args))
+            default_sg_rule_obj.create()
+        return self._make_default_security_group_rule_dict(default_sg_rule_obj)
+
+    @db_api.CONTEXT_WRITER
+    def delete_default_security_group_rule(self, context, sg_rule_template_id):
+        """Delete a default security rule template.
+
+        :param context: neutron api request context
+        :type context: neutron.context.Context
+        :param sg_rule_template_id: the id of the SecurityGroupDefaultRule to
+                                    delete
+        :type sg_rule_template_id: str uuid
+
+        :returns: None
+        """
+        default_sg_rule_obj = (
+            sg_default_rules_obj.SecurityGroupDefaultRule(context))
+        default_sg_rule_obj.id = sg_rule_template_id
+        default_sg_rule_obj.delete()
+
+    def _create_rules_from_template(self, context, project_id, sg, default_sg):
+        if default_sg:
+            filters = {'used_in_default_sg': True}
+        else:
+            filters = {'used_in_non_default_sg': True}
+        template_sg_rules = self.get_default_security_group_rules(
+            context, filters=filters)
+        for rule_args in template_sg_rules:
+            # We need to filter out attributes which are relevant only to
+            # the template rule and not to the rule itself
+            rule_args.pop('standard_attr_id', None)
+            rule_args.pop('description', None)
+            rule_args.pop('used_in_default_sg', None)
+            rule_args.pop('used_in_non_default_sg', None)
+            rule_args.pop('id', None)
+            if rule_args.get(
+                    'remote_group_id') == ext_sg_default_rules.PARENT_SG:
+                rule_args['remote_group_id'] = sg.id
+            new_rule = sg_obj.SecurityGroupRule(
+                context, id=uuidutils.generate_uuid(),
+                project_id=project_id, security_group_id=sg.id,
+                **rule_args)
+            new_rule.create()
+            sg.rules.append(new_rule)
+        sg.obj_reset_changes(['rules'])
+
+    def get_default_security_group_rules(self, context, filters=None,
+                                         fields=None, sorts=None, limit=None,
+                                         marker=None, page_reverse=False):
+        """Get default security rule templates.
+
+        :param context: neutron api request context
+        :type context: neutron.context.Context
+        :param filters: search criteria
+        :type filters: dict
+
+        :returns: SecurityGroupDefaultRule objects meeting the search criteria
+        """
+        filters = filters or {}
+        pager = base_obj.Pager(
+            sorts=sorts, marker=marker, limit=limit, page_reverse=page_reverse)
+        rule_objs = sg_default_rules_obj.SecurityGroupDefaultRule.get_objects(
+            context, _pager=pager, **filters)
+        return [
+            self._make_default_security_group_rule_dict(obj, fields)
+            for obj in rule_objs
+        ]
+
+    def get_default_security_group_rule(self, context, sg_rule_template_id,
+                                        fields=None):
+        """Get default security rule template.
+
+        :param context: neutron api request context
+        :type context: neutron.context.Context
+        :param sg_rule_template_id: the id of the SecurityGroupDefaultRule to
+                                    get
+        :type sg_rule_template_id: str uuid
+
+        :returns: a SecurityGroupDefaultRule object
+        """
+        rule_obj = self._get_default_security_group_rule(context,
+                                                         sg_rule_template_id)
+        return self._make_default_security_group_rule_dict(
+            rule_obj, fields=fields)
+
     def _get_ip_proto_number(self, protocol):
         if protocol is None:
             return
@@ -592,7 +793,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                                 str(constants.PROTO_NUM_IPV6_ROUTE)]:
             if rule['ethertype'] == constants.IPv4:
                 raise ext_sg.SecurityGroupEthertypeConflictWithProtocol(
-                        ethertype=rule['ethertype'], protocol=rule['protocol'])
+                    ethertype=rule['ethertype'], protocol=rule['protocol'])
 
     def _validate_single_tenant_and_group(self, security_group_rules):
         """Check that all rules belong to the same security group and tenant
@@ -631,20 +832,22 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                           'new_protocol': str(constants.PROTO_NUM_IPV6_ICMP)})
                 rule['protocol'] = str(constants.PROTO_NUM_IPV6_ICMP)
 
-    def _validate_security_group_rule(self, context, security_group_rule):
-        rule = security_group_rule['security_group_rule']
+    def _validate_base_security_group_rule_attributes(self, rule):
+        """Validate values of the basic attributes of the SG rule.
+
+        This method validates attributes which are common for the actual SG
+        rule as well as SG rule template.
+        """
         self._make_canonical_ipv6_icmp_protocol(rule)
         self._make_canonical_port_range(rule)
         self._validate_port_range(rule)
         self._validate_ip_prefix(rule)
         self._validate_ethertype_and_protocol(rule)
+        self._validate_multiple_remote_entites(rule)
 
-        remote = None
-        for key in ['remote_ip_prefix', 'remote_group_id',
-                    'remote_address_group_id']:
-            if remote and rule.get(key):
-                raise ext_sg.SecurityGroupMultipleRemoteEntites()
-            remote = rule.get(key) or remote
+    def _validate_security_group_rule(self, context, security_group_rule):
+        rule = security_group_rule['security_group_rule']
+        self._validate_base_security_group_rule_attributes(rule)
 
         remote_group_id = rule['remote_group_id']
         # Check that remote_group_id exists for tenant
@@ -682,31 +885,34 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         return sg_id
 
     def _make_security_group_rule_dict(self, security_group_rule, fields=None):
-        res = {'id': security_group_rule['id'],
-               'tenant_id': security_group_rule['tenant_id'],
-               'security_group_id': security_group_rule['security_group_id'],
-               'ethertype': security_group_rule['ethertype'],
-               'direction': security_group_rule['direction'],
-               'protocol': security_group_rule['protocol'],
-               'port_range_min': security_group_rule['port_range_min'],
-               'port_range_max': security_group_rule['port_range_max'],
-               'remote_ip_prefix': security_group_rule['remote_ip_prefix'],
-               'remote_address_group_id': security_group_rule[
-                   'remote_address_group_id'],
+        if isinstance(security_group_rule, base_obj.NeutronDbObject):
+            sg_rule_db = security_group_rule.db_obj
+        else:
+            sg_rule_db = security_group_rule
+        res = {'id': sg_rule_db.id,
+               'project_id': sg_rule_db.project_id,
+               'tenant_id': sg_rule_db.project_id,
+               'security_group_id': sg_rule_db.security_group_id,
+               'ethertype': sg_rule_db.ethertype,
+               'direction': sg_rule_db.direction,
+               'protocol': sg_rule_db.protocol,
+               'port_range_min': sg_rule_db.port_range_min,
+               'port_range_max': sg_rule_db.port_range_max,
+               'remote_ip_prefix': sg_rule_db.remote_ip_prefix,
+               'remote_address_group_id': sg_rule_db.remote_address_group_id,
                'normalized_cidr': self._get_normalized_cidr_from_rule(
-                   security_group_rule),
-               'remote_group_id': security_group_rule['remote_group_id'],
-               'standard_attr_id': security_group_rule.standard_attr.id,
+                   sg_rule_db),
+               'remote_group_id': sg_rule_db.remote_group_id,
+               'standard_attr_id': sg_rule_db.standard_attr.id,
                }
 
-        resource_extend.apply_funcs(ext_sg.SECURITYGROUPRULES, res,
-                                    security_group_rule)
+        resource_extend.apply_funcs(ext_sg.SECURITYGROUPRULES, res, sg_rule_db)
         return db_utils.resource_fields(res, fields)
 
     @staticmethod
     def _get_normalized_cidr_from_rule(rule):
         normalized_cidr = None
-        remote_ip_prefix = rule.get('remote_ip_prefix')
+        remote_ip_prefix = rule.remote_ip_prefix
         if remote_ip_prefix:
             normalized_cidr = str(
                 net.AuthenticIPNetwork(remote_ip_prefix).cidr)
@@ -725,7 +931,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 return none_char
             elif key == 'protocol':
                 return str(self._get_ip_proto_name_and_num(
-                               value, ethertype=rule.get('ethertype')))
+                    value, ethertype=rule.get('ethertype')))
             return str(value)
 
         comparison_keys = [
@@ -834,8 +1040,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         # be returned
         security_group_rule = self._get_security_group_rule(
             context_lib.get_admin_context(), id)
-        return self._make_security_group_rule_dict(
-            security_group_rule.db_obj, fields)
+        return self._make_security_group_rule_dict(security_group_rule, fields)
 
     def _get_security_group_rule(self, context, id):
         sgr = sg_obj.SecurityGroupRule.get_object(context, id=id)
@@ -1001,9 +1206,9 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         is either [] or not is_attr_set, otherwise return False
         """
         if (ext_sg.SECURITYGROUPS in port['port'] and
-            not (validators.is_attr_set(
-                     port['port'][ext_sg.SECURITYGROUPS]) and
-                 port['port'][ext_sg.SECURITYGROUPS] != [])):
+                not (validators.is_attr_set(
+                    port['port'][ext_sg.SECURITYGROUPS]) and
+                     port['port'][ext_sg.SECURITYGROUPS] != [])):
             return True
         return False
 
@@ -1013,8 +1218,9 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         This method is called both for port create and port update.
         """
         if (ext_sg.SECURITYGROUPS in port['port'] and
-            (validators.is_attr_set(port['port'][ext_sg.SECURITYGROUPS]) and
-             port['port'][ext_sg.SECURITYGROUPS] != [])):
+                (validators.is_attr_set(
+                    port['port'][ext_sg.SECURITYGROUPS]) and
+                 port['port'][ext_sg.SECURITYGROUPS] != [])):
             return True
         return False
 
@@ -1030,9 +1236,9 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         need_notify = False
         port_updates = port['port']
         if (ext_sg.SECURITYGROUPS in port_updates and
-            not helpers.compare_elements(
-                original_port.get(ext_sg.SECURITYGROUPS),
-                port_updates[ext_sg.SECURITYGROUPS])):
+                not helpers.compare_elements(
+                    original_port.get(ext_sg.SECURITYGROUPS),
+                    port_updates[ext_sg.SECURITYGROUPS])):
             # delete the port binding and read it with the new rules
             sgs = self._get_security_groups_on_port(context, port)
             port_updates[ext_sg.SECURITYGROUPS] = [sg.id for sg in sgs]
