@@ -21,6 +21,7 @@ from neutron_lib.utils import net as net_utils
 from oslo_utils import versionutils
 from oslo_versionedobjects import fields as obj_fields
 from sqlalchemy import func
+from sqlalchemy import or_
 from sqlalchemy import sql
 
 from neutron.db.models import dvr as dvr_models
@@ -30,6 +31,7 @@ from neutron.db.models import l3agent as rb_model
 from neutron.db import models_v2
 from neutron.objects import base
 from neutron.objects.qos import binding as qos_binding
+from neutron.plugins.ml2 import models as ml2_models
 
 
 @base.NeutronObjectRegistry.register
@@ -71,7 +73,8 @@ class RouterRoute(base.NeutronDbObject):
 @base.NeutronObjectRegistry.register
 class RouterExtraAttributes(base.NeutronDbObject):
     # Version 1.0: Initial version
-    VERSION = '1.0'
+    # Version 1.1: Added ECMP and BFD attributes
+    VERSION = '1.1'
 
     db_model = l3_attrs.RouterExtraAttributes
 
@@ -81,7 +84,10 @@ class RouterExtraAttributes(base.NeutronDbObject):
         'service_router': obj_fields.BooleanField(default=False),
         'ha': obj_fields.BooleanField(default=False),
         'ha_vr_id': obj_fields.IntegerField(nullable=True),
-        'availability_zone_hints': obj_fields.ListOfStringsField(nullable=True)
+        'availability_zone_hints': obj_fields.ListOfStringsField(
+            nullable=True),
+        'enable_default_route_bfd': obj_fields.BooleanField(default=False),
+        'enable_default_route_ecmp': obj_fields.BooleanField(default=False),
     }
 
     primary_keys = ['router_id']
@@ -109,7 +115,7 @@ class RouterExtraAttributes(base.NeutronDbObject):
 
     @classmethod
     @db_api.CONTEXT_READER
-    def get_router_agents_count(cls, context):
+    def get_router_agents_count(cls, context, ha=False, less_than=0):
         # TODO(sshank): This is pulled out from l3_agentschedulers_db.py
         # until a way to handle joins is figured out.
         binding_model = rb_model.RouterL3AgentBinding
@@ -121,12 +127,21 @@ class RouterExtraAttributes(base.NeutronDbObject):
                           l3_attrs.RouterExtraAttributes.router_id).
                      join(l3.Router).
                      group_by(binding_model.router_id).subquery())
-
-        query = (context.session.query(l3.Router, sub_query.c.count).
-                 outerjoin(sub_query))
+        count = func.coalesce(sub_query.c.count, 0)
+        query = (context.session.query(l3.Router, count).
+                 outerjoin(sub_query).join(l3_attrs.RouterExtraAttributes).
+                 filter(l3_attrs.RouterExtraAttributes.ha == ha))
+        if less_than > 0:
+            query = query.filter(count < less_than)
 
         return list(query)
 
+    def obj_make_compatible(self, primitive, target_version):
+        _target_version = versionutils.convert_version_to_tuple(target_version)
+        if _target_version < (1, 1):
+            primitive.pop('enable_default_route_bfd', None)
+            primitive.pop('enable_default_route_ecmp', None)
+
 
 @base.NeutronObjectRegistry.register
 class RouterPort(base.NeutronDbObject):
@@ -160,6 +175,15 @@ class RouterPort(base.NeutronDbObject):
         query = query.distinct()
         return [r[0] for r in query]
 
+    @classmethod
+    @db_api.CONTEXT_READER
+    def get_gw_port_ids_by_router_id(cls, context, router_id):
+        query = context.session.query(l3.RouterPort)
+        query = query.filter(
+            l3.RouterPort.router_id == router_id,
+            l3.RouterPort.port_type == n_const.DEVICE_OWNER_ROUTER_GW)
+        return [rp.port_id for rp in query]
+
 
 @base.NeutronObjectRegistry.register
 class DVRMacAddress(base.NeutronDbObject):
@@ -390,7 +414,7 @@ class FloatingIP(base.NeutronDbObject):
 
     @classmethod
     @db_api.CONTEXT_READER
-    def get_scoped_floating_ips(cls, context, router_ids):
+    def get_scoped_floating_ips(cls, context, router_ids, host=None):
         query = context.session.query(l3.FloatingIP,
                                       models_v2.SubnetPool.address_scope_id)
         query = query.join(
@@ -408,8 +432,25 @@ class FloatingIP(base.NeutronDbObject):
         # Filter out on router_ids
         query = query.filter(l3.FloatingIP.router_id.in_(router_ids))
 
-        # Remove duplicate rows based on FIP IDs
-        query = query.group_by(l3.FloatingIP.id)
+        # If a host value is provided, filter output to a specific host
+        if host is not None:
+            query = query.outerjoin(
+                ml2_models.PortBinding,
+                models_v2.Port.id == ml2_models.PortBinding.port_id)
+            # Also filter for ports with migrating_to as they may be relevant
+            # to this host but might not yet have the 'host' column updated
+            # if the migration is in a pre-live migration state
+            query = query.filter(or_(
+                ml2_models.PortBinding.host == host,
+                ml2_models.PortBinding.profile.like('%migrating_to%'),
+            ))
+
+        # Remove duplicate rows based on FIP IDs and the subnet pool address
+        # scope. Only one subnet pool (per IP version, 4 in this case) can
+        # be assigned to a subnet. The subnet pool address scope for a FIP is
+        # unique.
+        query = query.group_by(l3.FloatingIP.id,
+                               models_v2.SubnetPool.address_scope_id)
 
         for row in query:
             yield (cls._load_object(context, row[0]), row[1])
