@@ -40,16 +40,23 @@ class API(api.API, metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def set_lswitch_port(self, lport_name, if_exists=True, **columns):
+    def set_lswitch_port(self, lport_name, external_ids_update=None,
+                         if_exists=True, **columns):
         """Create a command to set OVN logical switch port fields
 
         :param lport_name:    The name of the lport
         :type lport_name:     string
+        :param external_ids_update: Dictionary of keys to be updated
+                              individually in the external IDs dictionary.
+                              If "external_ids" is defined in "columns",
+                              "external_ids" will override any
+                              "external_ids_update" value.
+        :type external_ids_update: dictionary
+        :param if_exists:     Do not fail if lport does not exist
+        :type if_exists:      bool
         :param columns:       Dictionary of port columns
                               Supported columns: macs, external_ids,
                                                  parent_name, tag, enabled
-        :param if_exists:     Do not fail if lport does not exist
-        :type if_exists:      bool
         :type columns:        dictionary
         :returns:             :class:`Command` with no result
         """
@@ -291,6 +298,15 @@ class API(api.API, metaclass=abc.ABCMeta):
         :returns:              a list of strings with the chassis names
         """
 
+    @abc.abstractmethod
+    def get_gateway_chassis_az_hints(self, gateway_id):
+        """Return the list of availability zone hints
+
+        :param gateway_id:     The gateway id
+        :type gateway_id:      string
+        :returns:              a list of strings with the availability zones
+        """
+
     @abc.abstractmethod
     def get_unhosted_gateways(self, port_physnet_dict, chassis_physnets,
                               gw_chassis, chassis_with_azs):
@@ -643,6 +659,20 @@ class SbAPI(api.API, metaclass=abc.ABCMeta):
         :returns: List with chassis.
         """
 
+    @abc.abstractmethod
+    def get_extport_chassis_from_cms_options(self):
+        """Get chassis eligible for hosting external ports from CMS options.
+
+        When admin wants to enable hosting external ports on different
+        chassis than gateway chassis as
+
+        ovs-vsctl set open .
+           external_ids:ovn-cms-options="enable-chassis-as-extport-host"
+        In this function, we parse ovn-cms-options and return these chassis
+
+        :returns: List with chassis
+        """
+
     @abc.abstractmethod
     def get_chassis_and_physnets(self):
         """Return a dict contains chassis name and physnets mapping.
@@ -658,3 +688,11 @@ class SbAPI(api.API, metaclass=abc.ABCMeta):
         :param chassis_type:    The type of chassis
         :type chassis_type:     string
         """
+
+    @abc.abstractmethod
+    def get_chassis_host_for_port(self, port_id):
+        """Return a list of Chassis name hosting the port
+
+        :param port_id: The port ID
+        :type port_id:  string
+        """
