@@ -15,6 +15,7 @@ import functools
 import socket
 import uuid
 
+from neutron_lib import constants
 from neutron_lib import exceptions as n_exc
 from neutron_lib.utils import helpers
 from oslo_log import log
@@ -159,6 +160,10 @@ class Backend(ovs_idl.Backend):
                                                         cls.schema)
         return cls._schema_helper
 
+    @classmethod
+    def get_schema_version(cls):
+        return cls.schema_helper.schema_json['version']
+
     @classmethod
     def schema_has_table(cls, table_name):
         return table_name in cls.schema_helper.schema_json['tables']
@@ -277,8 +282,9 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return cmd.AddLSwitchPortCommand(self, lport_name, lswitch_name,
                                          may_exist, **columns)
 
-    def set_lswitch_port(self, lport_name, if_exists=True, **columns):
-        return cmd.SetLSwitchPortCommand(self, lport_name,
+    def set_lswitch_port(self, lport_name, external_ids_update=None,
+                         if_exists=True, **columns):
+        return cmd.SetLSwitchPortCommand(self, lport_name, external_ids_update,
                                          if_exists, **columns)
 
     def update_lswitch_qos_options(self, port, if_exists=True, **qos):
@@ -295,11 +301,11 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
                                  "delete by lport-name"))
 
     def get_all_stateless_fip_nats(self):
-        cmd = self.db_find('NAT',
+        cmd = self.db_find(
+            'NAT',
             ('external_ids', '!=', {ovn_const.OVN_FIP_EXT_ID_KEY: ''}),
             ('options', '=', {'stateless': 'true'}),
-            ('type', '=', 'dnat_and_snat')
-        )
+            ('type', '=', 'dnat_and_snat'))
         return cmd.execute(check_error=True)
 
     def get_all_logical_switches_with_ports(self):
@@ -355,6 +361,10 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
                         columns['external_mac'] = nat.external_mac[0]
                     if nat.logical_port:
                         columns['logical_port'] = nat.logical_port[0]
+                    columns['external_ids'] = nat.external_ids
+                    columns['uuid'] = nat.uuid
+                    if utils.is_nat_gateway_port_supported(self):
+                        columns['gateway_port'] = nat.gateway_port
                     dnat_and_snats.append(columns)
                 elif nat.type == 'snat':
                     snat.append(columns)
@@ -730,7 +740,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         rc = self.db_find_rows('Load_Balancer', (
             'external_ids', '=',
             {ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY:
-                pf_const.PORT_FORWARDING_PLUGIN,
+             pf_const.PORT_FORWARDING_PLUGIN,
              ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: lrouter_name}))
         return [ovn_obj for ovn_obj in rc.execute(check_error=True)
                 if ovn_const.OVN_FIP_EXT_ID_KEY in ovn_obj.external_ids]
@@ -742,7 +752,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         result = self.db_find('Load_Balancer', (
             'external_ids', '=',
             {ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY:
-                pf_const.PORT_FORWARDING_PLUGIN,
+             pf_const.PORT_FORWARDING_PLUGIN,
              ovn_const.OVN_FIP_EXT_ID_KEY: fip_id})).execute(check_error=True)
         return result[0] if result else None
 
@@ -758,8 +768,8 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
         for nat in self.get_lrouter_nat_rules(utils.ovn_name(router_id)):
             if (nat['type'] == 'dnat_and_snat' and
-               nat['logical_ip'] == logical_ip and
-               nat['external_ip'] == external_ip):
+                    nat['logical_ip'] == logical_ip and
+                    nat['external_ip'] == external_ip):
                 return nat
 
     def check_revision_number(self, name, resource, resource_type,
@@ -784,6 +794,22 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         result = lrp.execute(check_error=True)
         return result[0] if result else None
 
+    def get_lrouter_gw_ports(self, lrouter_name):
+        lr = self.get_lrouter(lrouter_name)
+        gw_ports = []
+        for lrp in getattr(lr, 'ports', []):
+            lrp_ext_ids = getattr(lrp, 'external_ids', {})
+            if (ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY not in lrp_ext_ids or
+                    utils.ovn_name(lrp_ext_ids[
+                        ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY]) != lr.name):
+                continue
+            lrp_ha_cfg = (getattr(lrp, 'gateway_chassis', None) or
+                          getattr(lrp, 'options', {}).get(
+                              ovn_const.OVN_GATEWAY_CHASSIS_KEY))
+            if lrp_ha_cfg:
+                gw_ports.append(lrp)
+        return gw_ports
+
     def delete_lrouter_ext_gw(self, lrouter_name, if_exists=True):
         return cmd.DeleteLRouterExtGwCommand(self, lrouter_name, if_exists)
 
@@ -802,8 +828,8 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         port_groups = {}
         for row in self._tables['Port_Group'].rows.values():
             name = getattr(row, 'name')
-            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
-               name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
+            if (ovn_const.OVN_SG_EXT_ID_KEY not in row.external_ids or
+                    name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
                 continue
             data = {}
             for row_key in getattr(row, "_data", {}):
@@ -847,7 +873,8 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         return cls(conn)
 
     def _get_chassis_physnets(self, chassis):
-        bridge_mappings = chassis.external_ids.get('ovn-bridge-mappings', '')
+        other_config = utils.get_ovn_chassis_other_config(chassis)
+        bridge_mappings = other_config.get('ovn-bridge-mappings', '')
         mapping_dict = helpers.parse_mappings(bridge_mappings.split(','))
         return list(mapping_dict.keys())
 
@@ -864,8 +891,11 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def get_gateway_chassis_from_cms_options(self, name_only=True):
         return [ch.name if name_only else ch
                 for ch in self.chassis_list().execute(check_error=True)
-                if ovn_const.CMS_OPT_CHASSIS_AS_GW in
-                ch.external_ids.get(ovn_const.OVN_CMS_OPTIONS, '').split(',')]
+                if utils.is_gateway_chassis(ch)]
+
+    def get_extport_chassis_from_cms_options(self):
+        return [ch for ch in self.chassis_list().execute(check_error=True)
+                if utils.is_extport_host_chassis(ch)]
 
     def get_chassis_and_physnets(self):
         chassis_info_dict = {}
@@ -888,9 +918,9 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
                                                     card_serial_number):
         for ch in self.chassis_list().execute(check_error=True):
             if ('{}={}'
-                .format(ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
-                        card_serial_number)
-                    in ch.external_ids.get(
+                    .format(ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
+                            card_serial_number)
+                    in utils.get_ovn_chassis_other_config(ch).get(
                         ovn_const.OVN_CMS_OPTIONS, '').split(',')):
                 return ch
         msg = _('Chassis with %s %s %s does not exist'
@@ -906,7 +936,10 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         except idlutils.RowNotFound:
             return None
         cmd = self.db_find_rows('Port_Binding', ('datapath', '=', dp),
-                                ('type', '=', ovn_const.LSP_TYPE_LOCALPORT))
+                                ('type', '=', ovn_const.LSP_TYPE_LOCALPORT),
+                                ('external_ids', '=', {
+                                    ovn_const.OVN_DEVICE_OWNER_EXT_ID_KEY:
+                                        constants.DEVICE_OWNER_DISTRIBUTED}))
         return next(iter(cmd.execute(check_error=True)), None)
 
     def set_chassis_neutron_description(self, chassis, description,
@@ -945,3 +978,19 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         # and just start using chassis objects so db_find_rows could be used
         rows = self.db_list_rows('Port_Binding').execute(check_error=True)
         return [r for r in rows if r.chassis and r.chassis[0].name == chassis]
+
+    def get_chassis_host_for_port(self, port_id):
+        chassis = set()
+        cmd = self.db_find_rows('Port_Binding', ('logical_port', '=', port_id))
+        for row in cmd.execute(check_error=True):
+            try:
+                chassis.add(row.chassis[0].name)
+            except IndexError:
+                # Do not short-circuit here. Proceed to additional
+                # chassis handling
+                pass
+
+            if utils.is_additional_chassis_supported(self):
+                for ch in row.additional_chassis:
+                    chassis.add(ch.name)
+        return chassis
