@@ -207,7 +207,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     # ports sync operation later.
                     for n_port in db_ports:
                         if ((utils.is_security_groups_enabled(n_port) or
-                                utils.is_port_security_enabled(n_port)) and
+                             utils.is_port_security_enabled(n_port)) and
                                 n_port['id'] in ovn_ports):
                             txn.add(self.ovn_api.pg_add_ports(
                                 pg, n_port['id']))
@@ -284,7 +284,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         # Remove the common ones
         for na in list(neutron_acls):
-            for ovn_a in ovn_acls:
+            for ovn_a in ovn_acls.copy():
                 if all(item in na.items() for item in ovn_a.items()):
                     neutron_acls.remove(na)
                     ovn_acls.remove(ovn_a)
@@ -292,14 +292,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         num_acls_to_add = len(neutron_acls)
         num_acls_to_remove = len(ovn_acls) + num_acls_to_remove_from_ls
-        if 0 != num_acls_to_add or 0 != num_acls_to_remove:
+        if num_acls_to_add != 0 or num_acls_to_remove != 0:
             LOG.warning('ACLs-to-be-added %(add)d '
                         'ACLs-to-be-removed %(remove)d',
                         {'add': num_acls_to_add,
                          'remove': num_acls_to_remove})
 
         if self.mode == SYNC_MODE_REPAIR:
-            pg_resync_count = 0
+            one_time_pg_resync = True
             while True:
                 try:
                     with self.ovn_api.transaction(check_error=True) as txn:
@@ -311,13 +311,13 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                                 **acla, may_exist=True))
                 except idlutils.RowNotFound as row_err:
                     if row_err.msg.startswith("Cannot find Port_Group"):
-                        if pg_resync_count < 1:
+                        if one_time_pg_resync:
                             LOG.warning('Port group row was not found during '
                                         'ACLs sync. Will attempt to sync port '
                                         'groups one more time. The caught '
                                         'exception is: %s', row_err)
                             self.sync_port_groups(ctx)
-                            pg_resync_count += 1
+                            one_time_pg_resync = False
                             continue
                         LOG.error('Port group exception during ACL sync '
                                   'even after one more port group resync. '
@@ -351,7 +351,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         for db_route in db_routes:
             for ovn_route in ovn_routes:
                 if (ovn_route['destination'] == db_route['destination'] and
-                   ovn_route['nexthop'] == db_route['nexthop']):
+                        ovn_route['nexthop'] == db_route['nexthop']):
                     break
             else:
                 to_add.append(db_route)
@@ -359,7 +359,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         for ovn_route in ovn_routes:
             for db_route in db_routes:
                 if (ovn_route['destination'] == db_route['destination'] and
-                   ovn_route['nexthop'] == db_route['nexthop']):
+                        ovn_route['nexthop'] == db_route['nexthop']):
                     break
             else:
                 to_remove.append(ovn_route)
@@ -376,7 +376,8 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 continue
             for ovn_fip in ovn_fips:
                 if (ovn_fip['logical_ip'] == db_fip['fixed_ip_address'] and
-                   ovn_fip['external_ip'] == db_fip['floating_ip_address']):
+                        ovn_fip['external_ip'] ==
+                        db_fip['floating_ip_address']):
                     break
             else:
                 to_add.append(db_fip)
@@ -384,7 +385,8 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         for ovn_fip in ovn_fips:
             for db_fip in db_fips:
                 if (ovn_fip['logical_ip'] == db_fip['fixed_ip_address'] and
-                   ovn_fip['external_ip'] == db_fip['floating_ip_address']):
+                        ovn_fip['external_ip'] ==
+                        db_fip['floating_ip_address']):
                     break
             else:
                 to_remove.append(ovn_fip)
@@ -706,7 +708,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Deleting the router %s from OVN NB DB",
                                 lrouter['name'])
                     txn.add(self.ovn_api.delete_lrouter(
-                            utils.ovn_name(lrouter['name'])))
+                        utils.ovn_name(lrouter['name'])))
 
             for lrport_info in del_lrouter_ports_list:
                 LOG.warning("Router Port found in OVN but not in "
@@ -715,9 +717,9 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     LOG.warning("Deleting the port %s from OVN NB DB",
                                 lrport_info['port'])
                     txn.add(self.ovn_api.delete_lrouter_port(
-                            utils.ovn_lrouter_port_name(lrport_info['port']),
-                            utils.ovn_name(lrport_info['lrouter']),
-                            if_exists=False))
+                        utils.ovn_lrouter_port_name(lrport_info['port']),
+                        utils.ovn_name(lrport_info['lrouter']),
+                        if_exists=False))
             for sroute in update_sroutes_list:
                 if sroute['add']:
                     LOG.warning("Router %(id)s static routes %(route)s "
@@ -978,9 +980,9 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         LOG.debug('OVN sync metadata ports started')
         for net in self.core_plugin.get_networks(ctx):
             metadata_ports = self.core_plugin.get_ports(
-                    ctx, filters=dict(
-                        network_id=[net['id']],
-                        device_owner=[constants.DEVICE_OWNER_DISTRIBUTED]))
+                ctx, filters=dict(
+                    network_id=[net['id']],
+                    device_owner=[constants.DEVICE_OWNER_DISTRIBUTED]))
 
             if not metadata_ports:
                 LOG.warning('Missing metadata port found in Neutron for '
@@ -1184,14 +1186,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                         LOG.debug('Deleting port DHCPv4 options for (port %s)',
                                   lport_info['port'])
                         txn.add(self.ovn_api.delete_dhcp_options(
-                                ovn_all_dhcp_options['ports_v4'].pop(
-                                    lport_info['port'])['uuid']))
+                            ovn_all_dhcp_options['ports_v4'].pop(
+                                lport_info['port'])['uuid']))
                     if lport_info['port'] in ovn_all_dhcp_options['ports_v6']:
                         LOG.debug('Deleting port DHCPv6 options for (port %s)',
                                   lport_info['port'])
                         txn.add(self.ovn_api.delete_dhcp_options(
-                                ovn_all_dhcp_options['ports_v6'].pop(
-                                    lport_info['port'])['uuid']))
+                            ovn_all_dhcp_options['ports_v6'].pop(
+                                lport_info['port'])['uuid']))
 
         self._sync_port_dhcp_options(ctx, ports_need_sync_dhcp_opts,
                                      ovn_all_dhcp_options['ports_v4'],
