@@ -35,10 +35,10 @@ from neutron._i18n import _
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import extensions
 from neutron.common.ovn import utils
-from neutron.common import utils as common_utils
 from neutron.db.availability_zone import router as router_az_db
 from neutron.db import dns_db
 from neutron.db import extraroute_db
+from neutron.db import l3_extra_gws_db
 from neutron.db import l3_fip_pools_db
 from neutron.db import l3_fip_port_details
 from neutron.db import l3_fip_qos
@@ -67,6 +67,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                         l3_fip_qos.FloatingQoSDbMixin,
                         l3_gateway_ip_qos.L3_gw_ip_qos_db_mixin,
                         l3_fip_pools_db.FloatingIPPoolsMixin,
+                        l3_extra_gws_db.ExtraGatewaysDbOnlyMixin,
                         ):
     """Implementation of the OVN L3 Router Service Plugin.
 
@@ -88,6 +89,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         super(OVNL3RouterPlugin, self).__init__()
         self._plugin_property = None
         self._mech = None
+        self._initialize_plugin_driver()
         self._ovn_client_inst = None
         self.scheduler = l3_ovn_scheduler.get_scheduler()
         self.port_forwarding = port_forwarding.OVNPortForwarding(self)
@@ -139,20 +141,24 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             self._plugin_property = directory.get_plugin()
         return self._plugin_property
 
+    def _initialize_plugin_driver(self):
+        # This method initializes the mechanism driver variable and checks
+        # if any of the valid drivers ('ovn', 'ovn-sync') is loaded.
+        drivers = ('ovn', 'ovn-sync')
+        for driver in drivers:
+            try:
+                self._mech = self._plugin.mechanism_manager.mech_drivers[
+                    driver].obj
+                break
+            except KeyError:
+                pass
+        else:
+            raise ovn_l3_exc.MechanismDriverNotFound(mechanism_drivers=drivers)
+
     @property
     def _plugin_driver(self):
         if self._mech is None:
-            drivers = ('ovn', 'ovn-sync')
-            for driver in drivers:
-                try:
-                    self._mech = self._plugin.mechanism_manager.mech_drivers[
-                        driver].obj
-                    break
-                except KeyError:
-                    pass
-            else:
-                raise ovn_l3_exc.MechanismDriverNotFound(
-                        mechanism_drivers=drivers)
+            self._initialize_plugin_driver()
         return self._mech
 
     def get_plugin_type(self):
@@ -165,6 +171,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
 
     def create_router_precommit(self, resource, event, trigger, payload):
         context = payload.context
+        context.session.flush()
         router_id = payload.resource_id
         router_db = payload.metadata['router_db']
 
@@ -179,7 +186,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         except Exception:
             with excutils.save_and_reraise_exception():
                 # Delete the logical router
-                LOG.error('Unable to create lrouter for %s', router['id'])
+                LOG.exception('Unable to create lrouter %s', router['id'])
                 super(OVNL3RouterPlugin, self).delete_router(context,
                                                              router['id'])
         return router
@@ -193,7 +200,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                                            original_router)
         except Exception:
             with excutils.save_and_reraise_exception():
-                LOG.exception('Unable to update lrouter for %s', id)
+                LOG.exception('Unable to update lrouter %s', id)
                 revert_router = {'router': original_router}
                 super(OVNL3RouterPlugin, self).update_router(context, id,
                                                              revert_router)
@@ -206,6 +213,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             self._ovn_client.delete_router(context, id)
         except Exception:
             with excutils.save_and_reraise_exception():
+                LOG.exception('Unable to delete lrouter %s', id)
                 super(OVNL3RouterPlugin, self).create_router(
                     context, {'router': original_router})
 
@@ -242,6 +250,9 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                 context, router_id, router_interface_info)
         except Exception:
             with excutils.save_and_reraise_exception():
+                LOG.exception(
+                    'Unable to add router interface to lrouter %s. '
+                    'Interface info: %s', router_id, interface_info)
                 super(OVNL3RouterPlugin, self).remove_router_interface(
                     context, router_id, router_interface_info)
 
@@ -258,6 +269,9 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                 context, port_id, router_id=router_id, subnet_ids=subnet_ids)
         except Exception:
             with excutils.save_and_reraise_exception():
+                LOG.exception(
+                    'Unable to remove router interface from lrouter %s. '
+                    'Interface info: %s', router_id, interface_info)
                 super(OVNL3RouterPlugin, self).add_router_interface(
                     context, router_id, interface_info)
         return router_interface_info
@@ -285,7 +299,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
     def update_floatingip(self, context, id, floatingip):
         fip = super(OVNL3RouterPlugin, self).update_floatingip(context, id,
                                                                floatingip)
-        self._ovn_client.update_floatingip(context, fip)
+        self._ovn_client.update_floatingip(context, fip, floatingip)
         return fip
 
     def update_floatingip_status(self, context, floatingip_id, status):
@@ -361,26 +375,21 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                 port = self._plugin.update_port(
                     context, port['id'],
                     {'port': {portbindings.HOST_ID: host}})
-
+                # Updates OVN NB database with hostname for lsp router
+                # gateway port
+                with self._nb_ovn.transaction(check_error=True) as txn:
+                    ext_ids = (
+                        "external_ids",
+                        {ovn_const.OVN_HOST_ID_EXT_ID_KEY: host},
+                    )
+                    txn.add(
+                        self._nb_ovn.db_set(
+                            "Logical_Switch_Port", port["id"], ext_ids
+                        )
+                    )
             if port['status'] != status:
                 self._plugin.update_port_status(context, port['id'], status)
 
-    def _get_availability_zones_from_router_port(self, lrp_name):
-        """Return the availability zones hints for the router port.
-
-        Return a list of availability zones hints associated with the
-        router that the router port belongs to.
-        """
-        context = n_context.get_admin_context()
-        if not self._plugin_driver.list_availability_zones(context):
-            return []
-
-        lrp = self._nb_ovn.get_lrouter_port(lrp_name)
-        router = self.get_router(
-            context, lrp.external_ids[ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY])
-        az_hints = common_utils.get_az_hints(router)
-        return az_hints
-
     def schedule_unhosted_gateways(self, event_from_chassis=None):
         # GW ports and its physnets.
         port_physnet_dict = self._get_gateway_port_physnet_mapping()
@@ -429,7 +438,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                 chassis_physnets=chassis_with_physnets,
                 availability_zone_hints=az_hints)
             chassis = self.scheduler.select(
-                self._nb_ovn, self._sb_ovn, g_name, candidates=candidates,
+                self._nb_ovn, g_name, candidates=candidates,
                 existing_chassis=existing_chassis)
             if primary and primary != chassis[0]:
                 if primary not in chassis:
@@ -523,7 +532,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             return []
 
         return [az.strip() for az in lr.external_ids.get(
-                ovn_const.OVN_AZ_HINTS_EXT_ID_KEY, '').split(',')
+            ovn_const.OVN_AZ_HINTS_EXT_ID_KEY, '').split(',')
                 if az.strip()]
 
     def validate_availability_zones(self, context, resource_type,
