@@ -24,6 +24,8 @@ from neutron.agent.common import utils
 from neutron.agent.linux import ip_lib
 from neutron.cmd.sanity import checks
 from neutron.common import utils as common_utils
+from neutron.plugins.ml2.drivers.openvswitch.agent \
+    import ovs_dvr_neutron_agent as ovsdvragt
 from neutron.plugins.ml2.drivers.openvswitch.agent \
     import ovs_neutron_agent as ovsagt
 from neutron.tests.common import base as common_base
@@ -103,9 +105,14 @@ class ARPSpoofTestCase(OVSAgentTestBase):
         details = {'flows': self.br.dump_all_flows(),
                    'vifs': map(nicevif, self.br.get_vif_ports()),
                    'src_ip': self.src_addr,
-                   'dest_ip': self.dst_addr,
-                   'sourt_port': nicedev(self.src_p),
-                   'dest_port': nicedev(self.dst_p)}
+                   'dst_ip': self.dst_addr,
+                   'src_port': nicedev(self.src_p),
+                   'dst_port': nicedev(self.dst_p),
+                   'src_neigh4': self.src_p.neigh.dump(n_const.IP_VERSION_4),
+                   'src_neigh6': self.src_p.neigh.dump(n_const.IP_VERSION_6),
+                   'dst_neigh4': self.dst_p.neigh.dump(n_const.IP_VERSION_4),
+                   'dst_neigh6': self.dst_p.neigh.dump(n_const.IP_VERSION_6),
+                   }
         self.addDetail('arp-test-state',
                        text_content(jsonutils.dumps(details, indent=5)))
 
@@ -119,14 +126,16 @@ class ARPSpoofTestCase(OVSAgentTestBase):
         self._setup_arp_spoof_for_port(self.dst_p.name, [self.dst_addr])
         self.src_p.addr.add('%s/24' % self.src_addr)
         self.dst_p.addr.add('%s/24' % self.dst_addr)
-        net_helpers.assert_ping(self.src_namespace, self.dst_addr)
+        net_helpers.assert_ping(self.src_namespace, self.dst_addr,
+                                retry_count=2)
 
     def test_mac_spoof_blocks_wrong_mac(self):
         self._setup_arp_spoof_for_port(self.src_p.name, [self.src_addr])
         self._setup_arp_spoof_for_port(self.dst_p.name, [self.dst_addr])
         self.src_p.addr.add('%s/24' % self.src_addr)
         self.dst_p.addr.add('%s/24' % self.dst_addr)
-        net_helpers.assert_ping(self.src_namespace, self.dst_addr)
+        net_helpers.assert_ping(self.src_namespace, self.dst_addr,
+                                retry_count=2)
         # changing the allowed mac should stop the port from working
         self._setup_arp_spoof_for_port(self.src_p.name, [self.src_addr],
                                        mac='00:11:22:33:44:55')
@@ -142,7 +151,8 @@ class ARPSpoofTestCase(OVSAgentTestBase):
         # make sure the IPv6 addresses are ready before pinging
         self.src_p.addr.wait_until_address_ready(self.src_addr)
         self.dst_p.addr.wait_until_address_ready(self.dst_addr)
-        net_helpers.assert_ping(self.src_namespace, self.dst_addr)
+        net_helpers.assert_ping(self.src_namespace, self.dst_addr,
+                                retry_count=2)
 
     def test_arp_spoof_blocks_response(self):
         # this will prevent the destination from responding to the ARP
@@ -188,7 +198,8 @@ class ARPSpoofTestCase(OVSAgentTestBase):
                                                          self.dst_addr])
         self.src_p.addr.add('%s/24' % self.src_addr)
         self.dst_p.addr.add('%s/24' % self.dst_addr)
-        net_helpers.assert_ping(self.src_namespace, self.dst_addr)
+        net_helpers.assert_ping(self.src_namespace, self.dst_addr,
+                                retry_count=2)
 
     def test_arp_spoof_icmpv6_neigh_advt_allowed_address_pairs(self):
         self.src_addr = '2000::1'
@@ -200,7 +211,8 @@ class ARPSpoofTestCase(OVSAgentTestBase):
         # make sure the IPv6 addresses are ready before pinging
         self.src_p.addr.wait_until_address_ready(self.src_addr)
         self.dst_p.addr.wait_until_address_ready(self.dst_addr)
-        net_helpers.assert_ping(self.src_namespace, self.dst_addr)
+        net_helpers.assert_ping(self.src_namespace, self.dst_addr,
+                                retry_count=2)
 
     def test_arp_spoof_allowed_address_pairs_0cidr(self):
         self._setup_arp_spoof_for_port(self.dst_p.name, ['9.9.9.9/0',
@@ -217,7 +229,8 @@ class ARPSpoofTestCase(OVSAgentTestBase):
                                        psec=False)
         self.src_p.addr.add('%s/24' % self.src_addr)
         self.dst_p.addr.add('%s/24' % self.dst_addr)
-        net_helpers.assert_ping(self.src_namespace, self.dst_addr)
+        net_helpers.assert_ping(self.src_namespace, self.dst_addr,
+                                retry_count=2)
 
     def test_arp_spoof_disable_network_port(self):
         # block first and then disable port security to make sure old rules
@@ -228,7 +241,8 @@ class ARPSpoofTestCase(OVSAgentTestBase):
             device_owner=n_const.DEVICE_OWNER_ROUTER_GW)
         self.src_p.addr.add('%s/24' % self.src_addr)
         self.dst_p.addr.add('%s/24' % self.dst_addr)
-        net_helpers.assert_ping(self.src_namespace, self.dst_addr)
+        net_helpers.assert_ping(self.src_namespace, self.dst_addr,
+                                retry_count=2)
 
     def _setup_arp_spoof_for_port(self, port, addrs, psec=True,
                                   device_owner='nobody', mac=None):
@@ -299,8 +313,9 @@ class OVSFlowTestCase(OVSAgentTestBase):
     """
 
     def setUp(self):
+        dvr_enabled = True
         cfg.CONF.set_override('enable_distributed_routing',
-                              True,
+                              dvr_enabled,
                               group='AGENT')
         super(OVSFlowTestCase, self).setUp()
         self.phys_br = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
@@ -322,7 +337,9 @@ class OVSFlowTestCase(OVSAgentTestBase):
                 prefix=cfg.CONF.OVS.tun_peer_patch_port),
             common_utils.get_rand_device_name(
                 prefix=cfg.CONF.OVS.int_peer_patch_port))
-        self.br_tun.setup_default_table(self.tun_p, True)
+        self.br_tun.setup_default_table(self.tun_p, True, dvr_enabled)
+        ovsdvragt.OVSDVRNeutronAgent._setup_dvr_flows_on_tun_br(self.br_tun,
+                                                                self.tun_p)
 
     def test_provision_local_vlan(self):
         kwargs = {'port': 123, 'lvid': 888, 'segmentation_id': 777}
