@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 from datetime import datetime
 import errno
 import os
@@ -21,16 +22,13 @@ from unittest import mock
 import warnings
 
 import fixtures
-from neutron_lib import fixture
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
-from oslo_concurrency import lockutils
 from oslo_config import cfg
-from oslo_db import exception as os_db_exc
-from oslo_db.sqlalchemy import provision
 from oslo_log import log
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
+from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron.agent.linux import utils
 from neutron.api import extensions as exts
@@ -53,6 +51,7 @@ from neutron.tests import base
 from neutron.tests.common import base as common_base
 from neutron.tests.common import helpers
 from neutron.tests.functional.resources import process
+from neutron.tests.unit.extensions import test_securitygroup
 from neutron.tests.unit.plugins.ml2 import test_plugin
 import neutron.wsgi
 
@@ -61,7 +60,6 @@ LOG = log.getLogger(__name__)
 # This is the directory from which infra fetches log files for functional tests
 DEFAULT_LOG_DIR = os.path.join(helpers.get_test_log_path(),
                                'dsvm-functional-logs')
-SQL_FIXTURE_LOCK = 'sql_fixture_lock'
 
 
 def config_decorator(method_to_decorate, config_tuples):
@@ -134,27 +132,6 @@ class BaseSudoTestCase(BaseLoggingTestCase):
                           new=ovs_agent_decorator).start()
 
 
-class OVNSqlFixture(fixture.StaticSqlFixture):
-
-    @classmethod
-    @lockutils.synchronized(SQL_FIXTURE_LOCK)
-    def _init_resources(cls):
-        cls.schema_resource = provision.SchemaResource(
-            provision.DatabaseResource("sqlite"),
-            cls._generate_schema, teardown=False)
-        dependency_resources = {}
-        for name, resource in cls.schema_resource.resources:
-            dependency_resources[name] = resource.getResource()
-        cls.schema_resource.make(dependency_resources)
-        cls.engine = dependency_resources['database'].engine
-
-    def _delete_from_schema(self, engine):
-        try:
-            super(OVNSqlFixture, self)._delete_from_schema(engine)
-        except os_db_exc.DBNonExistentTable:
-            pass
-
-
 class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
                             BaseLoggingTestCase):
 
@@ -227,6 +204,11 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         self._start_ovn_northd()
         self.addCleanup(self._reset_agent_cache_singleton)
         self.addCleanup(self._reset_ovn_client_placement_extension)
+        plugin = directory.get_plugin()
+        mock.patch.object(
+            plugin, 'get_default_security_group_rules',
+            return_value=copy.deepcopy(
+                test_securitygroup.RULES_TEMPLATE_FOR_DEFAULT_SG)).start()
 
     def _reset_agent_cache_singleton(self):
         neutron_agent.AgentCache._instance = None
@@ -251,16 +233,6 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         raise FileNotFoundError(
                     errno.ENOENT, os.strerror(errno.ENOENT), msg)
 
-    # FIXME(lucasagomes): Workaround for
-    # https://bugs.launchpad.net/networking-ovn/+bug/1808146. We should
-    # investigate and properly fix the problem. This method is just a
-    # workaround to alleviate the gate for now and should not be considered
-    # a proper fix.
-    def _setup_database_fixtures(self):
-        fixture = OVNSqlFixture()
-        self.useFixture(fixture)
-        self.engine = fixture.engine
-
     def get_additional_service_plugins(self):
         p = super(TestOVNFunctionalBase, self).get_additional_service_plugins()
         p.update({'revision_plugin_name': 'revisions',
@@ -362,7 +334,8 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         self.addCleanup(self.stop)
         # NOTE(ralonsoh): do not access to the DB at exit when the SQL
         # connection is already closed, to avoid useless exception messages.
-        mock.patch.object(self.mech_driver, '_clean_hash_ring').start()
+        mock.patch.object(
+            self.mech_driver, '_remove_node_from_hash_ring').start()
         self.mech_driver.pre_fork_initialize(
             mock.ANY, mock.ANY, trigger_cls.trigger)
 
@@ -406,8 +379,12 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         if self.maintenance_worker:
             self.mech_driver.nb_synchronizer.stop()
             self.mech_driver.sb_synchronizer.stop()
-        self.mech_driver.nb_ovn.ovsdb_connection.stop()
-        self.mech_driver.sb_ovn.ovsdb_connection.stop()
+        for ovn_conn in (self.mech_driver.nb_ovn.ovsdb_connection,
+                         self.mech_driver.sb_ovn.ovsdb_connection):
+            try:
+                ovn_conn.stop(timeout=10)
+            except Exception:  # pylint:disable=bare-except
+                pass
 
     def restart(self):
         self.stop()
@@ -423,7 +400,8 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         self._start_ovn_northd()
 
     def add_fake_chassis(self, host, physical_nets=None, external_ids=None,
-                         name=None, azs=None, enable_chassis_as_gw=False):
+                         name=None, azs=None, enable_chassis_as_gw=False,
+                         enable_chassis_as_extport=False, other_config=None):
         def append_cms_options(ext_ids, value):
             if 'ovn-cms-options' not in ext_ids:
                 ext_ids['ovn-cms-options'] = value
@@ -432,19 +410,22 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
 
         physical_nets = physical_nets or []
         external_ids = external_ids or {}
+        other_config = other_config or {}
         if azs is None:
             azs = ['ovn']
         if azs:
-            append_cms_options(external_ids, 'availability-zones=')
-            external_ids['ovn-cms-options'] += ':'.join(azs)
+            append_cms_options(other_config, 'availability-zones=')
+            other_config['ovn-cms-options'] += ':'.join(azs)
         if enable_chassis_as_gw:
-            append_cms_options(external_ids, 'enable-chassis-as-gw')
+            append_cms_options(other_config, 'enable-chassis-as-gw')
+        if enable_chassis_as_extport:
+            append_cms_options(other_config, 'enable-chassis-as-extport-host')
 
         bridge_mapping = ",".join(["%s:br-provider%s" % (phys_net, i)
                                   for i, phys_net in enumerate(physical_nets)])
         if name is None:
             name = uuidutils.generate_uuid()
-        external_ids['ovn-bridge-mappings'] = bridge_mapping
+        other_config['ovn-bridge-mappings'] = bridge_mapping
         # We'll be using different IP addresses every time for the Encap of
         # the fake chassis as the SB schema doesn't allow to have two entries
         # with same (ip,type) pairs as of OVS 2.11. This shouldn't have any
@@ -455,7 +436,8 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         self._counter += 1
         chassis = self.sb_api.chassis_add(
             name, ['geneve'], '172.24.4.%d' % self._counter,
-            external_ids=external_ids, hostname=host).execute(check_error=True)
+            external_ids=external_ids, hostname=host,
+            other_config=other_config).execute(check_error=True)
         if self.sb_api.is_table_present('Chassis_Private'):
             nb_cfg_timestamp = timeutils.utcnow_ts() * 1000
             self.sb_api.db_create(
@@ -467,6 +449,13 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
     def del_fake_chassis(self, chassis, if_exists=True):
         self.sb_api.chassis_del(
             chassis, if_exists=if_exists).execute(check_error=True)
-        if self.sb_api.is_table_present('Chassis_Private'):
-            self.sb_api.db_destroy(
-                'Chassis_Private', chassis).execute(check_error=True)
+        try:
+            if self.sb_api.is_table_present('Chassis_Private'):
+                self.sb_api.db_destroy(
+                    'Chassis_Private', chassis).execute(check_error=True)
+        except idlutils.RowNotFound:
+            # NOTE(ykarel ): ovsdbapp >= 2.4.2 handles Chassis_Private
+            # record delete with chassis
+            # try/except can be dropped when neutron requirements.txt
+            # include ovsdbapp>=2.4.2
+            pass
