@@ -16,10 +16,13 @@
 from unittest import mock
 
 from oslo_config import cfg
+from oslo_utils import strutils
+from oslo_utils import uuidutils
 
 from futurist import periodics
 from neutron_lib.api.definitions import external_net as extnet_apidef
 from neutron_lib.api.definitions import floating_ip_port_forwarding as pf_def
+from neutron_lib.api.definitions import provider_net as provnet_apidef
 from neutron_lib import constants as n_const
 from neutron_lib import context as n_context
 from neutron_lib.exceptions import l3 as lib_l3_exc
@@ -50,6 +53,8 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
         ext_mgr = test_extraroute.ExtraRouteTestExtensionManager()
         self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
         self.maint = maintenance.DBInconsistenciesPeriodics(self._ovn_client)
+        # Release the unneeded lock
+        self.maint._idl.set_lock(None)
         self.context = n_context.get_admin_context()
         # Always verify inconsistencies for all objects.
         db_rev.INCONSISTENCIES_OLDER_THAN = -1
@@ -60,10 +65,19 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
                     ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY) == name):
                 return row
 
-    def _create_network(self, name, external=False):
-        data = {'network': {'name': name, 'tenant_id': self._tenant_id,
+    def _create_network(self, name, external=False, provider=None,
+                        net_type=None):
+        data = {'network': {'name': name,
                             extnet_apidef.EXTERNAL: external}}
-        req = self.new_create_request('networks', data, self.fmt)
+        if net_type:
+            data['network'][provnet_apidef.NETWORK_TYPE] = net_type
+        if provider:
+            net_type = net_type or 'flat'
+            data['network'][provnet_apidef.NETWORK_TYPE] = net_type
+            data['network'][provnet_apidef.PHYSICAL_NETWORK] = provider
+
+        req = self.new_create_request('networks', data, self.fmt,
+                                      as_admin=True)
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['network']
 
@@ -74,9 +88,8 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
         return self.deserialize(self.fmt, res)['network']
 
     def _create_port(self, name, net_id, security_groups=None,
-                     device_owner=None):
+                     device_owner=None, vnic_type=None):
         data = {'port': {'name': name,
-                         'tenant_id': self._tenant_id,
                          'network_id': net_id}}
 
         if security_groups is not None:
@@ -85,6 +98,9 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
         if device_owner is not None:
             data['port']['device_owner'] = device_owner
 
+        if vnic_type is not None:
+            data['port']['binding:vnic_type'] = vnic_type
+
         req = self.new_create_request('ports', data, self.fmt)
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['port']
@@ -131,7 +147,6 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
         data = {'subnet': {'name': name,
                            'network_id': net_id,
                            'ip_version': ip_version,
-                           'tenant_id': self._tenant_id,
                            'cidr': cidr,
                            'enable_dhcp': True}}
         data['subnet'].update(kwargs)
@@ -152,10 +167,13 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
                 return row
 
     def _create_router(self, name, external_gateway_info=None):
-        data = {'router': {'name': name, 'tenant_id': self._tenant_id}}
+        data = {'router': {'name': name}}
+        as_admin = False
         if external_gateway_info is not None:
             data['router']['external_gateway_info'] = external_gateway_info
-        req = self.new_create_request('routers', data, self.fmt)
+            as_admin = bool(external_gateway_info.get('enable_snat'))
+        req = self.new_create_request('routers', data, self.fmt,
+                                      as_admin=as_admin)
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['router']
 
@@ -173,7 +191,6 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
 
     def _create_security_group(self):
         data = {'security_group': {'name': 'sgtest',
-                                   'tenant_id': self._tenant_id,
                                    'description': 'SpongeBob Rocks!'}}
         req = self.new_create_request('security-groups', data, self.fmt)
         res = req.get_response(self.api)
@@ -189,8 +206,7 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
                                         'protocol': n_const.PROTO_NAME_TCP,
                                         'ethertype': n_const.IPv4,
                                         'port_range_min': 22,
-                                        'port_range_max': 22,
-                                        'tenant_id': self._tenant_id}}
+                                        'port_range_max': 22}}
         req = self.new_create_request('security-group-rules', data, self.fmt)
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['security_group_rule']
@@ -366,7 +382,8 @@ class TestMaintenance(_TestMaintenanceHelper):
         # Assert the revision number no longer exists
         self.assertIsNone(db_rev.get_revision_row(
             self.context,
-            neutron_obj['id']))
+            neutron_obj['id'],
+            resource_type=ovn_const.TYPE_PORTS))
 
     def test_subnet_global_dhcp4_opts(self):
         obj_name = 'globaltestsubnet'
@@ -778,8 +795,8 @@ class TestMaintenance(_TestMaintenanceHelper):
         p1 = self._create_port('testp1', net1['id'])
         logical_ip = p1['fixed_ips'][0]['ip_address']
         fip_info = {'floatingip': {
-            'description': 'test_fip',
             'tenant_id': self._tenant_id,
+            'description': 'test_fip',
             'floating_network_id': ext_net['id'],
             'port_id': p1['id'],
             'fixed_ip_address': logical_ip}}
@@ -956,6 +973,26 @@ class TestMaintenance(_TestMaintenanceHelper):
             # Assert load balancer for port forwarding is gone
             self.assertFalse(self._find_pf_lb(router_id, fip_id))
 
+    def test_check_for_ha_chassis_group(self):
+        net1 = self._create_network('network1test', external=False)
+        self._create_subnet('subnet1test', net1['id'])
+        p1 = self._create_port('testp1', net1['id'], vnic_type='direct')
+
+        # Remove the HA Chassis Group register, created during the port
+        # creation.
+        self.nb_api.set_lswitch_port(p1['id'], ha_chassis_group=[]).execute(
+            check_error=True)
+        hcg_uuid = next(iter(self.nb_api._tables['HA_Chassis_Group'].rows))
+        self.nb_api.ha_chassis_group_del(hcg_uuid).execute(check_error=True)
+        lsp = self.nb_api.lookup('Logical_Switch_Port', p1['id'])
+        self.assertEqual([], lsp.ha_chassis_group)
+
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_for_ha_chassis_group)
+        hcg_uuid = next(iter(self.nb_api._tables['HA_Chassis_Group'].rows))
+        lsp = self.nb_api.lookup('Logical_Switch_Port', p1['id'])
+        self.assertEqual(hcg_uuid, lsp.ha_chassis_group[0].uuid)
+
     def test_remove_duplicated_chassis_registers(self):
         hostnames = ['host1', 'host2']
         for hostname in hostnames:
@@ -1014,6 +1051,185 @@ class TestMaintenance(_TestMaintenanceHelper):
         # "Chassis_Private" register was missing.
         self.assertEqual(2, len(chassis_result))
 
+    def _test_check_provider_distributed_ports(
+            self, is_distributed_fip, net_type, expected_value=None):
+        cfg.CONF.set_override(
+            'enable_distributed_floating_ip', is_distributed_fip, group='ovn')
+        net_args = {'net_type': net_type}
+        if net_type == n_const.TYPE_FLAT:
+            net_args['provider'] = 'datacentre'
+        net = self._create_network(
+            'net_distributed_ports_test', **net_args)
+        subnet = self._create_subnet('subnet_distributed_ports_test',
+                                     net['id'])
+        router = self._create_router('router_distributed_ports_test')
+        self._add_router_interface(router['id'], subnet['id'])
+
+        # Lets make sure that reside-on-chassis-redirect is not set for the LRP
+        lr = self.nb_api.lookup('Logical_Router',
+                                utils.ovn_name(router['id']))
+        lrp = lr.ports[0]
+        self.nb_api.db_remove(
+            'Logical_Router_Port',
+            lrp.name,
+            'options',
+            ovn_const.LRP_OPTIONS_RESIDE_REDIR_CH
+        ).execute()
+
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_provider_distributed_ports)
+
+        lrp = self.nb_api.lookup('Logical_Router_Port', lrp.name)
+        if net_type in [n_const.TYPE_VLAN, n_const.TYPE_FLAT]:
+            self.assertEqual(
+                expected_value,
+                strutils.bool_from_string(
+                    lrp.options[ovn_const.LRP_OPTIONS_RESIDE_REDIR_CH]))
+        else:
+            self.assertNotIn(
+                ovn_const.LRP_OPTIONS_RESIDE_REDIR_CH,
+                lrp.options)
+
+    def test_check_provider_distributed_ports_dvr_vlan_net(self):
+        self._test_check_provider_distributed_ports(True, 'vlan', False)
+
+    def test_check_provider_distributed_ports_non_dvr_vlan_net(self):
+        self._test_check_provider_distributed_ports(False, 'vlan', True)
+
+    def test_check_provider_distributed_ports_dvr_flat_net(self):
+        self._test_check_provider_distributed_ports(True, 'flat', False)
+
+    def test_check_provider_distributed_ports_non_dvr_flat_net(self):
+        self._test_check_provider_distributed_ports(False, 'flat', True)
+
+    def test_check_provider_distributed_ports_dvr_geneve_net(self):
+        self._test_check_provider_distributed_ports(True, 'geneve')
+
+    def test_check_provider_distributed_ports_non_dvr_geneve_net(self):
+        self._test_check_provider_distributed_ports(False, 'geneve')
+
+    def test_floating_ip_with_gateway_port(self):
+        ext_net = self._create_network('ext_networktest', external=True)
+        ext_subnet = self._create_subnet(
+            'ext_subnettest',
+            ext_net['id'],
+            **{'cidr': '100.0.0.0/24',
+               'gateway_ip': '100.0.0.254',
+               'allocation_pools': [
+                   {'start': '100.0.0.2', 'end': '100.0.0.253'}],
+               'enable_dhcp': False})
+        net1 = self._create_network('network1test', external=False)
+        subnet1 = self._create_subnet('subnet1test', net1['id'])
+        external_gateway_info = {
+            'enable_snat': True,
+            'network_id': ext_net['id'],
+            'external_fixed_ips': [
+                {'ip_address': '100.0.0.2', 'subnet_id': ext_subnet['id']}]}
+        router = self._create_router(
+            'routertest', external_gateway_info=external_gateway_info)
+        self._add_router_interface(router['id'], subnet1['id'])
+
+        p1 = self._create_port('testp1', net1['id'])
+        logical_ip = p1['fixed_ips'][0]['ip_address']
+        fip_info = {'floatingip': {
+            'tenant_id': self._tenant_id,
+            'description': 'test_fip',
+            'floating_network_id': ext_net['id'],
+            'port_id': p1['id'],
+            'fixed_ip_address': logical_ip}}
+
+        # Create floating IP without gateway_port
+        with mock.patch.object(utils,
+                'is_nat_gateway_port_supported', return_value=False):
+            fip = self.l3_plugin.create_floatingip(self.context, fip_info)
+
+        self.assertEqual(router['id'], fip['router_id'])
+        self.assertEqual('testp1', fip['port_details']['name'])
+        self.assertIsNotNone(self.nb_api.get_lswitch_port(fip['port_id']))
+
+        rules = self.nb_api.get_all_logical_routers_with_rports()[0]
+        fip_rule = rules['dnat_and_snats'][0]
+        if utils.is_nat_gateway_port_supported(self.nb_api):
+            self.assertEqual([], fip_rule['gateway_port'])
+        else:
+            self.assertNotIn('gateway_port', fip_rule)
+
+        # Call the maintenance task and check that the value has been
+        # updated in the NAT rule
+        self.assertRaises(periodics.NeverAgain,
+            self.maint.update_nat_floating_ip_with_gateway_port_reference)
+
+        rules = self.nb_api.get_all_logical_routers_with_rports()[0]
+        fip_rule = rules['dnat_and_snats'][0]
+
+        if utils.is_nat_gateway_port_supported(self.nb_api):
+            self.assertNotEqual([], fip_rule['gateway_port'])
+        else:
+            self.assertNotIn('gateway_port', fip_rule)
+
+    def _get_nb_global_external_ids(self):
+        return self.nb_api.db_get(
+            'NB_Global', '.', 'external_ids').execute(check_error=True)
+
+    def test_set_fip_distributed_flag(self):
+        ovn_config.cfg.CONF.set_override(
+            'enable_distributed_floating_ip', True, 'ovn')
+        nb_global_ext_id = self._get_nb_global_external_ids()
+        self.assertNotIn(ovn_const.OVN_FIP_DISTRIBUTED_KEY, nb_global_ext_id)
+
+        self.assertRaises(
+            periodics.NeverAgain, self.maint.set_fip_distributed_flag)
+
+        nb_global_ext_id = self._get_nb_global_external_ids()
+        self.assertEqual(
+            "True", nb_global_ext_id[ovn_const.OVN_FIP_DISTRIBUTED_KEY])
+
+    def _test_set_fip_distributed_flag_change(
+            self, original_value, config_value):
+        ovn_config.cfg.CONF.set_override(
+            'enable_distributed_floating_ip', config_value, 'ovn')
+        self.nb_api.db_set(
+            'NB_Global', '.', external_ids={
+                ovn_const.OVN_FIP_DISTRIBUTED_KEY: str(original_value)}
+        ).execute(check_error=True)
+        nb_global_ext_id = self._get_nb_global_external_ids()
+        self.assertEqual(
+            str(original_value),
+            nb_global_ext_id[ovn_const.OVN_FIP_DISTRIBUTED_KEY])
+
+        self.assertRaises(
+            periodics.NeverAgain, self.maint.set_fip_distributed_flag)
+
+        nb_global_ext_id = self._get_nb_global_external_ids()
+        self.assertEqual(
+            str(config_value),
+            nb_global_ext_id[ovn_const.OVN_FIP_DISTRIBUTED_KEY])
+
+    def test_set_fip_distributed_flag_changed(self):
+        self._test_set_fip_distributed_flag_change(
+            original_value=False,
+            config_value=True)
+
+    def test_set_fip_distributed_flag_unchanged(self):
+        self._test_set_fip_distributed_flag_change(
+            original_value=True,
+            config_value=True)
+
+    def test_set_network_type(self):
+        net1 = self._create_network(uuidutils.generate_uuid())
+        ls_name = utils.ovn_name(net1['id'])
+        self.nb_api.db_remove(
+            'Logical_Switch', ls_name, 'external_ids',
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY).execute(check_error=True)
+        ls = self.nb_api.lookup('Logical_Switch', ls_name)
+        self.assertIsNone(ls.external_ids.get(
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY))
+
+        self.assertRaises(periodics.NeverAgain, self.maint.set_network_type)
+        ls = self.nb_api.lookup('Logical_Switch', ls_name)
+        self.assertEqual(net1[provnet_apidef.NETWORK_TYPE],
+                         ls.external_ids.get(ovn_const.OVN_NETTYPE_EXT_ID_KEY))
+
 
 class TestLogMaintenance(_TestMaintenanceHelper,
                          test_log_driver.LogApiTestCaseBase):
