@@ -27,6 +27,7 @@ from neutron_lib.services.qos import constants as qos_const
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import constants as ovsdbapp_const
+from sqlalchemy.dialects.mysql import dialect as mysql_dialect
 
 from neutron.common.ovn import acl as acl_utils
 from neutron.common.ovn import constants as ovn_const
@@ -43,15 +44,18 @@ from neutron.tests.functional import base
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.extensions import test_extraroute
 from neutron.tests.unit.extensions import test_securitygroup
+from neutron.tests.unit import testlib_api
 
 
-class TestOvnNbSync(base.TestOVNFunctionalBase):
+class TestOvnNbSync(testlib_api.MySQLTestCaseMixin,
+                    base.TestOVNFunctionalBase):
 
     _extension_drivers = ['port_security', 'dns', 'qos', 'revision_plugin']
 
     def setUp(self, *args):
-        ovn_config.cfg.CONF.set_override('dns_domain', 'ovn.test')
         super(TestOvnNbSync, self).setUp(maintenance_worker=True)
+        self.assertEqual(mysql_dialect.name, self.db.engine.dialect.name)
+        ovn_config.cfg.CONF.set_override('dns_domain', 'ovn.test')
         ext_mgr = test_extraroute.ExtraRouteTestExtensionManager()
         self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)
         sg_mgr = test_securitygroup.SecurityGroupTestExtensionManager()
@@ -365,7 +369,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
             port_req.get_response(self.api)
 
         # External network and subnet
-        e1 = self._make_network(self.fmt, 'e1', True,
+        e1 = self._make_network(self.fmt, 'e1', True, as_admin=True,
                                 arg_list=('router:external',
                                           'provider:network_type',
                                           'provider:physical_network'),
@@ -963,8 +967,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         db_metadata_ports_ids = set()
         db_metadata_ports_nets = set()
         for port in db_ports['ports']:
-            if (port['device_owner'] == constants.DEVICE_OWNER_DISTRIBUTED and
-                    port['device_id'].startswith('ovnmeta')):
+            if utils.is_ovn_metadata_port(port):
                 db_metadata_ports_ids.add(port['id'])
                 db_metadata_ports_nets.add(port['network_id'])
         db_networks = self._list('networks')
@@ -1474,7 +1477,6 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         db_pgs = []
         for sg in self._list('security-groups')['security_groups']:
             db_pgs.append(utils.ovn_port_group_name(sg['id']))
-        db_pgs.append(ovn_const.OVN_DROP_PORT_GROUP_NAME)
 
         nb_pgs = _plugin_nb_ovn.get_sg_port_groups()
 
@@ -1484,7 +1486,8 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
 
         if should_match:
             self.assertCountEqual(nb_pgs, db_pgs)
-            self.assertCountEqual(mn_pgs, db_pgs)
+            # pg_drop port group doesn't have corresponding neutron sg
+            self.assertEqual(len(mn_pgs), len(db_pgs) + 1)
         else:
             self.assertRaises(AssertionError, self.assertCountEqual,
                               nb_pgs, db_pgs)
@@ -1501,9 +1504,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         """
         db_ports = self._list('ports')
         db_metadata_ports = [port for port in db_ports['ports'] if
-                             port['device_owner'] ==
-                             constants.DEVICE_OWNER_DISTRIBUTED and
-                             port['device_id'].startswith('ovnmeta')]
+                             utils.is_ovn_metadata_port(port)]
         lswitches = {}
         ports_to_delete = len(db_metadata_ports) / 2
         for port in db_metadata_ports:
@@ -1608,20 +1609,23 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         net = self.deserialize(self.fmt, res)['network']
         self._create_subnet(self.fmt, net['id'], '10.0.0.0/24')
 
-        res = self._create_qos_policy(self.fmt, 'qos_maxbw')
+        res = self._create_qos_policy(self.fmt, 'qos_maxbw', is_admin=True)
         qos_maxbw = self.deserialize(self.fmt, res)['policy']
         self._create_qos_rule(self.fmt, qos_maxbw['id'],
                               qos_const.RULE_TYPE_BANDWIDTH_LIMIT,
-                              max_kbps=1000, max_burst_kbps=800)
+                              max_kbps=1000, max_burst_kbps=800,
+                              is_admin=True)
         self._create_qos_rule(self.fmt, qos_maxbw['id'],
                               qos_const.RULE_TYPE_BANDWIDTH_LIMIT,
                               direction=constants.INGRESS_DIRECTION,
-                              max_kbps=700, max_burst_kbps=600)
+                              max_kbps=700, max_burst_kbps=600,
+                              is_admin=True)
 
-        res = self._create_qos_policy(self.fmt, 'qos_maxbw')
+        res = self._create_qos_policy(self.fmt, 'qos_maxbw', is_admin=True)
         qos_dscp = self.deserialize(self.fmt, res)['policy']
         self._create_qos_rule(self.fmt, qos_dscp['id'],
-                              qos_const.RULE_TYPE_DSCP_MARKING, dscp_mark=14)
+                              qos_const.RULE_TYPE_DSCP_MARKING, dscp_mark=14,
+                              is_admin=True)
 
         res = self._create_port(
             self.fmt, net['id'], arg_list=('qos_policy_id', ),
@@ -1677,7 +1681,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
                                                 {'floatingip': body})
 
     def test_sync_fip_qos_policies(self):
-        res = self._create_network(self.fmt, 'n1_ext', True,
+        res = self._create_network(self.fmt, 'n1_ext', True, as_admin=True,
                                    arg_list=('router:external', ),
                                    **{'router:external': True})
         net_ext = self.deserialize(self.fmt, res)['network']
@@ -1687,15 +1691,17 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         net_int = self.deserialize(self.fmt, res)['network']
         self._create_subnet(self.fmt, net_int['id'], '10.10.0.0/24')
 
-        res = self._create_qos_policy(self.fmt, 'qos_maxbw')
+        res = self._create_qos_policy(self.fmt, 'qos_maxbw', is_admin=True)
         qos_maxbw = self.deserialize(self.fmt, res)['policy']
         self._create_qos_rule(self.fmt, qos_maxbw['id'],
                               qos_const.RULE_TYPE_BANDWIDTH_LIMIT,
-                              max_kbps=1000, max_burst_kbps=800)
+                              max_kbps=1000, max_burst_kbps=800,
+                              is_admin=True)
         self._create_qos_rule(self.fmt, qos_maxbw['id'],
                               qos_const.RULE_TYPE_BANDWIDTH_LIMIT,
                               direction=constants.INGRESS_DIRECTION,
-                              max_kbps=700, max_burst_kbps=600)
+                              max_kbps=700, max_burst_kbps=600,
+                              is_admin=True)
 
         # Create a router with net_ext as GW network and net_int as internal
         # one, and a floating IP on the external network.
@@ -1750,7 +1756,7 @@ class TestOvnNbSync(base.TestOVNFunctionalBase):
         self._validate_qos_records()
 
     def test_fip_nat_revert_to_stateful(self):
-        res = self._create_network(self.fmt, 'n1_ext', True,
+        res = self._create_network(self.fmt, 'n1_ext', True, as_admin=True,
                                    arg_list=('router:external', ),
                                    **{'router:external': True})
         net_ext = self.deserialize(self.fmt, res)['network']
@@ -1814,6 +1820,7 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
             self.plugin, self.mech_driver.sb_ovn, self.mech_driver)
         self.addCleanup(self.sb_synchronizer.stop)
         self.ctx = context.get_admin_context()
+        self.host1 = uuidutils.generate_uuid()
 
     def _sync_resources(self):
         self.sb_synchronizer.sync_hostname_and_physical_networks(self.ctx)
@@ -1847,23 +1854,23 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
     def test_ovn_sb_sync_add_new_host(self):
         with self.network() as network:
             network_id = network['network']['id']
-        self.create_segment(network_id, 'physnet1', 50)
-        self.add_fake_chassis('host1', ['physnet1'])
+        self.create_segment(network_id, self.physnet, 50)
+        self.add_fake_chassis(self.host1, [self.physnet])
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
         self.assertFalse(segment_hosts)
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({'host1'}, segment_hosts)
+        self.assertEqual({self.host1}, segment_hosts)
 
     def test_ovn_sb_sync_update_existing_host(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, 'physnet1', 50)
+        segment = self.create_segment(network_id, self.physnet, 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, 'host1', {segment['id']})
+            self.ctx, self.host1, {segment['id']})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({'host1'}, segment_hosts)
-        self.add_fake_chassis('host1', ['physnet2'])
+        self.assertEqual({self.host1}, segment_hosts)
+        self.add_fake_chassis(self.host1, [self.physnet2])
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
         self.assertFalse(segment_hosts)
@@ -1871,13 +1878,14 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
     def test_ovn_sb_sync_delete_stale_host(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, 'physnet1', 50)
+        segment = self.create_segment(network_id, self.physnet, 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, 'host1', {segment['id']})
-        _ = self.create_agent('host1', bridge_mappings={'physnet1': 'eth0'})
+            self.ctx, self.host1, {segment['id']})
+        _ = self.create_agent(self.host1,
+                              bridge_mappings={self.physnet: 'eth0'})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({'host1'}, segment_hosts)
-        # Since there is no chassis in the sb DB, host1 is the stale host
+        self.assertEqual({self.host1}, segment_hosts)
+        # Since there is no chassis in the sb DB, self.host1 is the stale host
         # recorded in neutron DB. It should be deleted after sync.
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
@@ -1886,57 +1894,61 @@ class TestOvnSbSync(base.TestOVNFunctionalBase):
     def test_ovn_sb_sync_host_with_no_agent_not_deleted(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, 'physnet1', 50)
+        segment = self.create_segment(network_id, self.physnet, 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, 'host1', {segment['id']})
-        _ = self.create_agent('host1', bridge_mappings={'physnet1': 'eth0'},
+            self.ctx, self.host1, {segment['id']})
+        _ = self.create_agent(self.host1,
+                              bridge_mappings={self.physnet: 'eth0'},
                               agent_type="Not OVN Agent")
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({'host1'}, segment_hosts)
-        # There is no chassis in the sb DB, host1 does not have an agent
+        self.assertEqual({self.host1}, segment_hosts)
+        # There is no chassis in the sb DB, self.host1 does not have an agent
         # so it is not deleted.
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({'host1'}, segment_hosts)
+        self.assertEqual({self.host1}, segment_hosts)
 
     def test_ovn_sb_sync_host_with_other_agent_type_not_deleted(self):
         with self.network() as network:
             network_id = network['network']['id']
-        segment = self.create_segment(network_id, 'physnet1', 50)
+        segment = self.create_segment(network_id, self.physnet, 50)
         segments_db.update_segment_host_mapping(
-            self.ctx, 'host1', {segment['id']})
+            self.ctx, self.host1, {segment['id']})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({'host1'}, segment_hosts)
-        # There is no chassis in the sb DB, host1 does not have an agent
+        self.assertEqual({self.host1}, segment_hosts)
+        # There is no chassis in the sb DB, self.host1 does not have an agent
         # so it is not deleted.
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        self.assertEqual({'host1'}, segment_hosts)
+        self.assertEqual({self.host1}, segment_hosts)
 
     def test_ovn_sb_sync(self):
+        host2 = uuidutils.generate_uuid()
+        host3 = uuidutils.generate_uuid()
+        host4 = uuidutils.generate_uuid()
         with self.network() as network:
             network_id = network['network']['id']
-        seg1 = self.create_segment(network_id, 'physnet1', 50)
-        self.create_segment(network_id, 'physnet2', 51)
+        seg1 = self.create_segment(network_id, self.physnet, 50)
+        self.create_segment(network_id, self.physnet2, 51)
         segments_db.update_segment_host_mapping(
-            self.ctx, 'host1', {seg1['id']})
+            self.ctx, self.host1, {seg1['id']})
         segments_db.update_segment_host_mapping(
-            self.ctx, 'host2', {seg1['id']})
+            self.ctx, host2, {seg1['id']})
         segments_db.update_segment_host_mapping(
-            self.ctx, 'host3', {seg1['id']})
+            self.ctx, host3, {seg1['id']})
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
-        _ = self.create_agent('host1')
-        _ = self.create_agent('host2', bridge_mappings={'physnet2': 'eth0'})
-        _ = self.create_agent('host3', bridge_mappings={'physnet3': 'eth0'})
-        self.assertEqual({'host1', 'host2', 'host3'}, segment_hosts)
-        self.add_fake_chassis('host2', ['physnet2'])
-        self.add_fake_chassis('host3', ['physnet3'])
-        self.add_fake_chassis('host4', ['physnet1'])
+        _ = self.create_agent(self.host1)
+        _ = self.create_agent(host2, bridge_mappings={self.physnet2: 'eth0'})
+        _ = self.create_agent(host3, bridge_mappings={self.physnet3: 'eth0'})
+        self.assertEqual({self.host1, host2, host3}, segment_hosts)
+        self.add_fake_chassis(host2, [self.physnet2])
+        self.add_fake_chassis(host3, [self.physnet3])
+        self.add_fake_chassis(host4, [self.physnet])
         self._sync_resources()
         segment_hosts = segments_db.get_hosts_mapped_with_segments(self.ctx)
         # host1 should be cleared since it is not in the chassis DB. host3
         # should be cleared since there is no segment for mapping.
-        self.assertEqual({'host2', 'host4'}, segment_hosts)
+        self.assertEqual({host2, host4}, segment_hosts)
 
 
 class TestOvnNbSyncOverTcp(TestOvnNbSync):
