@@ -27,6 +27,7 @@ from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event
 from ovsdbapp.backend.ovs_idl import idlutils
+from sqlalchemy.dialects.mysql import dialect as mysql_dialect
 import tenacity
 
 from neutron.common.ovn import constants as ovn_const
@@ -42,6 +43,7 @@ from neutron.tests.functional.resources.ovsdb import fixtures
 from neutron.tests.functional.resources import process
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.extensions import test_l3
+from neutron.tests.unit import testlib_api
 
 
 class WaitForDataPathBindingCreateEvent(event.WaitEvent):
@@ -84,10 +86,12 @@ class GlobalTestEvent(DistributedLockTestEvent):
     GLOBAL = True
 
 
-class TestNBDbMonitor(base.TestOVNFunctionalBase):
+class TestNBDbMonitor(testlib_api.MySQLTestCaseMixin,
+                      base.TestOVNFunctionalBase):
 
     def setUp(self):
         super(TestNBDbMonitor, self).setUp()
+        self.assertEqual(mysql_dialect.name, self.db.engine.dialect.name)
         self.chassis = self.add_fake_chassis('ovs-host1')
         self.l3_plugin = directory.get_plugin(plugin_constants.L3)
         self.net = self._make_network(self.fmt, 'net1', True)
@@ -105,12 +109,13 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
                     allowedaddresspairs.ADDRESS_PAIRS: allowed_address_pairs
                     }
         port_res = self._create_port(self.fmt, self.net['network']['id'],
+                                     is_admin=True,
                                      arg_list=arg_list, **host_arg)
         port = self.deserialize(self.fmt, port_res)['port']
         return port
 
     def _create_fip(self, port, fip_address):
-        e1 = self._make_network(self.fmt, 'e1', True,
+        e1 = self._make_network(self.fmt, 'e1', True, as_admin=True,
                                 arg_list=('router:external',
                                           'provider:network_type',
                                           'provider:physical_network'),
@@ -323,13 +328,17 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         pb_port_vip = self.sb_api.db_find_rows(
             'Port_Binding', ('logical_port', '=', port_id)).execute(
             check_error=True)[0]
+        pb_virtual_parent = str(pb_port_parent.uuid)
         self.sb_api.db_set(
             'Port_Binding', pb_port_vip.uuid,
-            ('virtual_parent', pb_port_parent.uuid)).execute(check_error=True)
+            ('chassis', pb_port_parent.chassis),
+            ('virtual_parent', pb_virtual_parent)).execute(check_error=True)
 
     def _check_port_binding_type(self, port_id, port_type):
         def is_port_binding_type(port_id, port_type):
             bp = self._find_port_binding(port_id)
+            if not bp:
+                return False
             return port_type == bp.type
 
         check = functools.partial(is_port_binding_type, port_id, port_type)
@@ -338,8 +347,19 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
     def _check_port_virtual_parents(self, port_id, vparents):
         def is_port_virtual_parents(port_id, vparents):
             bp = self._find_port_binding(port_id)
-            return (vparents ==
-                    bp.options.get(ovn_const.LSP_OPTIONS_VIRTUAL_PARENTS_KEY))
+            vp = bp.options.get(ovn_const.LSP_OPTIONS_VIRTUAL_PARENTS_KEY)
+
+            # If the given vparents is None, or if the current value is None
+            # Then just do a check on that, no need to split strings if it
+            # is not required
+            if None in (vp, vparents):
+                return vp == vparents
+
+            # Since the virtual parents is a string, representing a list of
+            # ports, we should make a set() and compare sets
+            bp_set = {p for p in vp.split(',')}
+            vp_set = {p for p in vparents.split(',')}
+            return bp_set == vp_set
 
         check = functools.partial(is_port_virtual_parents, port_id, vparents)
         n_utils.wait_until_true(check, timeout=10)
@@ -417,6 +437,106 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         self.assertRaises(n_utils.WaitTimeout, n_utils.wait_until_true,
                           lambda: mock_update_vip_host.called, timeout=5)
 
+    def _check_port_host_set(self, port_id, host_id):
+        # This function checks if given host_id matches the values in the
+        # neutron DB as well as in the OVN DB for the port with given port_id
+        core_plugin = directory.get_plugin()
+
+        # Get port from neutron DB
+        port = core_plugin.get_ports(
+            self.context, filters={'id': [port_id]})[0]
+
+        # Get port from OVN DB
+        bp = self._find_port_binding(port_id)
+        ovn_host_id = bp.external_ids.get(ovn_const.OVN_HOST_ID_EXT_ID_KEY)
+
+        # Check that both neutron and ovn are the same as given host_id
+        return port[portbindings.HOST_ID] == host_id == ovn_host_id
+
+    def _check_port_and_port_binding_revision_number(self, port_id):
+
+        def is_port_and_port_binding_same_revision_number(port_id):
+            # This function checks if given port matches the revision_number
+            # in the neutron DB as well as in the OVN DB for the port_binding
+            core_plugin = directory.get_plugin()
+
+            # Get port from neutron DB
+            port = core_plugin.get_ports(
+                self.context, filters={'id': [port_id]})[0]
+
+            # Get port binding from OVN DB
+            bp = self._find_port_binding(port_id)
+            ovn_port_binding_revision_number = bp.external_ids.get(
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY, ovn_const.INITIAL_REV_NUM)
+
+            # Check that both neutron and ovn are the same as given host_id
+            return port['revision_number'] == int(
+                ovn_port_binding_revision_number)
+
+        check = functools.partial(
+            is_port_and_port_binding_same_revision_number, port_id)
+        n_utils.wait_until_true(check, timeout=10)
+
+    def test_virtual_port_host_update_upon_failover(self):
+        # NOTE: we can't simulate traffic, but we can simulate the event that
+        # would've been triggered by OVN, which is what we do.
+
+        # The test is based to test_virtual_port_host_update, though in this
+        # test we actually test the OVNMechanismDriver.update_virtual_port_host
+        # method, that updates the hostname in neutron and OVN
+        # We do not extensively check if the allowed-address-pair is being kept
+        # up-to-date, since test_virtual_port_host_update does this already.
+
+        # 1) Setup a second chassis
+        second_chassis_name = 'ovs-host2'
+        second_chassis = self.add_fake_chassis(second_chassis_name)
+
+        # 2) Create port with the VIP for allowed address pair setup
+        vip = self.create_port(device_owner='', host='')
+        vip_address = vip['fixed_ips'][0]['ip_address']
+        allowed_address_pairs = [{'ip_address': vip_address}]
+        self._check_port_binding_type(vip['id'], '')
+        self._check_port_and_port_binding_revision_number(vip['id'])
+
+        # 3) Create two ports with the allowed address pairs set.
+        hosts = ('ovs-host1', second_chassis_name)
+        ports = []
+        for idx in range(len(hosts)):
+            ports.append(self.create_port(host=hosts[idx]))
+            data = {'port': {'allowed_address_pairs': allowed_address_pairs}}
+            req = self.new_update_request('ports', data, ports[idx]['id'])
+            req.get_response(self.api)
+
+        port_ids = [p['id'] for p in ports]
+
+        # 4) Check that the vip port has become virtual and that both parents
+        # have been assigned to the port binding
+        self._check_port_binding_type(vip['id'], ovn_const.LSP_TYPE_VIRTUAL)
+        self._check_port_virtual_parents(vip['id'], ','.join(port_ids))
+        self._check_port_and_port_binding_revision_number(vip['id'])
+
+        # 5) Bind the ports to a host, so a chassis is bound, which is
+        # required for the update_virtual_port_host method. Without this
+        # chassis set, it will not set a hostname in the DB's
+        self._test_port_binding_and_status(ports[0]['id'], 'bind', 'ACTIVE')
+        self.chassis = second_chassis
+        self._test_port_binding_and_status(ports[1]['id'], 'bind', 'ACTIVE')
+        self._check_port_and_port_binding_revision_number(vip['id'])
+
+        # 6) For both ports, bind vip on parent and check hostname in DBs
+        for idx in range(len(ports)):
+            # Set port binding to the first port, and update the chassis
+            self._set_port_binding_virtual_parent(vip['id'], ports[idx]['id'])
+            self._check_port_and_port_binding_revision_number(vip['id'])
+
+            # Check if the host_id has been updated in OVN and DB
+            # by the event that eventually calls for method
+            # OVNMechanismDriver.update_virtual_port_host
+            n_utils.wait_until_true(
+                lambda: self._check_port_host_set(vip['id'], hosts[idx]),
+                timeout=10)
+            self._check_port_and_port_binding_revision_number(vip['id'])
+
 
 class TestNBDbMonitorOverTcp(TestNBDbMonitor):
     def get_ovsdb_server_protocol(self):
@@ -462,7 +582,8 @@ class TestSBDbMonitor(base.TestOVNFunctionalBase, test_l3.L3NatTestCaseMixin):
 
         kwargs = {'arg_list': (external_net.EXTERNAL,),
                   external_net.EXTERNAL: True}
-        ext_net = self._make_network(self.fmt, 'ext_net', True, **kwargs)
+        ext_net = self._make_network(self.fmt, 'ext_net', True, as_admin=True,
+                                     **kwargs)
         self._make_subnet(self.fmt, ext_net, '10.251.0.1', '10.251.0.0/24',
                           enable_dhcp=True)
         router = self._make_router(self.fmt, self._tenant_id)
@@ -508,18 +629,18 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
 
     def test_agent_change_controller(self):
         self.assertEqual(neutron_agent.ControllerGatewayAgent,
-                type(neutron_agent.AgentCache()[self.chassis_name]))
+                type(neutron_agent.AgentCache().get(self.chassis_name)))
         self.sb_api.db_set('Chassis', self.chassis_name, ('other_config',
                 {'ovn-cms-options': ''})).execute(check_error=True)
         n_utils.wait_until_true(lambda:
-                neutron_agent.AgentCache()[self.chassis_name].
+                neutron_agent.AgentCache().get(self.chassis_name).
                 chassis.other_config['ovn-cms-options'] == '')
         self.assertEqual(neutron_agent.ControllerAgent,
-                type(neutron_agent.AgentCache()[self.chassis_name]))
+                type(neutron_agent.AgentCache().get(self.chassis_name)))
 
     def test_agent_updated_at_use_nb_cfg_timestamp(self):
         def check_agent_ts():
-            agent = neutron_agent.AgentCache()[self.chassis_name]
+            agent = neutron_agent.AgentCache().get(self.chassis_name)
             chassis_ts = self.sb_api.db_get(
                 'Chassis_Private', self.chassis_name,
                 'nb_cfg_timestamp').execute(check_error=True)
@@ -542,7 +663,7 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
         try:
             n_utils.wait_until_true(check_agent_ts, timeout=5)
         except n_utils.WaitTimeout:
-            agent = neutron_agent.AgentCache()[self.chassis_name]
+            agent = neutron_agent.AgentCache().get(self.chassis_name)
             chassis_ts = self.sb_api.db_get(
                 'Chassis_Private', self.chassis_name,
                 'nb_cfg_timestamp').execute(check_error=True)
@@ -551,14 +672,14 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
 
     def test_agent_restart(self):
         def check_agent_up():
-            agent = neutron_agent.AgentCache()[self.chassis_name]
+            agent = neutron_agent.AgentCache().get(self.chassis_name)
             return agent.alive
 
         def check_agent_down():
             return not check_agent_up()
 
         def check_nb_cfg_timestamp_is_not_null():
-            agent = neutron_agent.AgentCache()[self.chassis_name]
+            agent = neutron_agent.AgentCache().get(self.chassis_name)
             return agent.updated_at != 0
 
         if not self.sb_api.is_table_present('Chassis_Private'):
