@@ -27,25 +27,24 @@ from neutron_lib.exceptions import agent as agent_exc
 from oslo_config import cfg
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event
-from ovsdbapp.tests.functional import base as ovs_base
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_revision_numbers_db as db_rev
-from neutron.plugins.ml2.drivers.ovn.mech_driver import mech_driver
-from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
-from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.tests import base as tests_base
 from neutron.tests.functional import base
+from neutron.tests.unit.extensions import test_securitygroup as test_sg
 
 VHU_MODE = 'server'
 OVS_VIF_DETAILS = {
     portbindings.CAP_PORT_FILTER: True,
     portbindings.VIF_DETAILS_CONNECTIVITY: portbindings.CONNECTIVITY_L2,
     portbindings.VIF_DETAILS_BOUND_DRIVERS: {'0': 'ovn'},
+    portbindings.VIF_DETAILS_BRIDGE_NAME: 'br-int',
+    portbindings.OVS_DATAPATH_TYPE: 'system',
 }
 VHOSTUSER_VIF_DETAILS = {
     portbindings.CAP_PORT_FILTER: False,
@@ -53,34 +52,39 @@ VHOSTUSER_VIF_DETAILS = {
     'vhostuser_ovs_plug': True,
     portbindings.VIF_DETAILS_CONNECTIVITY: portbindings.CONNECTIVITY_L2,
     portbindings.VIF_DETAILS_BOUND_DRIVERS: {'0': 'ovn'},
+    portbindings.VIF_DETAILS_BRIDGE_NAME: 'br-int',
+    portbindings.OVS_DATAPATH_TYPE: 'netdev',
 }
 
 
 class TestPortBinding(base.TestOVNFunctionalBase):
 
-    def setUp(self):
-        super(TestPortBinding, self).setUp()
+    def setUp(self, **kwargs):
+        super().setUp(**kwargs)
         self.ovs_host = 'ovs-host'
         self.dpdk_host = 'dpdk-host'
         self.invalid_dpdk_host = 'invalid-host'
         self.insecure_host = 'insecure-host'
         self.smartnic_dpu_host = 'smartnic-dpu-host'
         self.smartnic_dpu_serial = 'fake-smartnic-dpu-serial'
-        self.add_fake_chassis(self.ovs_host)
+        self.add_fake_chassis(
+            self.ovs_host,
+            other_config={ovn_const.OVN_DATAPATH_TYPE: 'system'})
         self.add_fake_chassis(
             self.dpdk_host,
-            external_ids={'datapath-type': 'netdev',
+            other_config={ovn_const.OVN_DATAPATH_TYPE: 'netdev',
                           'iface-types': 'dummy,dummy-internal,dpdkvhostuser'})
 
         self.add_fake_chassis(
             self.invalid_dpdk_host,
-            external_ids={'datapath-type': 'netdev',
+            other_config={ovn_const.OVN_DATAPATH_TYPE: 'netdev',
                           'iface-types': 'dummy,dummy-internal,geneve,vxlan'})
         self.add_fake_chassis(
             self.smartnic_dpu_host,
-            external_ids={ovn_const.OVN_CMS_OPTIONS: '{}={}'.format(
+            other_config={ovn_const.OVN_CMS_OPTIONS: '{}={}'.format(
                 ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
-                self.smartnic_dpu_serial)})
+                self.smartnic_dpu_serial),
+                ovn_const.OVN_DATAPATH_TYPE: 'system'})
         self.n1 = self._make_network(self.fmt, 'n1', True)
         res = self._create_subnet(self.fmt, self.n1['network']['id'],
                                   '10.0.0.0/24')
@@ -104,20 +108,21 @@ class TestPortBinding(base.TestOVNFunctionalBase):
                 'network_id': self.n1['network']['id'],
                 'tenant_id': self._tenant_id})
 
-            port_req = self.new_create_request('ports', port_data, self.fmt)
+            port_req = self.new_create_request('ports', port_data, self.fmt,
+                                               as_admin=True)
             port_res = port_req.get_response(self.api)
             p = self.deserialize(self.fmt, port_res)
             port_id = p['port']['id']
         else:
             port_req = self.new_update_request('ports', port_data, port_id,
-                                               self.fmt)
+                                               self.fmt, as_admin=True)
             port_res = port_req.get_response(self.api)
             self.deserialize(self.fmt, port_res)
 
         return port_id
 
     def _port_show(self, port_id):
-        port_req = self.new_show_request('ports', port_id)
+        port_req = self.new_show_request('ports', port_id, as_admin=True)
         port_res = port_req.get_response(self.api)
         return self.deserialize(self.fmt, port_res)
 
@@ -154,9 +159,13 @@ class TestPortBinding(base.TestOVNFunctionalBase):
         self._verify_vif_details(port_id, self.dpdk_host, 'vhostuser',
                                  expected_vif_details)
 
+        expected_vif_details = copy.deepcopy(VHOSTUSER_VIF_DETAILS)
+        expected_vif_details.pop('vhostuser_mode')
+        expected_vif_details.pop('vhostuser_ovs_plug')
+        expected_vif_details[portbindings.CAP_PORT_FILTER] = True
         port_id = self._create_or_update_port(hostname=self.invalid_dpdk_host)
         self._verify_vif_details(port_id, self.invalid_dpdk_host, 'ovs',
-                                 OVS_VIF_DETAILS)
+                                 expected_vif_details)
 
     def test_port_binding_create_remote_managed_port(self):
         pci_vendor_info = 'fake-pci-vendor-info'
@@ -208,8 +217,12 @@ class TestPortBinding(base.TestOVNFunctionalBase):
 
         port_id = self._create_or_update_port(port_id=port_id,
                                               hostname=self.invalid_dpdk_host)
+        expected_vif_details = copy.deepcopy(VHOSTUSER_VIF_DETAILS)
+        expected_vif_details.pop('vhostuser_mode')
+        expected_vif_details.pop('vhostuser_ovs_plug')
+        expected_vif_details[portbindings.CAP_PORT_FILTER] = True
         self._verify_vif_details(port_id, self.invalid_dpdk_host, 'ovs',
-                                 OVS_VIF_DETAILS)
+                                 expected_vif_details)
 
     def test_port_binding_update_remote_managed_port(self):
         port_id = self._create_or_update_port(
@@ -718,13 +731,13 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
     def _test_external_port_create_switchdev(self, vnic_type):
         port_data = {
             'port': {'network_id': self.n1['network']['id'],
-                     'tenant_id': self._tenant_id,
                      portbindings.VNIC_TYPE: vnic_type,
                      ovn_const.OVN_PORT_BINDING_PROFILE: {
                          ovn_const.PORT_CAP_PARAM: [
                              ovn_const.PORT_CAP_SWITCHDEV]}}}
 
-        port_req = self.new_create_request('ports', port_data, self.fmt)
+        port_req = self.new_create_request('ports', port_data, self.fmt,
+                                           as_admin=True)
         port_res = port_req.get_response(self.api)
         port = self.deserialize(self.fmt, port_res)['port']
 
@@ -772,7 +785,8 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
                      ovn_const.PORT_CAP_PARAM: [
                          ovn_const.PORT_CAP_SWITCHDEV]}}}
         port_req = self.new_update_request(
-            'ports', port_upt_data, port['id'], self.fmt)
+            'ports', port_upt_data, port['id'], self.fmt,
+            as_admin=True)
         port_res = port_req.get_response(self.api)
         port = self.deserialize(self.fmt, port_res)['port']
 
@@ -793,58 +807,39 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
     def test_external_port_update_switchdev_vnic_macvtap(self):
         self._test_external_port_update_switchdev(portbindings.VNIC_MACVTAP)
 
+    def test_external_port_network_update(self):
+        net_id = self.n1['network']['id']
+        port_data = {
+            'port': {'network_id': net_id,
+                     'tenant_id': self._tenant_id,
+                     portbindings.VNIC_TYPE: 'direct'}}
 
-class TestCreateDefaultDropPortGroup(base.BaseLoggingTestCase,
-                                     ovs_base.FunctionalTestCase):
-    schemas = ['OVN_Southbound', 'OVN_Northbound']
-    PG_NAME = ovn_const.OVN_DROP_PORT_GROUP_NAME
-
-    def setUp(self):
-        super(TestCreateDefaultDropPortGroup, self).setUp()
-        ovn_conf.register_opts()
-        self.api = impl_idl_ovn.OvsdbNbOvnIdl(
-            self.connection['OVN_Northbound'])
-        self.addCleanup(self.api.pg_del(self.PG_NAME, if_exists=True).execute,
-                        check_error=True)
-
-    def test_port_group_exists(self):
-        """Test new port group is not added or modified.
-
-        If Port Group was not existent, acls would be added.
-        """
-        self.api.pg_add(
-            self.PG_NAME, acls=[], may_exist=True).execute(check_error=True)
-        mech_driver.create_default_drop_port_group(self.api)
-        port_group = self.api.get_port_group(self.PG_NAME)
-        self.assertFalse(port_group.acls)
-
-    def _test_pg_with_ports(self, expected_ports=None):
-        expected_ports = expected_ports or []
-        mech_driver.create_default_drop_port_group(self.api)
-        port_group = self.api.get_port_group(self.PG_NAME)
-        self.assertCountEqual(
-            expected_ports, [port.name for port in port_group.ports])
-
-    def test_with_ports_available(self):
-        expected_ports = ['port1', 'port2']
-        testing_pg = 'testing'
-        testing_ls = 'testing'
-        with self.api.transaction(check_error=True) as txn:
-            txn.add(self.api.pg_add(
-                testing_pg,
-                external_ids={ovn_const.OVN_SG_EXT_ID_KEY: 'foo'}))
-            txn.add(self.api.ls_add(testing_ls))
-            port_uuids = [txn.add(self.api.lsp_add(testing_ls, port))
-                          for port in expected_ports]
-            txn.add(self.api.pg_add_ports(testing_pg, port_uuids))
-
-        self.addCleanup(self.api.pg_del(testing_pg, if_exists=True).execute,
-                        check_error=True)
-
-        self._test_pg_with_ports(expected_ports)
-
-    def test_without_ports(self):
-        self._test_pg_with_ports(expected_ports=[])
+        # Create external port
+        port_req = self.new_create_request('ports', port_data, self.fmt)
+        port_res = port_req.get_response(self.api)
+        port = self.deserialize(self.fmt, port_res)['port']
+        ovn_port = self._find_port_row_by_name(port['id'])
+        self.assertEqual(ovn_const.LSP_TYPE_EXTERNAL, ovn_port.type)
+        # Update MTU of network with external port
+        mtu_value = self.n1['network']['mtu'] - 100
+        dhcp_options = (
+            self.mech_driver._ovn_client._nb_idl.get_subnet_dhcp_options(
+                self.sub['subnet']['id'])
+        )
+        self.assertNotEqual(
+            int(dhcp_options['subnet']['options']['mtu']),
+            mtu_value)
+        data = {'network': {'mtu': mtu_value}}
+        req = self.new_update_request(
+            'networks', data, self.n1['network']['id'], self.fmt)
+        req.get_response(self.api)
+        dhcp_options = (
+            self.mech_driver._ovn_client._nb_idl.get_subnet_dhcp_options(
+                self.sub['subnet']['id'])
+        )
+        self.assertEqual(
+            int(dhcp_options['subnet']['options']['mtu']),
+            mtu_value)
 
 
 class TestSecurityGroup(base.TestOVNFunctionalBase):
@@ -859,6 +854,10 @@ class TestSecurityGroup(base.TestOVNFunctionalBase):
             'tenant_id': self._tenant_id,
             'is_default': True,
         }
+        mock.patch.object(
+            self.plugin, 'get_default_security_group_rules',
+            return_value=copy.deepcopy(
+                test_sg.RULES_TEMPLATE_FOR_CUSTOM_SG)).start()
 
     def _find_acls_for_sg(self, sg_id):
         rows = self.nb_api.db_find_rows('ACL').execute(check_error=True)
@@ -970,7 +969,7 @@ class TestProvnetPorts(base.TestOVNFunctionalBase):
 
     def test_network_segments_localnet_ports(self):
         n1 = self._make_network(
-                self.fmt, 'n1', True,
+                self.fmt, 'n1', True, as_admin=True,
                 arg_list=('provider:network_type',
                           'provider:segmentation_id',
                           'provider:physical_network'),
@@ -1017,6 +1016,7 @@ class TestMetadataPorts(base.TestOVNFunctionalBase):
 
     def setUp(self, *args, **kwargs):
         super().setUp(*args, **kwargs)
+        self.plugin = self.mech_driver._plugin
         self._ovn_client = self.mech_driver._ovn_client
         self.meta_regex = re.compile(r'%s,(\d+\.\d+\.\d+\.\d+)' %
                                      constants.METADATA_V4_CIDR)
@@ -1039,9 +1039,9 @@ class TestMetadataPorts(base.TestOVNFunctionalBase):
         res = self._list_ports(self.fmt, net_id=net_id)
         return self.deserialize(self.fmt, res)['ports']
 
-    def _check_metadata_port(self, net_id, fixed_ip):
+    def _check_metadata_port(self, net_id, fixed_ip, fail=True):
         for port in self._list_ports_ovn(net_id=net_id):
-            if ovn_client.OVNClient.is_metadata_port(port):
+            if utils.is_ovn_metadata_port(port):
                 self.assertEqual(net_id, port['network_id'])
                 if fixed_ip:
                     self.assertIn(fixed_ip, port['fixed_ips'])
@@ -1049,13 +1049,17 @@ class TestMetadataPorts(base.TestOVNFunctionalBase):
                     self.assertEqual([], port['fixed_ips'])
                 return port['id']
 
-        self.fail('Metadata port is not present in network %s or data is not '
-                  'correct' % self.n1_id)
+        if fail:
+            self.fail('Metadata port is not present in network %s or data is '
+                      'not correct' % self.n1_id)
 
     def _check_subnet_dhcp_options(self, subnet_id, cidr):
-        # This method checks the DHCP options CIDR and returns, if exits, the
-        # metadata port IP address, included in the classless static routes.
+        # This method checks DHCP options for a subnet ID, and if they exist,
+        # verifies the CIDR matches. Returns the metadata port IP address
+        # if it is included in the classless static routes, else returns None.
         dhcp_opts = self._ovn_client._nb_idl.get_subnet_dhcp_options(subnet_id)
+        if not dhcp_opts['subnet']:
+            return
         self.assertEqual(cidr, dhcp_opts['subnet']['cidr'])
         routes = dhcp_opts['subnet']['options'].get('classless_static_route')
         if not routes:
@@ -1084,6 +1088,35 @@ class TestMetadataPorts(base.TestOVNFunctionalBase):
         fixed_ip = {'subnet_id': subnet['id'], 'ip_address': metatada_ip}
         self._check_metadata_port(self.n1_id, fixed_ip)
 
+    def test_update_subnet_ipv4(self):
+        self._create_network_ovn(metadata_enabled=True)
+        subnet = self._create_subnet_ovn('10.0.0.0/24')
+        metatada_ip = self._check_subnet_dhcp_options(subnet['id'],
+                                                      '10.0.0.0/24')
+        fixed_ip = {'subnet_id': subnet['id'], 'ip_address': metatada_ip}
+        port_id = self._check_metadata_port(self.n1_id, fixed_ip)
+
+        # Disable DHCP, port should still be present
+        subnet['enable_dhcp'] = False
+        self._ovn_client.update_subnet(self.context, subnet,
+                                       self.n1['network'])
+        port_id = self._check_metadata_port(self.n1_id, None)
+        self.assertIsNone(self._check_subnet_dhcp_options(subnet['id'], []))
+
+        # Delete metadata port
+        self.plugin.delete_port(self.context, port_id)
+        port_id = self._check_metadata_port(self.n1_id, None, fail=False)
+        self.assertIsNone(port_id)
+
+        # Enable DHCP, metadata port should have been re-created
+        subnet['enable_dhcp'] = True
+        self._ovn_client.update_subnet(self.context, subnet,
+                                       self.n1['network'])
+        metatada_ip = self._check_subnet_dhcp_options(subnet['id'],
+                                                      '10.0.0.0/24')
+        fixed_ip = {'subnet_id': subnet['id'], 'ip_address': metatada_ip}
+        port_id = self._check_metadata_port(self.n1_id, fixed_ip)
+
     def test_subnet_ipv4_no_metadata(self):
         self._create_network_ovn(metadata_enabled=False)
         subnet = self._create_subnet_ovn('10.0.0.0/24')
@@ -1157,6 +1190,19 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         _, status = self.plugin.create_or_update_agent(self.context, agent)
         return status['id']
 
+    def _check_chassis_registers(self, present=True):
+        chassis = self.sb_api.lookup('Chassis', self.chassis, default=None)
+        chassis_name = chassis.name if chassis else None
+        if self.sb_api.is_table_present('Chassis_Private'):
+            ch_private = self.sb_api.lookup(
+                'Chassis_Private', self.chassis, default=None)
+            ch_private_name = ch_private.name if ch_private else None
+            self.assertEqual(chassis_name, ch_private_name)
+        if present:
+            self.assertEqual(self.chassis, chassis_name)
+        else:
+            self.assertIsNone(chassis)
+
     def test_agent_show(self):
         for agent_id in self.agent_types.values():
             self.assertTrue(self.plugin.get_agent(self.context, agent_id))
@@ -1170,7 +1216,7 @@ class TestAgentApi(base.TestOVNFunctionalBase):
                 'Chassis_Private', self.chassis, 'nb_cfg_timestamp'
             ).execute(check_error=True)
             updated_at = datetime.datetime.fromtimestamp(
-                int(chassis_ts / 1000), datetime.timezone.utc)
+                int(chassis_ts / 1000))
             # if table Chassis_Private present, agent.updated_at is
             # Chassis_Private.nb_cfg_timestamp
             self.assertEqual(updated_at, heartbeat_timestamp)
@@ -1190,7 +1236,8 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         # then Chassis_Private.chassis = []; both metadata and controller
         # agents will still be present in the agent list.
         agent_event = AgentWaitEvent(self.mech_driver, [self.chassis],
-                                     events=(event.RowEvent.ROW_UPDATE,))
+                                     events=(event.RowEvent.ROW_UPDATE,
+                                             event.RowEvent.ROW_DELETE,))
         self.sb_api.idl.notify_handler.watch_event(agent_event)
         self.sb_api.chassis_del(self.chassis).execute(check_error=True)
         self.assertTrue(agent_event.wait())
@@ -1205,12 +1252,15 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         self.assertRaises(agent_exc.AgentNotFound, self.plugin.get_agent,
                           self.context, agent_id)
 
-        # OVN controller agent deletion, that triggers the "Chassis" register
-        # deletion. The "Chassis" register deletion triggers the host OVN
-        # agents deletion, both controller and metadata if present.
+        # OVN controller agent deletion, that triggers the "Chassis" and
+        # "Chassis_Private" registers deletion. The registers deletion triggers
+        # the host OVN agents deletion, both controller and metadata if
+        # present.
         controller_id = self.agent_types[ovn_const.OVN_CONTROLLER_AGENT]
         metadata_id = self.agent_types[ovn_const.OVN_METADATA_AGENT]
+        self._check_chassis_registers()
         self.plugin.delete_agent(self.context, controller_id)
+        self._check_chassis_registers(present=False)
         self.assertRaises(agent_exc.AgentNotFound, self.plugin.get_agent,
                           self.context, controller_id)
         self.assertEqual(
@@ -1220,3 +1270,105 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         self.plugin.delete_agent(self.context, metadata_id)
         self.assertRaises(agent_exc.AgentNotFound, self.plugin.get_agent,
                           self.context, metadata_id)
+
+
+class TestNATRuleGatewayPort(base.TestOVNFunctionalBase):
+
+    def setUp(self):
+        super().setUp()
+        self._ovn_client = self.mech_driver._ovn_client
+
+    def deserialize(self, content_type, response):
+        ctype = 'application/%s' % content_type
+        data = self._deserializers[ctype].deserialize(response.body)['body']
+        return data
+
+    def _create_router(self, name, external_gateway_info=None):
+        data = {'router': {'name': name, 'tenant_id': self._tenant_id,
+                           'external_gateway_info': external_gateway_info}}
+        as_admin = bool(external_gateway_info.get('enable_snat'))
+        req = self.new_create_request('routers', data, self.fmt,
+                                      as_admin=as_admin)
+        res = req.get_response(self.api)
+        return self.deserialize(self.fmt, res)['router']
+
+    def _process_router_interface(self, action, router_id, subnet_id):
+        req = self.new_action_request(
+            'routers', {'subnet_id': subnet_id}, router_id,
+            '%s_router_interface' % action)
+        res = req.get_response(self.api)
+        return self.deserialize(self.fmt, res)
+
+    def _add_router_interface(self, router_id, subnet_id):
+        return self._process_router_interface('add', router_id, subnet_id)
+
+    def _create_port(self, name, net_id, security_groups=None,
+                     device_owner=None):
+        data = {'port': {'name': name,
+                         'tenant_id': self._tenant_id,
+                         'network_id': net_id}}
+
+        if security_groups is not None:
+            data['port']['security_groups'] = security_groups
+
+        if device_owner is not None:
+            data['port']['device_owner'] = device_owner
+
+        req = self.new_create_request('ports', data, self.fmt)
+        res = req.get_response(self.api)
+        return self.deserialize(self.fmt, res)['port']
+
+    def test_create_floatingip(self):
+        ext_net = self._make_network(
+            self.fmt, 'ext_networktest', True, as_admin=True,
+            arg_list=('router:external',
+                      'provider:network_type',
+                      'provider:physical_network'),
+            **{'router:external': True,
+               'provider:network_type': 'flat',
+               'provider:physical_network': 'public'})['network']
+        res = self._create_subnet(self.fmt, ext_net['id'],
+            '100.0.0.0/24', gateway_ip='100.0.0.254',
+            allocation_pools=[{'start': '100.0.0.2',
+                               'end': '100.0.0.253'}],
+            enable_dhcp=False)
+        ext_subnet = self.deserialize(self.fmt, res)['subnet']
+        net1 = self._make_network(
+            self.fmt, 'network1test', True)['network']
+        res = self._create_subnet(self.fmt, net1['id'],
+            '192.168.0.0/24', gateway_ip='192.168.0.1',
+            allocation_pools=[{'start': '192.168.0.2',
+                               'end': '192.168.0.253'}],
+            enable_dhcp=False)
+        subnet1 = self.deserialize(self.fmt, res)['subnet']
+        external_gateway_info = {
+            'enable_snat': True,
+            'network_id': ext_net['id'],
+            'external_fixed_ips': [
+                {'ip_address': '100.0.0.2', 'subnet_id': ext_subnet['id']}]}
+        router = self._create_router(
+            'routertest', external_gateway_info=external_gateway_info)
+        self._add_router_interface(router['id'], subnet1['id'])
+
+        p1 = self._create_port('testp1', net1['id'])
+        logical_ip = p1['fixed_ips'][0]['ip_address']
+        fip_info = {'floatingip': {
+            'tenant_id': self._tenant_id,
+            'description': 'test_fip',
+            'floating_network_id': ext_net['id'],
+            'port_id': p1['id'],
+            'fixed_ip_address': logical_ip}}
+
+        fip = self.l3_plugin.create_floatingip(self.context, fip_info)
+
+        self.assertEqual(router['id'], fip['router_id'])
+        self.assertEqual('testp1', fip['port_details']['name'])
+        self.assertIsNotNone(self.nb_api.get_lswitch_port(fip['port_id']))
+
+        rules = self.nb_api.get_all_logical_routers_with_rports()[0]
+        fip_rule = rules['dnat_and_snats'][0]
+
+        if utils.is_nat_gateway_port_supported(self.nb_api):
+            self.assertNotEqual([], fip_rule['gateway_port'])
+        else:
+            self.assertNotIn('gateway_port', fip_rule)
