@@ -32,12 +32,14 @@ from neutron.tests.functional.resources.ovsdb import events
 
 
 class TestRouter(base.TestOVNFunctionalBase):
-    def setUp(self):
-        super(TestRouter, self).setUp()
+    def setUp(self, **kwargs):
+        super().setUp(**kwargs)
         self.chassis1 = self.add_fake_chassis(
-            'ovs-host1', physical_nets=['physnet1', 'physnet3'])
+            'ovs-host1', physical_nets=['physnet1', 'physnet3'],
+            enable_chassis_as_gw=True, azs=[])
         self.chassis2 = self.add_fake_chassis(
-            'ovs-host2', physical_nets=['physnet2', 'physnet3'])
+            'ovs-host2', physical_nets=['physnet2', 'physnet3'],
+            enable_chassis_as_gw=True, azs=[])
         self.cr_lrp_pb_event = events.WaitForCrLrpPortBindingEvent()
         self.sb_api.idl.notify_handler.watch_event(self.cr_lrp_pb_event)
 
@@ -63,7 +65,7 @@ class TestRouter(base.TestOVNFunctionalBase):
         if physnet:
             arg_list = arg_list + (pnet.PHYSICAL_NETWORK,)
             net_arg[pnet.PHYSICAL_NETWORK] = physnet
-        network = self._make_network(self.fmt, name, True,
+        network = self._make_network(self.fmt, name, True, as_admin=True,
                                      arg_list=arg_list, **net_arg)
         if cidr:
             self._make_subnet(self.fmt, network, gateway, cidr,
@@ -87,20 +89,18 @@ class TestRouter(base.TestOVNFunctionalBase):
                     self.sb_api.tables['Chassis'].rows.values()]
         for row in self.nb_api.tables[
                 'Logical_Router_Port'].rows.values():
-            if self._l3_ha_supported():
-                chassis = [gwc.chassis_name for gwc in row.gateway_chassis]
-                self.assertCountEqual(expected, chassis)
-            else:
-                rc = row.options.get(ovn_const.OVN_GATEWAY_CHASSIS_KEY)
-                self.assertIn(rc, expected)
+            chassis = [gwc.chassis_name for gwc in row.gateway_chassis]
+            self.assertCountEqual(expected, chassis)
 
     def _check_gateway_chassis_candidates(self, candidates,
-                                          router_az_hints=None):
+                                          router_az_hints=None,
+                                          physnet='physnet1'):
         # In this test, fake_select() is called once from _create_router()
         # and later from schedule_unhosted_gateways()
         ovn_client = self.l3_plugin._ovn_client
+        net_type = 'vlan' if physnet else 'geneve'
         ext1 = self._create_ext_network(
-            'ext1', 'vlan', 'physnet1', 1, "10.0.0.1", "10.0.0.0/24")
+            'ext1', net_type, physnet, 1, "10.0.0.1", "10.0.0.0/24")
         # mock select function and check if it is called with expected
         # candidates.
 
@@ -131,19 +131,18 @@ class TestRouter(base.TestOVNFunctionalBase):
 
     def test_gateway_chassis_with_cms_and_bridge_mappings(self):
         # Both chassis1 and chassis3 are having proper bridge mappings,
-        # but only chassis3 is having enable-chassis-as-gw.
-        # Test if chassis3 is selected as candidate or not.
+        # but only chassis1 is having enable-chassis-as-gw.
+        # Test if chassis1 is selected as candidate or not.
         self.chassis3 = self.add_fake_chassis(
-            'ovs-host3', physical_nets=['physnet1'],
-            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'})
-        self._check_gateway_chassis_candidates([self.chassis3])
+            'ovs-host3', physical_nets=['physnet1'], azs=[])
+        self._check_gateway_chassis_candidates([self.chassis1])
 
     def test_gateway_chassis_with_cms_and_no_bridge_mappings(self):
         # chassis1 is having proper bridge mappings.
         # chassis3 is having enable-chassis-as-gw, but no bridge mappings.
         self.chassis3 = self.add_fake_chassis(
             'ovs-host3',
-            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'})
+            other_config={'ovn-cms-options': 'enable-chassis-as-gw'})
         ovn_client = self.l3_plugin._ovn_client
         ext1 = self._create_ext_network(
             'ext1', 'vlan', 'physnet1', 1, "10.0.0.1", "10.0.0.0/24")
@@ -170,12 +169,10 @@ class TestRouter(base.TestOVNFunctionalBase):
         # Test if chassis3 is selected as candidate or not.
         self.chassis3 = self.add_fake_chassis(
             'ovs-host3', physical_nets=['physnet1'],
-            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'},
-            azs=['ovn'])
+            azs=['ovn'], enable_chassis_as_gw=True)
         self.chassis4 = self.add_fake_chassis(
             'ovs-host4', physical_nets=['physnet1'],
-            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'},
-            azs=['ovn2'])
+            azs=['ovn2'], enable_chassis_as_gw=True)
         self._check_gateway_chassis_candidates([self.chassis3],
                                                router_az_hints=['ovn'])
 
@@ -185,11 +182,9 @@ class TestRouter(base.TestOVNFunctionalBase):
         # AvailabilityZoneNotFound. after create will delete if.
         # add chassis4 is having azs [ovn2], not match routers az_hints [ovn]
         self.chassis3 = self.add_fake_chassis(
-            'ovs-host3', physical_nets=['physnet1'],
-            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'})
+            'ovs-host3', physical_nets=['physnet1'], enable_chassis_as_gw=True)
         self.chassis4 = self.add_fake_chassis(
-            'ovs-host4', physical_nets=['physnet1'],
-            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'},
+            'ovs-host4', physical_nets=['physnet1'], enable_chassis_as_gw=True,
             azs=['ovn2'])
         ovn_client = self.l3_plugin._ovn_client
         ext1 = self._create_ext_network(
@@ -217,10 +212,10 @@ class TestRouter(base.TestOVNFunctionalBase):
         # Test if chassis1 is selected as candidate or not.
         self._check_gateway_chassis_candidates([self.chassis1])
 
-    def _l3_ha_supported(self):
-        # If the Gateway_Chassis table exists in SB database, then it
-        # means that L3 HA is supported.
-        return self.nb_api.tables.get('Gateway_Chassis')
+    def test_gateway_chassis_no_physnet_tunnelled_network(self):
+        # The GW network is tunnelled, no physnet defined --> no possible
+        # candidates.
+        self._check_gateway_chassis_candidates([], physnet=None)
 
     def test_gateway_chassis_least_loaded_scheduler(self):
         # This test will create 4 routers each with its own gateway.
@@ -241,26 +236,17 @@ class TestRouter(base.TestOVNFunctionalBase):
         # At this point we expect two gateways to be present in chassis1
         # and two in chassis2. If schema supports L3 HA, we expect each
         # chassis to host 2 priority 2 gateways and 2 priority 1 ones.
-        if self._l3_ha_supported():
-            # Each chassis contains a dict of (priority, # of ports hosted).
-            # {1: 2, 2: 2} means that this chassis hosts 2 ports of prio 1
-            # and two ports of prio 2.
-            expected = {self.chassis1: {1: 2, 2: 2},
-                        self.chassis2: {1: 2, 2: 2}}
-        else:
-            # For non L3 HA, each chassis should contain two gateway ports.
-            expected = {self.chassis1: 2,
-                        self.chassis2: 2}
+        # Each chassis contains a dict of (priority, # of ports hosted).
+        # {1: 2, 2: 2} means that this chassis hosts 2 ports of prio 1
+        # and two ports of prio 2.
+        expected = {self.chassis1: {1: 2, 2: 2},
+                    self.chassis2: {1: 2, 2: 2}}
         sched_info = {}
         for row in self.nb_api.tables[
                 'Logical_Router_Port'].rows.values():
-            if self._l3_ha_supported():
-                for gwc in row.gateway_chassis:
-                    chassis = sched_info.setdefault(gwc.chassis_name, {})
-                    chassis[gwc.priority] = chassis.get(gwc.priority, 0) + 1
-            else:
-                rc = row.options.get(ovn_const.OVN_GATEWAY_CHASSIS_KEY)
-                sched_info[rc] = sched_info.get(rc, 0) + 1
+            for gwc in row.gateway_chassis:
+                chassis = sched_info.setdefault(gwc.chassis_name, {})
+                chassis[gwc.priority] = chassis.get(gwc.priority, 0) + 1
         self.assertEqual(expected, sched_info)
 
     def _get_gw_port(self, router_id):
@@ -439,6 +425,12 @@ class TestRouter(base.TestOVNFunctionalBase):
             self.assertGreaterEqual(client_select.call_count, 3)
             self.assertGreaterEqual(plugin_select.call_count, 3)
 
+    def _find_port_binding(self, port_id):
+        cmd = self.sb_api.db_find_rows('Port_Binding',
+                                       ('logical_port', '=', port_id))
+        rows = cmd.execute(check_error=True)
+        return rows[0] if rows else None
+
     def test_router_gateway_port_binding_host_id(self):
         # Test setting chassis on chassisredirect port in Port_Binding table,
         # will update host_id of corresponding router gateway port
@@ -461,14 +453,30 @@ class TestRouter(base.TestOVNFunctionalBase):
                              may_exist=True).execute(check_error=True)
 
         def check_port_binding_host_id(port_id):
+            # Get port from Neutron DB
             port = core_plugin.get_ports(
                 self.context, filters={'id': [port_id]})[0]
-            return port[portbindings.HOST_ID] == host_id
+            # Get port from OVN DB
+            bp = self._find_port_binding(port_id)
+            ovn_host_id = bp.external_ids.get(ovn_const.OVN_HOST_ID_EXT_ID_KEY)
+            return port[portbindings.HOST_ID] == host_id == ovn_host_id
 
         # Test if router gateway port updated with this chassis
         n_utils.wait_until_true(lambda: check_port_binding_host_id(
             gw_port_id))
 
+        # Simulate failover to another chassis and check host_id in Neutron DB
+        # and external_ids:neutron:host_id in OVN DB are updated
+        chassis = idlutils.row_by_value(
+            self.sb_api.idl, "Chassis", "name", self.chassis2
+        )
+        host_id = chassis.hostname
+        self.sb_api.lsp_unbind(logical_port).execute(check_error=True)
+        self.sb_api.lsp_bind(logical_port, self.chassis2).execute(
+            check_error=True
+        )
+        n_utils.wait_until_true(lambda: check_port_binding_host_id(gw_port_id))
+
     def _validate_router_ipv6_ra_configs(self, lrp_name, expected_ra_confs):
         lrp = idlutils.row_by_value(self.nb_api.idl,
                                     'Logical_Router_Port', 'name', lrp_name)
@@ -531,19 +539,17 @@ class TestRouter(base.TestOVNFunctionalBase):
                     chassis[gwc.priority] = chassis.get(gwc.priority, 0) + 1
             return sched_info
 
-        if not self._l3_ha_supported():
-            self.skipTest('L3 HA not supported')
         ovn_client = self.l3_plugin._ovn_client
         chassis4 = self.add_fake_chassis(
-            'ovs-host4', physical_nets=['physnet4'], external_ids={
+            'ovs-host4', physical_nets=['physnet4'], other_config={
                 'ovn-cms-options': 'enable-chassis-as-gw'})
         ovn_client._ovn_scheduler = l3_sched.OVNGatewayLeastLoadedScheduler()
         ext1 = self._create_ext_network(
             'ext1', 'flat', 'physnet4', None, "30.0.0.1", "30.0.0.0/24")
         gw_info = {'network_id': ext1['network']['id']}
-        # Tries to create 20 routers with a gateway. Since we're using
+        # Tries to create 5 routers with a gateway. Since we're using
         # physnet4, the chassis candidates will be chassis4 initially.
-        num_routers = 20
+        num_routers = 5
         for i in range(num_routers):
             try:
                 router = self._create_router('router%d' % i, gw_info=gw_info)
@@ -567,7 +573,7 @@ class TestRouter(base.TestOVNFunctionalBase):
 
         # Add another chassis as a gateway chassis
         chassis5 = self.add_fake_chassis(
-            'ovs-host5', physical_nets=['physnet4'], external_ids={
+            'ovs-host5', physical_nets=['physnet4'], other_config={
                 'ovn-cms-options': 'enable-chassis-as-gw'})
         # Add a node as compute node. Compute node wont be
         # used to schedule the router gateway ports therefore
@@ -597,8 +603,7 @@ class TestRouter(base.TestOVNFunctionalBase):
             chassis_list.append(
                 self.add_fake_chassis(
                     'ovs-host%s' % i, physical_nets=['physnet1'],
-                    external_ids={
-                        'ovn-cms-options': 'enable-chassis-as-gw'}))
+                    other_config={'ovn-cms-options': 'enable-chassis-as-gw'}))
 
         ext1 = self._create_ext_network(
             'ext1', 'vlan', 'physnet1', 1, "10.0.0.1", "10.0.0.0/24")
