@@ -13,11 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import time
 from unittest import mock
 
 from neutron_lib import constants
 from neutron_lib.plugins.ml2 import ovs_constants as ovs_const
 from oslo_utils import excutils
+from oslo_utils import uuidutils
 from pyroute2.netlink import exceptions as pyroute2_exc
 
 from neutron.agent.common import ovs_lib
@@ -72,6 +74,8 @@ class TestBase(base.BaseTestCase):
         self.ip_dev = self.ip_dev_p.start()
         self.ip_p = mock.patch.object(ip_lib, 'IPWrapper')
         self.ip = self.ip_p.start()
+        link = self.ip.return_value.device.return_value.link
+        link.address = 'aa:bb:cc:dd:ee:ff'
         self.device_exists_p = mock.patch.object(ip_lib, 'device_exists')
         self.device_exists = self.device_exists_p.start()
         self.get_devices_with_ip_p = mock.patch.object(ip_lib,
@@ -385,6 +389,10 @@ class TestABCDriver(TestBase):
 
 class TestOVSInterfaceDriver(TestBase):
 
+    def setUp(self):
+        super().setUp()
+        self.mock_sleep = mock.patch.object(time, 'sleep').start()
+
     def test_get_device_name(self):
         br = interface.OVSInterfaceDriver(self.conf)
         device_name = br.get_device_name(FakePort())
@@ -461,17 +469,18 @@ class TestOVSInterfaceDriver(TestBase):
             expected = [
                 mock.call(),
                 mock.call().device('tap0'),
+                mock.ANY,
                 mock.call().device().link.set_address('aa:bb:cc:dd:ee:ff'),
                 mock.call().device().link.set_address('aa:bb:cc:dd:ee:ff')]
             if namespace:
                 expected.extend(
                     [mock.call().ensure_namespace(namespace),
                      mock.call().ensure_namespace().add_device_to_namespace(
-                         mock.ANY),
+                         mock.ANY, is_ovs_port=True),
                      mock.call().ensure_namespace().add_device_to_namespace(
-                         mock.ANY),
+                         mock.ANY, is_ovs_port=True),
                      mock.call().ensure_namespace().add_device_to_namespace(
-                         mock.ANY)])
+                         mock.ANY, is_ovs_port=True)])
             expected.extend([
                 mock.call(namespace=namespace),
                 mock.call().device('tap0'),
@@ -492,12 +501,11 @@ class TestOVSInterfaceDriver(TestBase):
                     reraise = mock.patch.object(
                         excutils, 'save_and_reraise_exception')
                     reraise.start()
-                    ip_wrapper = mock.Mock()
                     for exception in (OSError(),
                                       pyroute2_exc.NetlinkError(22),
                                       RuntimeError()):
-                        ip_wrapper.ensure_namespace.side_effect = exception
-                        self.ip.return_value = ip_wrapper
+                        ip = self.ip.return_value
+                        ip.ensure_namespace.side_effect = exception
                         delete_port.reset_mock()
                         ovs.plug_new(
                             '01234567-1234-1234-99',
@@ -523,13 +531,14 @@ class TestOVSInterfaceDriver(TestBase):
         self.ip.ensure_namespace.return_value = namespace_obj
         namespace_obj.add_device_to_namespace.side_effect = (
             ip_lib.NetworkInterfaceNotFound)
-        device = mock.MagicMock()
+        namespace_name = uuidutils.generate_uuid()
+        device = mock.MagicMock(namespace=namespace_name)
         self.assertRaises(
             ip_lib.NetworkInterfaceNotFound,
             ovs._add_device_to_namespace,
             self.ip, device, "test-ns")
         self.assertEqual(10, namespace_obj.add_device_to_namespace.call_count)
-        self.assertIsNone(device.namespace)
+        self.assertEqual(namespace_name, device.namespace)
 
 
 class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
@@ -568,6 +577,7 @@ class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
 
             root_dev = mock.Mock()
             ns_dev = mock.Mock()
+            ns_dev.link.address = 'aa:bb:cc:dd:ee:ff'
             self.ip().add_veth = mock.Mock(return_value=(root_dev, ns_dev))
             mock.patch.object(
                 interface, '_get_veth',
