@@ -23,11 +23,11 @@ from neutron_lib import constants
 from neutron_lib import exceptions
 from oslo_utils import netutils
 from oslo_utils import uuidutils
-import pyroute2
+from pyroute2.netlink import exceptions as netlink_exceptions
 from pyroute2.netlink.rtnl import ifinfmsg
 from pyroute2.netlink.rtnl import ndmsg
-from pyroute2 import NetlinkError
 from pyroute2 import netns
+from pyroute2.nslink import nslink
 import testtools
 
 from neutron.agent.common import utils  # noqa
@@ -357,6 +357,21 @@ class TestIpWrapper(base.BaseTestCase):
                 self.assertNotIn(mock.call().delete('ns'),
                                  ip_ns_cmd_cls.mock_calls)
 
+    def test_garbage_collect_namespace_existing_broken(self):
+        with mock.patch.object(ip_lib, 'IpNetnsCommand') as ip_ns_cmd_cls:
+            ip_ns_cmd_cls.return_value.exists.return_value = True
+
+            ip = ip_lib.IPWrapper(namespace='ns')
+
+            with mock.patch.object(ip, 'get_devices',
+                                   side_effect=OSError(errno.EINVAL, None)
+                                   ) as mock_get_devices:
+                self.assertTrue(ip.garbage_collect_namespace())
+
+                mock_get_devices.assert_called_once_with()
+                expected = [mock.call().delete('ns')]
+                ip_ns_cmd_cls.assert_has_calls(expected)
+
     @mock.patch.object(priv_lib, 'create_interface')
     def test_add_vlan(self, create):
         retval = ip_lib.IPWrapper().add_vlan('eth0.1', 'eth0', '1')
@@ -507,7 +522,8 @@ class TestIpWrapper(base.BaseTestCase):
     def test_add_device_to_namespace(self):
         dev = mock.Mock()
         ip_lib.IPWrapper(namespace='ns').add_device_to_namespace(dev)
-        dev.assert_has_calls([mock.call.link.set_netns('ns')])
+        dev.assert_has_calls(
+            [mock.call.link.set_netns('ns', is_ovs_port=False)])
 
     def test_add_device_to_namespace_is_none(self):
         dev = mock.Mock()
@@ -777,7 +793,8 @@ class TestIpAddrCommand(TestIPCmdBase):
             6, self.parent.name, self.addr_cmd._parent.namespace)
 
     def test_wait_until_address_ready(self):
-        self.addr_cmd.list = mock.Mock(return_value=[{'tentative': False}])
+        self.addr_cmd.list = mock.Mock(
+            return_value=[{'tentative': False, 'dadfailed': False}])
         # this address is not tentative or failed so it should return
         self.assertIsNone(self.addr_cmd.wait_until_address_ready(
             '2001:470:9:1224:fd91:272:581e:3a32'))
@@ -787,6 +804,24 @@ class TestIpAddrCommand(TestIPCmdBase):
         with testtools.ExpectedException(ip_lib.AddressNotReady):
             self.addr_cmd.wait_until_address_ready('abcd::1234')
 
+    def test_wait_until_address_dadfailed(self):
+        self.addr_cmd.list = mock.Mock(
+            return_value=[{'tentative': True, 'dadfailed': True}])
+        with testtools.ExpectedException(ip_lib.DADFailed):
+            self.addr_cmd.wait_until_address_ready('abcd::1234')
+
+    @mock.patch.object(common_utils, 'wait_until_true')
+    def test_wait_until_address_ready_success_one_timeout(self, mock_wuntil):
+        tentative_address = 'fe80::3023:39ff:febc:22ae'
+        self.addr_cmd.list = mock.Mock(return_value=[
+            dict(scope='link', dadfailed=False, tentative=True, dynamic=False,
+                 cidr=tentative_address + '/64'),
+            dict(scope='link', dadfailed=False, tentative=False, dynamic=False,
+                 cidr=tentative_address + '/64')])
+        self.assertIsNone(self.addr_cmd.wait_until_address_ready(
+            tentative_address, wait_time=3))
+        self.assertEqual(1, mock_wuntil.call_count)
+
     def test_wait_until_address_ready_timeout(self):
         tentative_address = 'fe80::3023:39ff:febc:22ae'
         self.addr_cmd.list = mock.Mock(return_value=[
@@ -957,7 +992,7 @@ class TestIpNeighCommand(TestIPCmdBase):
         self.addCleanup(privileged.default.set_client_mode, True)
         privileged.default.set_client_mode(False)
 
-    @mock.patch.object(pyroute2, 'NetNS')
+    @mock.patch.object(nslink, 'NetNS')
     def test_add_entry(self, mock_netns):
         mock_netns_instance = mock_netns.return_value
         mock_netns_enter = mock_netns_instance.__enter__.return_value
@@ -972,20 +1007,20 @@ class TestIpNeighCommand(TestIPCmdBase):
             ifindex=1,
             state=ndmsg.states['permanent'])
 
-    @mock.patch.object(pyroute2, 'NetNS')
+    @mock.patch.object(nslink, 'NetNS')
     def test_add_entry_nonexistent_namespace(self, mock_netns):
         mock_netns.side_effect = OSError(errno.ENOENT, None)
         with testtools.ExpectedException(ip_lib.NetworkNamespaceNotFound):
             self.neigh_cmd.add('192.168.45.100', 'cc:dd:ee:ff:ab:cd')
 
-    @mock.patch.object(pyroute2, 'NetNS')
+    @mock.patch.object(nslink, 'NetNS')
     def test_add_entry_other_error(self, mock_netns):
         expected_exception = OSError(errno.EACCES, None)
         mock_netns.side_effect = expected_exception
         with testtools.ExpectedException(expected_exception.__class__):
             self.neigh_cmd.add('192.168.45.100', 'cc:dd:ee:ff:ab:cd')
 
-    @mock.patch.object(pyroute2, 'NetNS')
+    @mock.patch.object(nslink, 'NetNS')
     def test_delete_entry(self, mock_netns):
         mock_netns_instance = mock_netns.return_value
         mock_netns_enter = mock_netns_instance.__enter__.return_value
@@ -1002,10 +1037,11 @@ class TestIpNeighCommand(TestIPCmdBase):
     @mock.patch.object(priv_lib, '_run_iproute_neigh')
     def test_delete_entry_not_exist(self, mock_run_iproute):
         # trying to delete a non-existent entry shouldn't raise an error
-        mock_run_iproute.side_effect = NetlinkError(errno.ENOENT, None)
+        mock_run_iproute.side_effect = netlink_exceptions.NetlinkError(
+            errno.ENOENT, None)
         self.neigh_cmd.delete('192.168.45.100', 'cc:dd:ee:ff:ab:cd')
 
-    @mock.patch.object(pyroute2, 'NetNS')
+    @mock.patch.object(nslink, 'NetNS')
     def test_dump_entries(self, mock_netns):
         mock_netns_instance = mock_netns.return_value
         mock_netns_enter = mock_netns_instance.__enter__.return_value
