@@ -15,6 +15,7 @@
 
 from collections import namedtuple
 from os import path
+import shlex
 from unittest import mock
 
 import fixtures
@@ -22,9 +23,13 @@ import neutron_lib
 from neutron_lib.api.definitions import extra_dhcp_opt as edo_ext
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_const
+from neutron_lib import exceptions as n_exc
+from oslo_concurrency import processutils
 from oslo_config import cfg
+import testtools
 
 from neutron.common.ovn import constants
+from neutron.common.ovn import exceptions as ovn_exc
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.tests import base
@@ -49,43 +54,53 @@ class TestUtils(base.BaseTestCase):
     def test_get_system_dns_resolvers(self):
         tempdir = self.useFixture(fixtures.TempDir()).path
         resolver_file_name = tempdir + '/resolv.conf'
-        tmp_resolv_file = open(resolver_file_name, 'w')
-        tmp_resolv_file.writelines(RESOLV_CONF_TEMPLATE)
-        tmp_resolv_file.close()
+        with open(resolver_file_name, 'w') as f:
+            f.writelines(RESOLV_CONF_TEMPLATE)
         expected_dns_resolvers = RESOLV_DNS_SERVERS
         observed_dns_resolvers = utils.get_system_dns_resolvers(
             resolver_file=resolver_file_name)
         self.assertEqual(expected_dns_resolvers, observed_dns_resolvers)
 
-    def test_is_gateway_chassis(self):
-        chassis = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
-            'external_ids': {'ovn-cms-options': 'enable-chassis-as-gw'}})
-        non_gw_chassis_0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
-            'external_ids': {'ovn-cms-options': ''}})
-        non_gw_chassis_1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={})
-        non_gw_chassis_2 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
-            'external_ids': {}})
+    def _test_is_chassis(self, is_gateway=False, is_extport=False):
+        if is_gateway:
+            cms_option_value = constants.CMS_OPT_CHASSIS_AS_GW
+            func = utils.is_gateway_chassis
+        if is_extport:
+            cms_option_value = constants.CMS_OPT_CHASSIS_AS_EXTPORT_HOST
+            func = utils.is_extport_host_chassis
 
-        self.assertTrue(utils.is_gateway_chassis(chassis))
-        self.assertFalse(utils.is_gateway_chassis(non_gw_chassis_0))
-        self.assertFalse(utils.is_gateway_chassis(non_gw_chassis_1))
-        self.assertFalse(utils.is_gateway_chassis(non_gw_chassis_2))
+        chassis = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
+            'other_config': {'ovn-cms-options': cms_option_value}})
+        wrong_chassis_0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
+            'other_config': {'ovn-cms-options': ''}})
+        wrong_chassis_1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
+            'other_config': {}})
+
+        self.assertTrue(func(chassis))
+        self.assertFalse(func(wrong_chassis_0))
+        self.assertFalse(func(wrong_chassis_1))
+
+    def test_is_gateway_chassis(self):
+        self._test_is_chassis(is_gateway=True)
+
+    def test_is_extport_host_chassis(self):
+        self._test_is_chassis(is_extport=True)
 
     def test_get_chassis_availability_zones_no_azs(self):
         chassis = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
-            'external_ids': {'ovn-cms-options': 'enable-chassis-as-gw'}})
+            'other_config': {'ovn-cms-options': 'enable-chassis-as-gw'}})
         self.assertEqual(set(), utils.get_chassis_availability_zones(chassis))
 
     def test_get_chassis_availability_zones_one_az(self):
         chassis = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
-            'external_ids': {'ovn-cms-options':
+            'other_config': {'ovn-cms-options':
                              'enable-chassis-as-gw,availability-zones=az0'}})
         self.assertEqual(
             {'az0'}, utils.get_chassis_availability_zones(chassis))
 
     def test_get_chassis_availability_zones_multiple_az(self):
         chassis = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
-            'external_ids': {
+            'other_config': {
                 'ovn-cms-options':
                 'enable-chassis-as-gw,availability-zones=az0:az1 :az2:: :'}})
         self.assertEqual(
@@ -94,7 +109,7 @@ class TestUtils(base.BaseTestCase):
 
     def test_get_chassis_availability_zones_malformed(self):
         chassis = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
-            'external_ids': {'ovn-cms-options':
+            'other_config': {'ovn-cms-options':
                              'enable-chassis-as-gw,availability-zones:az0'}})
         self.assertEqual(
             set(), utils.get_chassis_availability_zones(chassis))
@@ -153,16 +168,16 @@ class TestUtils(base.BaseTestCase):
     def test_get_chassis_in_azs(self):
         ch0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
             'name': 'ch0',
-            'external_ids': {
+            'other_config': {
                 'ovn-cms-options':
                 'enable-chassis-as-gw,availability-zones=az0:az1:az2'}})
         ch1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
             'name': 'ch1',
-            'external_ids': {
+            'other_config': {
                 'ovn-cms-options': 'enable-chassis-as-gw'}})
         ch2 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
             'name': 'ch2',
-            'external_ids': {
+            'other_config': {
                 'ovn-cms-options':
                 'enable-chassis-as-gw,availability-zones=az1:az5'}})
 
@@ -177,29 +192,59 @@ class TestUtils(base.BaseTestCase):
             set(),
             utils.get_chassis_in_azs(chassis_list, ['az6']))
 
-    def test_get_gateway_chassis_without_azs(self):
+    def test_get_chassis_without_azs(self):
         ch0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
             'name': 'ch0',
-            'external_ids': {
+            'other_config': {
                 'ovn-cms-options':
                 'enable-chassis-as-gw,availability-zones=az0:az1:az2'}})
         ch1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
             'name': 'ch1',
-            'external_ids': {
+            'other_config': {
                 'ovn-cms-options': 'enable-chassis-as-gw'}})
         ch2 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
             'name': 'ch2',
-            'external_ids': {
+            'other_config': {
                 'ovn-cms-options':
                 'enable-chassis-as-gw,availability-zones=az1:az5'}})
         ch3 = fakes.FakeOvsdbRow.create_one_ovsdb_row(attrs={
             'name': 'ch3',
-            'external_ids': {}})
+            'other_config': {}})
 
         chassis_list = [ch0, ch1, ch2, ch3]
         self.assertEqual(
-            {'ch1'},
-            utils.get_gateway_chassis_without_azs(chassis_list))
+            {'ch1', 'ch3'},
+            utils.get_chassis_without_azs(chassis_list))
+
+    def test_is_ovn_metadata_port(self):
+        meta_port = {
+            'device_owner': n_const.DEVICE_OWNER_DISTRIBUTED,
+            'device_id': 'ovnmeta-12345'}
+        non_meta_port_0 = {
+            'device_owner': n_const.DEVICE_OWNER_DISTRIBUTED,
+            'device_id': 'dhcp-12345'}
+        non_meta_port_1 = {
+            'device_owner': n_const.DEVICE_OWNER_DHCP,
+            'device_id': 'ovnmeta-12345'}
+
+        self.assertTrue(utils.is_ovn_metadata_port(meta_port))
+        self.assertFalse(utils.is_ovn_metadata_port(non_meta_port_0))
+        self.assertFalse(utils.is_ovn_metadata_port(non_meta_port_1))
+
+    def test_is_ovn_lb_hm_port(self):
+        ovn_lb_hm_port = {
+            'device_owner': constants.OVN_LB_HM_PORT_DISTRIBUTED,
+            'device_id': 'ovn-lb-hm-12345'}
+        non_ovn_lb_hm_port_0 = {
+            'device_owner': n_const.DEVICE_OWNER_DISTRIBUTED,
+            'device_id': 'ovnmeta-12345'}
+        non_ovn_lb_hm_port_1 = {
+            'device_owner': n_const.DEVICE_OWNER_DHCP,
+            'device_id': 'dhcp-12345'}
+
+        self.assertTrue(utils.is_ovn_lb_hm_port(ovn_lb_hm_port))
+        self.assertFalse(utils.is_ovn_lb_hm_port(non_ovn_lb_hm_port_0))
+        self.assertFalse(utils.is_ovn_lb_hm_port(non_ovn_lb_hm_port_1))
 
 
 class TestGateWayChassisValidity(base.BaseTestCase):
@@ -431,6 +476,34 @@ class TestDHCPUtils(base.BaseTestCase):
         # Assert no options were passed
         self.assertEqual({}, options)
 
+    def test_get_lsp_dhcp_opts_for_domain_search(self):
+        opt = {'opt_name': 'domain-search',
+               'opt_value': 'openstack.org,ovn.org',
+               'ip_version': 4}
+        port = {portbindings.VNIC_TYPE: portbindings.VNIC_NORMAL,
+                edo_ext.EXTRADHCPOPTS: [opt]}
+
+        dhcp_disabled, options = utils.get_lsp_dhcp_opts(port, 4)
+        self.assertFalse(dhcp_disabled)
+        # Assert option got translated to "domain_search_list" and
+        # the value is a string (double-quoted)
+        expected_options = {'domain_search_list': '"openstack.org,ovn.org"'}
+        self.assertEqual(expected_options, options)
+
+    def test_get_lsp_dhcp_opts_sanitize_map(self):
+        opt = {'opt_name': 'classless-static-route',
+               'opt_value': '128.128.128.128/32,22.2.0.2',
+               'ip_version': 4}
+        port = {portbindings.VNIC_TYPE: portbindings.VNIC_NORMAL,
+                edo_ext.EXTRADHCPOPTS: [opt]}
+        dhcp_disabled, options = utils.get_lsp_dhcp_opts(port, 4)
+        self.assertFalse(dhcp_disabled)
+        # Assert option got translated to "classless_static_route" and
+        # the value is a map (wrapped with {})
+        expected_options = {
+            'classless_static_route': '{128.128.128.128/32,22.2.0.2}'}
+        self.assertEqual(expected_options, options)
+
 
 class TestGetDhcpDnsServers(base.BaseTestCase):
 
@@ -573,8 +646,8 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'parent_name': 'fake-parent-port-uuid',
             'tag': 42
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {constants.OVN_PORT_BINDING_PROFILE: expect}))
 
@@ -582,8 +655,8 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'vtep-physical-switch': 'fake-physical-switch-uuid',
             'vtep-logical-switch': 'fake-logical-switch-uuid',
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {constants.OVN_PORT_BINDING_PROFILE: expect}))
 
@@ -596,8 +669,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         }
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_DIRECT,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_DIRECT,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -611,8 +685,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         }
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_DIRECT,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_DIRECT,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -625,11 +700,11 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'pf_mac_address': '00:53:00:00:00:42',
             'vf_num': 42,
         }
-        self.assertDictEqual(
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_REMOTE_MANAGED, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
-                 constants.OVN_PORT_BINDING_PROFILE: expect}),
-            expect)
+                 constants.OVN_PORT_BINDING_PROFILE: expect}))
 
     def test_valid_input_surplus_keys(self):
         # Confirm that extra keys are allowed
@@ -643,16 +718,22 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
         del(expect['optional_information_provided_by_nova'])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, portbindings.VNIC_DIRECT,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: portbindings.VNIC_DIRECT,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
 
+    def test_valid_input_no_binding_profile(self):
+        # Confirm that we treat a port without binding:profile as valid
+        self.assertEqual(utils.BPInfo({}, None, []),
+                         utils.validate_and_get_data_from_binding_profile({}))
+
     def test_unknown_profile_items_pruned(self):
         # Confirm that unknown profile items are pruned
         self.assertEqual(
-            {},
+            utils.BPInfo({}, portbindings.VNIC_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {constants.OVN_PORT_BINDING_PROFILE: {
                     'unknown-key': 'unknown-data'}}))
@@ -661,16 +742,16 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         expect = {
             'key': 'value',
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, self.VNIC_FAKE_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_NORMAL,
                  constants.OVN_PORT_BINDING_PROFILE: expect}))
         expect = {
             'key': None,
         }
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, self.VNIC_FAKE_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_NORMAL,
                  constants.OVN_PORT_BINDING_PROFILE: expect}))
@@ -691,17 +772,17 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'other_key': 'value',
         }
         # This param set is valid for VNIC_FAKE_NORMAL with 'other_key' pruned.
-        expect = binding_profile.copy()
-        del(expect['other_key'])
-        self.assertDictEqual(
-            expect,
+        expected_bp = binding_profile.copy()
+        del(expected_bp['other_key'])
+        self.assertEqual(
+            utils.BPInfo(expected_bp, self.VNIC_FAKE_NORMAL, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_NORMAL,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
         # It is valid for VNIC_FAKE_OTHER
-        expect = binding_profile.copy()
-        self.assertDictEqual(
-            expect,
+        expected_bp = binding_profile.copy()
+        self.assertEqual(
+            utils.BPInfo(expected_bp, self.VNIC_FAKE_OTHER, []),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -729,7 +810,7 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             'third_key': 'value',
         }
         self.assertEqual(
-            {},
+            utils.BPInfo({}, self.VNIC_FAKE_OTHER, ['fake-capability']),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -742,8 +823,9 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
         }
         expect = binding_profile.copy()
         del(expect[constants.PORT_CAP_PARAM])
-        self.assertDictEqual(
-            expect,
+        self.assertEqual(
+            utils.BPInfo(expect, self.VNIC_FAKE_OTHER,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -760,7 +842,8 @@ class TestValidateAndGetDataFromBindingProfile(base.BaseTestCase):
             constants.PORT_CAP_PARAM: [constants.PORT_CAP_SWITCHDEV],
         }
         self.assertEqual(
-            {},
+            utils.BPInfo({}, self.VNIC_FAKE_OTHER,
+                         [constants.PORT_CAP_SWITCHDEV]),
             utils.validate_and_get_data_from_binding_profile(
                 {portbindings.VNIC_TYPE: self.VNIC_FAKE_OTHER,
                  constants.OVN_PORT_BINDING_PROFILE: binding_profile}))
@@ -810,3 +893,357 @@ class TestRetryDecorator(base.BaseTestCase):
 
         # number of exceptions + one successful call
         self.assertEqual(number_of_exceptions + 1, method.call_count)
+
+
+class TestOvsdbClientCommand(base.BaseTestCase):
+    class OvsdbClientTestCommand(utils.OvsdbClientCommand):
+        COMMAND = 'test'
+
+    def setUp(self):
+        super().setUp()
+        self.nb_connection = 'ovn_nb_connection'
+        self.sb_connection = 'ovn_sb_connection'
+
+        ovn_conf.register_opts()
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_nb_connection',
+            self.nb_connection,
+            group='ovn')
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_sb_connection',
+            self.sb_connection,
+            group='ovn')
+        self.m_exec = mock.patch.object(processutils, 'execute').start()
+
+    def assert_exec_call(self, expected):
+        self.m_exec.assert_called_with(
+            *shlex.split(expected), log_errors=processutils.LOG_FINAL_ERROR)
+
+    def test_run_northbound(self):
+        expected = ('ovsdb-client %s %s --timeout 180 '
+                    '\'["OVN_Northbound", "foo"]\'' % (
+                        self.OvsdbClientTestCommand.COMMAND,
+                        self.nb_connection))
+        self.OvsdbClientTestCommand.run(['OVN_Northbound', 'foo'])
+        self.assert_exec_call(expected)
+
+    def test_run_southbound(self):
+        expected = ('ovsdb-client %s %s --timeout 180 '
+                    '\'["OVN_Southbound", "foo"]\'' % (
+                        self.OvsdbClientTestCommand.COMMAND,
+                        self.sb_connection))
+        self.OvsdbClientTestCommand.run(['OVN_Southbound', 'foo'])
+        self.assert_exec_call(expected)
+
+    def test_run_northbound_with_ssl(self):
+        private_key = 'north_pk'
+        certificate = 'north_cert'
+        ca_auth = 'north_ca_auth'
+
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_nb_private_key',
+            private_key,
+            group='ovn')
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_nb_certificate',
+            certificate,
+            group='ovn')
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_nb_ca_cert',
+            ca_auth,
+            group='ovn')
+
+        expected = ('ovsdb-client %s %s --timeout 180 '
+                    '-p %s '
+                    '-c %s '
+                    '-C %s '
+                    '\'["OVN_Northbound", "foo"]\'' % (
+                        self.OvsdbClientTestCommand.COMMAND,
+                        self.nb_connection,
+                        private_key,
+                        certificate,
+                        ca_auth))
+
+        self.OvsdbClientTestCommand.run(['OVN_Northbound', 'foo'])
+        self.assert_exec_call(expected)
+
+    def test_run_southbound_with_ssl(self):
+        private_key = 'north_pk'
+        certificate = 'north_cert'
+        ca_auth = 'north_ca_auth'
+
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_sb_private_key',
+            private_key,
+            group='ovn')
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_sb_certificate',
+            certificate,
+            group='ovn')
+        ovn_conf.cfg.CONF.set_default(
+            'ovn_sb_ca_cert',
+            ca_auth,
+            group='ovn')
+
+        expected = ('ovsdb-client %s %s --timeout 180 '
+                    '-p %s '
+                    '-c %s '
+                    '-C %s '
+                    '\'["OVN_Southbound", "foo"]\'' % (
+                        self.OvsdbClientTestCommand.COMMAND,
+                        self.sb_connection,
+                        private_key,
+                        certificate,
+                        ca_auth))
+
+        self.OvsdbClientTestCommand.run(['OVN_Southbound', 'foo'])
+        self.assert_exec_call(expected)
+
+    def test_run_empty_list(self):
+        with testtools.ExpectedException(KeyError):
+            self.OvsdbClientTestCommand.run([])
+
+    def test_run_bad_schema(self):
+        with testtools.ExpectedException(KeyError):
+            self.OvsdbClientTestCommand.run(['foo'])
+
+
+class GetSubnetsAddressScopeTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.ml2_plugin = mock.Mock()
+
+    def test_no_subnets(self):
+        subnets = []
+        fixed_ips = mock.ANY
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('', ''), (address4, address6))
+
+    def test_no_subnetpool(self):
+        subnets = [
+            {'id': 'subnet1', 'subnetpool_id': None},
+            {'id': 'subnet2', 'subnetpool_id': None},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1'},
+            {'subnet_id': 'subnet2'},
+        ]
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('', ''), (address4, address6))
+
+    def test_no_address_scope(self):
+        subnets = [
+            {'id': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
+            {'id': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1'},
+            {'subnet_id': 'subnet2'},
+        ]
+        self.ml2_plugin.get_subnetpool.side_effect = n_exc.SubnetPoolNotFound(
+            subnetpool_id='snp')
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('', ''), (address4, address6))
+
+    def test_address_scope(self):
+        subnets = [
+            {'id': 'subnet1', 'subnetpool_id': 'pool_ipv4'},
+            {'id': 'subnet2', 'subnetpool_id': 'pool_ipv6'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1'},
+            {'subnet_id': 'subnet2'},
+        ]
+        self.ml2_plugin.get_subnetpool.side_effect = [
+            {'address_scope_id': 'scope4', 'ip_version': n_const.IP_VERSION_4},
+            {'address_scope_id': 'scope6', 'ip_version': n_const.IP_VERSION_6},
+        ]
+        address4, address6 = utils.get_subnets_address_scopes(
+            mock.ANY, subnets, fixed_ips, self.ml2_plugin)
+        self.assertEqual(('scope4', 'scope6'), (address4, address6))
+
+
+class GetPortTypeVirtualAndParentsTestCase(base.BaseTestCase):
+
+    def test_no_subnets(self):
+        subnets = []
+        fixed_ips = []
+        port_type, virtual_ip, virtual_parents = (
+            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
+                                                    'port1', mock.ANY))
+        self.assertEqual(('', None, None),
+                         (port_type, virtual_ip, virtual_parents))
+
+    @mock.patch.object(utils, 'get_virtual_port_parents', return_value=[])
+    def test_no_parents(self, *args):
+        subnets = [
+            {'id': 'subnet1'},
+            {'id': 'subnet2'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1', 'ip_address': '1.2.3.4'},
+            {'subnet_id': 'subnet2', 'ip_address': '1.2.3.5'},
+        ]
+        port_type, virtual_ip, virtual_parents = (
+            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
+                                                    'port1', mock.ANY))
+        self.assertEqual(('', None, None),
+                         (port_type, virtual_ip, virtual_parents))
+
+    @mock.patch.object(utils, 'get_virtual_port_parents',
+                       return_value=['parent1', 'parent2'])
+    def test_with_parents(self, *args):
+        subnets = [
+            {'id': 'subnet1'},
+            {'id': 'subnet2'},
+        ]
+        fixed_ips = [
+            {'subnet_id': 'subnet1', 'ip_address': '1.2.3.4'},
+            {'subnet_id': 'subnet2', 'ip_address': '1.2.3.5'},
+        ]
+        port_type, virtual_ip, virtual_parents = (
+            utils.get_port_type_virtual_and_parents(subnets, fixed_ips, 'net1',
+                                                    'port1', mock.ANY))
+        self.assertEqual((constants.LSP_TYPE_VIRTUAL, '1.2.3.4',
+                          'parent1,parent2'),
+                         (port_type, virtual_ip, virtual_parents))
+
+
+class DetermineBindHostTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.mock_sb_idl = mock.Mock()
+        self.get_chassis_by_card_serial_from_cms_options = (
+            self.mock_sb_idl.get_chassis_by_card_serial_from_cms_options)
+        self.fake_smartnic_hostname = 'fake-chassis-hostname'
+        self.get_chassis_by_card_serial_from_cms_options.return_value = (
+            fakes.FakeChassis.create(
+                attrs={'hostname': self.fake_smartnic_hostname}))
+
+    def test_vnic_normal_unbound_port(self):
+        self.assertEqual(
+            '',
+            utils.determine_bind_host(self.mock_sb_idl, {}))
+
+    def test_vnic_normal_bound_port(self):
+        port = {
+            portbindings.HOST_ID: 'fake-binding-host-id',
+        }
+        self.assertEqual(
+            'fake-binding-host-id',
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_normal_port_context(self):
+        context = mock.MagicMock()
+        context.host = 'fake-binding-host-id'
+        self.assertEqual(
+            'fake-binding-host-id',
+            utils.determine_bind_host(self.mock_sb_idl, {},
+                                      port_context=context))
+
+    def test_vnic_remote_managed_unbound_port_no_binding_profile(self):
+        port = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            constants.OVN_PORT_BINDING_PROFILE: {},
+        }
+        self.assertEqual(
+            '',
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_remote_managed_unbound_port(self):
+        port = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            constants.OVN_PORT_BINDING_PROFILE: {
+                constants.VIF_DETAILS_PCI_VENDOR_INFO: 'fake-pci-vendor-info',
+                constants.VIF_DETAILS_PCI_SLOT: 'fake-pci-slot',
+                constants.VIF_DETAILS_PHYSICAL_NETWORK: None,
+                constants.VIF_DETAILS_CARD_SERIAL_NUMBER: 'fake-serial',
+                constants.VIF_DETAILS_PF_MAC_ADDRESS: 'fake-pf-mac',
+                constants.VIF_DETAILS_VF_NUM: 42,
+            },
+        }
+        self.assertEqual(
+            self.fake_smartnic_hostname,
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_remote_managed_bound_port(self):
+        port = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            portbindings.HOST_ID: 'fake-binding-host-id',
+            constants.OVN_PORT_BINDING_PROFILE: {
+                constants.VIF_DETAILS_PCI_VENDOR_INFO: 'fake-pci-vendor-info',
+                constants.VIF_DETAILS_PCI_SLOT: 'fake-pci-slot',
+                constants.VIF_DETAILS_PHYSICAL_NETWORK: None,
+                constants.VIF_DETAILS_CARD_SERIAL_NUMBER: 'fake-serial',
+                constants.VIF_DETAILS_PF_MAC_ADDRESS: 'fake-pf-mac',
+                constants.VIF_DETAILS_VF_NUM: 42,
+            },
+        }
+        self.assertEqual(
+            self.fake_smartnic_hostname,
+            utils.determine_bind_host(self.mock_sb_idl, port))
+
+    def test_vnic_remote_managed_port_context(self):
+        context = mock.MagicMock()
+        context.current = {
+            portbindings.VNIC_TYPE: portbindings.VNIC_REMOTE_MANAGED,
+            constants.OVN_PORT_BINDING_PROFILE: {
+                constants.VIF_DETAILS_PCI_VENDOR_INFO: 'fake-pci-vendor-info',
+                constants.VIF_DETAILS_PCI_SLOT: 'fake-pci-slot',
+                constants.VIF_DETAILS_PHYSICAL_NETWORK: None,
+                constants.VIF_DETAILS_CARD_SERIAL_NUMBER: 'fake-serial',
+                constants.VIF_DETAILS_PF_MAC_ADDRESS: 'fake-pf-mac',
+                constants.VIF_DETAILS_VF_NUM: 42,
+            },
+        }
+        context.host = 'fake-binding-host-id'
+        self.assertEqual(
+            self.fake_smartnic_hostname,
+            utils.determine_bind_host(self.mock_sb_idl, {},
+                                      port_context=context))
+
+
+class ValidatePortForwardingConfigurationTestCase(base.BaseTestCase):
+
+    def setUp(self):
+        super().setUp()
+        ovn_conf.register_opts()
+
+    def test_validation_when_distributed_fip_disabled(self):
+        cfg.CONF.set_override(
+            'enable_distributed_floating_ip', False, group='ovn')
+        cfg.CONF.set_override('service_plugins', 'some_plugin,port_forwarding')
+        cfg.CONF.set_override('tenant_network_types', 'geneve,vlan',
+                              group='ml2')
+        utils.validate_port_forwarding_configuration()
+
+    def test_validation_when_no_pf_plugin_enabled(self):
+        cfg.CONF.set_override(
+            'enable_distributed_floating_ip', True, group='ovn')
+        cfg.CONF.set_override('service_plugins', 'some_plugin')
+        cfg.CONF.set_override('tenant_network_types', 'geneve,vlan',
+                              group='ml2')
+        utils.validate_port_forwarding_configuration()
+
+    def test_validation_when_no_provider_net_configured(self):
+        cfg.CONF.set_override(
+            'enable_distributed_floating_ip', True, group='ovn')
+        cfg.CONF.set_override('service_plugins', 'some_plugin,port_forwarding')
+        cfg.CONF.set_override('tenant_network_types', 'geneve,vxlan',
+                              group='ml2')
+        utils.validate_port_forwarding_configuration()
+
+    def test_validation_when_pf_and_provider_net_enabled(self):
+        cfg.CONF.set_override(
+            'enable_distributed_floating_ip', True, group='ovn')
+        cfg.CONF.set_override('service_plugins', 'some_plugin,port_forwarding')
+        cfg.CONF.set_override('tenant_network_types', 'geneve,vlan',
+                              group='ml2')
+        self.assertRaises(
+            ovn_exc.InvalidPortForwardingConfiguration,
+            utils.validate_port_forwarding_configuration)
