@@ -114,6 +114,30 @@ class SystemAdminTests(RouterAPITestCase):
             'create_router:external_gateway_info:external_fixed_ips',
             self.alt_target)
 
+    def test_create_router_enable_default_route_bfd(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_router:enable_default_route_bfd',
+            self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_router:enable_default_route_bfd',
+            self.alt_target)
+
+    def test_create_router_enable_default_route_ecmp(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_router:enable_default_route_ecmp',
+            self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'create_router:enable_default_route_ecmp',
+            self.alt_target)
+
     def test_get_router(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -144,6 +168,16 @@ class SystemAdminTests(RouterAPITestCase):
             policy.enforce,
             self.context, 'get_router:ha', self.alt_target)
 
+    def test_get_routers_tags(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'get_routers_tags', self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'get_routers_tags', self.alt_target)
+
     def test_update_router(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -224,6 +258,40 @@ class SystemAdminTests(RouterAPITestCase):
             'update_router:external_gateway_info:external_fixed_ips',
             self.alt_target)
 
+    def test_update_router_enable_default_route_bfd(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'update_router:enable_default_route_bfd',
+            self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'update_router:enable_default_route_bfd',
+            self.alt_target)
+
+    def test_update_router_enable_default_route_ecmp(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'update_router:enable_default_route_ecmp',
+            self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'update_router:enable_default_route_ecmp',
+            self.alt_target)
+
+    def test_update_routers_tags(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'update_routers_tags', self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'update_routers_tags', self.alt_target)
+
     def test_delete_router(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -234,6 +302,16 @@ class SystemAdminTests(RouterAPITestCase):
             policy.enforce,
             self.context, 'delete_router', self.alt_target)
 
+    def test_delete_routers_tags(self):
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'delete_routers_tags', self.target)
+        self.assertRaises(
+            base_policy.InvalidScope,
+            policy.enforce,
+            self.context, 'delete_routers_tags', self.alt_target)
+
     def test_add_router_interface(self):
         self.assertRaises(
             base_policy.InvalidScope,
@@ -337,6 +415,30 @@ class AdminTests(RouterAPITestCase):
                 'create_router:external_gateway_info:external_fixed_ips',
                 self.alt_target))
 
+    def test_update_router_enable_default_route_bfd(self):
+        self.assertTrue(
+            policy.enforce(
+                self.context,
+                'update_router:enable_default_route_bfd',
+                self.target))
+        self.assertTrue(
+            policy.enforce(
+                self.context,
+                'update_router:enable_default_route_bfd',
+                self.alt_target))
+
+    def test_update_router_enable_default_route_ecmp(self):
+        self.assertTrue(
+            policy.enforce(
+                self.context,
+                'update_router:enable_default_route_ecmp',
+                self.target))
+        self.assertTrue(
+            policy.enforce(
+                self.context,
+                'update_router:enable_default_route_ecmp',
+                self.alt_target))
+
     def test_get_router(self):
         self.assertTrue(
             policy.enforce(self.context, 'get_router', self.target))
@@ -357,6 +459,12 @@ class AdminTests(RouterAPITestCase):
         self.assertTrue(
             policy.enforce(self.context, 'get_router:ha', self.alt_target))
 
+    def test_get_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'get_routers_tags', self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'get_routers_tags', self.alt_target))
+
     def test_update_router(self):
         self.assertTrue(
             policy.enforce(self.context, 'update_router', self.target))
@@ -419,12 +527,26 @@ class AdminTests(RouterAPITestCase):
                 'update_router:external_gateway_info:external_fixed_ips',
                 self.alt_target))
 
+    def test_update_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'update_routers_tags', self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'update_routers_tags',
+                           self.alt_target))
+
     def test_delete_router(self):
         self.assertTrue(
             policy.enforce(self.context, 'delete_router', self.target))
         self.assertTrue(
             policy.enforce(self.context, 'delete_router', self.alt_target))
 
+    def test_delete_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'delete_routers_tags', self.target))
+        self.assertTrue(
+            policy.enforce(self.context, 'delete_routers_tags',
+                           self.alt_target))
+
     def test_add_router_interface(self):
         self.assertTrue(
             policy.enforce(self.context,
@@ -524,6 +646,34 @@ class ProjectMemberTests(AdminTests):
             'create_router:external_gateway_info:external_fixed_ips',
             self.alt_target)
 
+    def test_update_router_enable_default_route_bfd(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context,
+            'update_router:enable_default_route_bfd',
+            self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context,
+            'update_router:enable_default_route_bfd',
+            self.alt_target)
+
+    def test_update_router_enable_default_route_ecmp(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context,
+            'update_router:enable_default_route_ecmp',
+            self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context,
+            'update_router:enable_default_route_ecmp',
+            self.alt_target)
+
     def test_get_router(self):
         self.assertTrue(
             policy.enforce(self.context, 'get_router', self.target))
@@ -552,6 +702,14 @@ class ProjectMemberTests(AdminTests):
             policy.enforce,
             self.context, 'get_router:ha', self.alt_target)
 
+    def test_get_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'get_routers_tags', self.target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'get_routers_tags', self.alt_target)
+
     def test_update_router(self):
         self.assertTrue(
             policy.enforce(self.context, 'update_router', self.target))
@@ -628,6 +786,14 @@ class ProjectMemberTests(AdminTests):
             'update_router:external_gateway_info:external_fixed_ips',
             self.alt_target)
 
+    def test_update_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'update_routers_tags', self.target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_routers_tags', self.alt_target)
+
     def test_delete_router(self):
         self.assertTrue(
             policy.enforce(self.context, 'delete_router', self.target))
@@ -636,6 +802,14 @@ class ProjectMemberTests(AdminTests):
             policy.enforce,
             self.context, 'delete_router', self.alt_target)
 
+    def test_delete_routers_tags(self):
+        self.assertTrue(
+            policy.enforce(self.context, 'delete_routers_tags', self.target))
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_routers_tags', self.alt_target)
+
     def test_add_router_interface(self):
         self.assertTrue(
             policy.enforce(self.context,
@@ -729,6 +903,16 @@ class ProjectReaderTests(ProjectMemberTests):
             self.context, 'update_router:external_gateway_info:network_id',
             self.alt_target)
 
+    def test_update_routers_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_routers_tags', self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'update_routers_tags', self.alt_target)
+
     def test_delete_router(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
@@ -739,6 +923,16 @@ class ProjectReaderTests(ProjectMemberTests):
             policy.enforce,
             self.context, 'delete_router', self.alt_target)
 
+    def test_delete_routers_tags(self):
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_routers_tags', self.target)
+        self.assertRaises(
+            base_policy.PolicyNotAuthorized,
+            policy.enforce,
+            self.context, 'delete_routers_tags', self.alt_target)
+
     def test_add_router_interface(self):
         self.assertRaises(
             base_policy.PolicyNotAuthorized,
