@@ -32,11 +32,11 @@ from neutron_lib.plugins import directory
 from neutron_lib.plugins import utils as plugin_utils
 from oslo_utils import uuidutils
 import testtools
-import webob.exc
 
 from neutron.db import extraroute_db
 from neutron.db import l3_db
 from neutron.db.models import l3 as l3_models
+from neutron.db.models import l3_attrs
 from neutron.db import models_v2
 from neutron.extensions import segment as segment_ext
 from neutron.objects import base as base_obj
@@ -251,27 +251,6 @@ class TestL3_NAT_dbonly_mixin(
             'fixed_ip_address_scope': mock.sentinel.address_scope_id,
             'id': mock.sentinel.fip_ip}, result)
 
-    def test__unique_floatingip_iterator(self):
-        context = mock.MagicMock()
-        query = mock.MagicMock()
-        query.order_by().__iter__.return_value = [
-            ({'id': 'id1'}, 'scope1'),
-            ({'id': 'id1'}, 'scope1'),
-            ({'id': 'id2'}, 'scope2'),
-            ({'id': 'id2'}, 'scope2'),
-            ({'id': 'id2'}, 'scope2'),
-            ({'id': 'id3'}, 'scope3')]
-        query.reset_mock()
-        with mock.patch.object(
-                l3_obj.FloatingIP, '_load_object',
-                side_effect=({'id': 'id1'}, {'id': 'id2'}, {'id': 'id3'})):
-            result = list(
-                l3_obj.FloatingIP._unique_floatingip_iterator(context, query))
-            query.order_by.assert_called_once_with(l3_models.FloatingIP.id)
-            self.assertEqual([({'id': 'id1'}, 'scope1'),
-                              ({'id': 'id2'}, 'scope2'),
-                              ({'id': 'id3'}, 'scope3')], result)
-
     @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_deletion_port_not_found(self, gp):
         # port not found doesn't prevent
@@ -289,9 +268,10 @@ class TestL3_NAT_dbonly_mixin(
         # without fixed IPs is allowed
         gp.return_value.get_port.return_value = {
             'device_owner': n_const.DEVICE_OWNER_ROUTER_INTF, 'fixed_ips': [],
-            'id': 'f'
+            'device_id': '44', 'id': 'f',
         }
-        self.db.prevent_l3_port_deletion(None, None)
+        with testtools.ExpectedException(n_exc.ServicePortInUse):
+            self.db.prevent_l3_port_deletion(mock.Mock(), None)
 
     @mock.patch.object(directory, 'get_plugin')
     def test_prevent_l3_port_no_router(self, gp):
@@ -327,6 +307,46 @@ class TestL3_NAT_dbonly_mixin(
 
             self.db.prevent_l3_port_deletion(ctx, None)
 
+    @mock.patch.object(l3_obj.FloatingIP, 'objects_exist')
+    @mock.patch.object(l3_obj.FloatingIP, 'get_objects')
+    def test_prevent_internal_ip_change_for_fip(self,
+                                                get_objects,
+                                                objects_exist):
+        ctx = context.get_admin_context()
+        port_id = 'test_internal_port'
+        new_fixed_ips = [{'subnet_id': 'test_subnet',
+                          'ip_address': '192.168.2.110'}]
+        fip_obj_dict = {'fixed_ip_address': '192.168.2.120',
+                        'id': 'floating_ip1',
+                        'port_id': port_id}
+        fip_obj = mock.Mock(**fip_obj_dict)
+        objects_exist.return_value = True
+        get_objects.return_value = [fip_obj]
+        with testtools.ExpectedException(n_exc.BadRequest):
+            self.db.prevent_internal_ip_change_for_fip(ctx, port_id,
+                                                       new_fixed_ips)
+
+    @mock.patch.object(l3_obj.FloatingIP, 'objects_exist')
+    @mock.patch.object(l3_obj.FloatingIP, 'get_objects')
+    def test_disassociate_floatingips_conflict_by_fip_attached(self,
+                                                               get_objects,
+                                                               objects_exist):
+        context_tenant = context.Context('tenant', 'tenant', is_admin=False)
+        objects_exist.return_value = True
+        get_objects.side_effect = [
+            [],
+            [{'id': 'floating_ip1', 'port_id': 'port_id'}]]
+        self.assertRaises(l3_db.FipAssociated,
+                          self.db.disassociate_floatingips,
+                          context_tenant,
+                          'port_id')
+        objects_exist.assert_called_once_with(
+            mock.ANY, fixed_port_id='port_id')
+        expected_calls = [
+                mock.call(context_tenant, fixed_port_id='port_id'),
+                mock.call(mock.ANY, fixed_port_id='port_id')]
+        get_objects.assert_has_calls(expected_calls)
+
     @mock.patch.object(directory, 'get_plugin')
     def test_subscribe_address_scope_of_subnetpool(self, gp):
         l3_db.L3RpcNotifierMixin()
@@ -514,6 +534,140 @@ class TestL3_NAT_dbonly_mixin(
             self.assertIsNone(
                 self.db._validate_gw_info(mock.ANY, info, [], router))
 
+    def test__raise_on_subnets_overlap_does_not_raise(self):
+        subnets = [
+            {'id': uuidutils.generate_uuid(),
+             'cidr': '10.1.0.0/24'},
+            {'id': uuidutils.generate_uuid(),
+             'cidr': '10.2.0.0/24'}]
+        self.db._raise_on_subnets_overlap(subnets[0], subnets[1])
+
+    def test__raise_on_subnets_overlap_raises(self):
+        subnets = [
+            {'id': uuidutils.generate_uuid(),
+             'cidr': '10.1.0.0/20'},
+            {'id': uuidutils.generate_uuid(),
+             'cidr': '10.1.10.0/24'}]
+        self.assertRaises(
+            n_exc.BadRequest, self.db._raise_on_subnets_overlap, subnets[0],
+            subnets[1])
+
+    def test__validate_one_router_ipv6_port_per_network(self):
+        port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 1),
+                    subnet_id='foo_subnet')])
+        rports = [l3_models.RouterPort(router_id='foo_router', port=port)]
+        router = l3_models.Router(
+            id='foo_router', attached_ports=rports, route_list=[],
+            gw_port_id=None)
+        new_port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network2',
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 2),
+                    subnet_id='foo_subnet')])
+        self.db._validate_one_router_ipv6_port_per_network(
+            router, new_port)
+
+    def test__validate_one_router_ipv6_port_per_network_mix_ipv4_ipv6(self):
+        port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '10.1.10.0/24').ip + 1),
+                    subnet_id='foo_subnet')])
+        rports = [l3_models.RouterPort(router_id='foo_router', port=port)]
+        router = l3_models.Router(
+            id='foo_router', attached_ports=rports, route_list=[],
+            gw_port_id=None)
+        new_port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 2),
+                    subnet_id='foo_subnet')])
+        self.db._validate_one_router_ipv6_port_per_network(
+            router, new_port)
+
+    def test__validate_one_router_ipv6_port_per_network_distributed_port(self):
+        port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                device_owner=n_const.DEVICE_OWNER_DVR_INTERFACE,
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 1),
+                    subnet_id='foo_subnet')])
+        rports = [l3_models.RouterPort(router_id='foo_router', port=port)]
+        router = l3_models.Router(
+            id='foo_router', attached_ports=rports, route_list=[],
+            gw_port_id=None)
+        new_port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                device_owner=n_const.DEVICE_OWNER_ROUTER_SNAT,
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 2),
+                    subnet_id='foo_subnet')])
+        self.db._validate_one_router_ipv6_port_per_network(router, new_port)
+
+    def test__validate_one_router_ipv6_port_per_network_centralized_snat_port(
+            self):
+        port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                device_owner=n_const.DEVICE_OWNER_ROUTER_SNAT,
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 1),
+                    subnet_id='foo_subnet')])
+        rports = [l3_models.RouterPort(router_id='foo_router', port=port)]
+        router = l3_models.Router(
+            id='foo_router', attached_ports=rports, route_list=[],
+            gw_port_id=None)
+        new_port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                device_owner=n_const.DEVICE_OWNER_DVR_INTERFACE,
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 2),
+                    subnet_id='foo_subnet')])
+        self.db._validate_one_router_ipv6_port_per_network(router, new_port)
+
+    def test__validate_one_router_ipv6_port_per_network_failed(self):
+        port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 1),
+                    subnet_id='foo_subnet')])
+        rports = [l3_models.RouterPort(router_id='foo_router', port=port)]
+        router = l3_models.Router(
+            id='foo_router', attached_ports=rports, route_list=[],
+            gw_port_id=None)
+        new_port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                network_id='foo_network',
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(netaddr.IPNetwork(
+                        '2001:db8::/32').ip + 2),
+                    subnet_id='foo_subnet')])
+        self.assertRaises(
+            n_exc.BadRequest,
+            self.db._validate_one_router_ipv6_port_per_network,
+            router,
+            new_port)
+
 
 class L3_NAT_db_mixin(base.BaseTestCase):
     def setUp(self):
@@ -697,6 +851,56 @@ class L3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         mock_log.warning.not_called_once()
         self._check_routerports((True, False))
 
+    @mock.patch.object(l3_db.L3_NAT_dbonly_mixin,
+                       '_check_for_dup_router_subnets')
+    @mock.patch.object(l3_db.L3_NAT_dbonly_mixin,
+                       '_raise_on_subnets_overlap')
+    def test_add_router_interface_by_port_overlap_detected(
+            self, mock_raise_on_subnets_overlap, mock_check_dup):
+        # NOTE(froyo): On a normal behaviour this overlapping would be detected
+        # by _check_for_dup_router_subnets, in order to evalue the code
+        # implemented to cover the race condition when two ports are added
+        # simultaneously using colliding cidrs we need to "fake" this method
+        # to overpass it and check we achieve the code part that cover the case
+        mock_check_dup.return_value = True
+        network2 = self.create_network('network2')
+        subnet = self.create_subnet(network2, '1.1.1.1', '1.1.1.0/24')
+        ipa = str(netaddr.IPNetwork(subnet['subnet']['cidr']).ip + 10)
+        fixed_ips = [{'subnet_id': subnet['subnet']['id'], 'ip_address': ipa}]
+        port = self.create_port(
+                network2['network']['id'], {'fixed_ips': fixed_ips})
+        self.mixin.add_router_interface(
+            self.ctx, self.router['id'],
+            interface_info={'port_id': port['port']['id']})
+        mock_raise_on_subnets_overlap.assert_not_called()
+        self.mixin.add_router_interface(
+            self.ctx, self.router['id'],
+            interface_info={'port_id': self.ports[0]['port']['id']})
+        mock_raise_on_subnets_overlap.assert_called_once()
+
+    @mock.patch.object(l3_db.L3_NAT_dbonly_mixin,
+                       '_check_for_dup_router_subnets')
+    @mock.patch.object(l3_db.L3_NAT_dbonly_mixin,
+                       '_raise_on_subnets_overlap')
+    def test_add_router_interface_by_subnet_overlap_detected(
+            self, mock_raise_on_subnets_overlap, mock_check_dup):
+        # NOTE(froyo): On a normal behaviour this overlapping would be detected
+        # by _check_for_dup_router_subnets, in order to evalue the code
+        # implemented to cover the race condition when two ports are added
+        # simultaneously using colliding cidrs we need to "fake" this method
+        # to overpass it and check we achieve the code part that cover the case
+        mock_check_dup.return_value = True
+        network2 = self.create_network('network2')
+        subnet = self.create_subnet(network2, '1.1.1.1', '1.1.1.0/24')
+        self.mixin.add_router_interface(
+            self.ctx, self.router['id'],
+            interface_info={'subnet_id': subnet['subnet']['id']})
+        mock_raise_on_subnets_overlap.assert_not_called()
+        self.mixin.add_router_interface(
+            self.ctx, self.router['id'],
+            interface_info={'subnet_id': self.subnets[0]['subnet']['id']})
+        mock_raise_on_subnets_overlap.assert_called_once()
+
     @mock.patch.object(port_obj, 'LOG')
     def test_remove_router_interface_by_subnet_removed_rport(self, mock_log):
         self._add_router_interfaces()
@@ -732,6 +936,20 @@ class L3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             ]
             mock_publish.assert_has_calls(expected_calls)
 
+    def test_create_router_extra_attr(self):
+        router_args = {'router': {'name': 'foo_router',
+                                  'admin_state_up': True,
+                                  'tenant_id': 'foo_tenant'}
+                       }
+        router_dict = self.create_router(router_args)
+        with db_api.CONTEXT_READER.using(self.ctx) as session:
+            r_extra_attrs = session.query(
+                l3_attrs.RouterExtraAttributes).filter(
+                    l3_attrs.RouterExtraAttributes.router_id ==
+                    router_dict['id']).all()
+        self.assertEqual(1, len(r_extra_attrs))
+        self.assertEqual(router_dict['id'], r_extra_attrs[0].router_id)
+
     def test_update_router_notify(self):
         with mock.patch.object(l3_db.registry, 'publish') as mock_publish:
             self.mixin.update_router(self.ctx, self.router['id'],
@@ -750,9 +968,9 @@ class L3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         with db_api.CONTEXT_WRITER.using(self.ctx):
             res = self._create_network(
                 self.fmt, name, True,
-                arg_list=(extnet_apidef.EXTERNAL,), **kwargs)
-            if res.status_int >= webob.exc.HTTPClientError.code:
-                raise webob.exc.HTTPClientError(code=res.status_int)
+                arg_list=(extnet_apidef.EXTERNAL,),
+                as_admin=True, **kwargs)
+            self._check_http_response(res)
             return self.deserialize(self.fmt, res)
 
     def test_update_router_gw_notify(self):
