@@ -26,7 +26,6 @@ from ovs import poller
 from ovs.stream import Stream
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import idlutils
-import testtools
 
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import hash_ring_manager
@@ -61,7 +60,9 @@ OVN_NB_SCHEMA = {
                 "port_security": {"type": {"key": "string",
                                            "min": 0,
                                            "max": "unlimited"}},
-                "up": {"type": {"key": "boolean", "min": 0, "max": 1}}},
+                "up": {"type": {"key": "boolean", "min": 0, "max": 1}},
+                "enabled": {"type": {"key": "boolean", "min": 0, "max": 1}},
+            },
             "indexes": [["name"]],
             "isRoot": False,
         },
@@ -82,6 +83,9 @@ OVN_SB_SCHEMA = {
                 "name": {"type": "string"},
                 "hostname": {"type": "string"},
                 "external_ids": {
+                    "type": {"key": "string", "value": "string",
+                             "min": 0, "max": "unlimited"}},
+                "other_config": {
                     "type": {"key": "string", "value": "string",
                              "min": 0, "max": "unlimited"}}},
             "isRoot": True,
@@ -355,6 +359,7 @@ class TestPortBindingChassisUpdateEvent(base.BaseTestCase):
             self.driver.set_port_status_up.assert_called()
         else:
             self.driver.set_port_status_up.assert_not_called()
+        self.driver.set_port_status_up.reset_mock()
 
     def test_event_matches(self):
         # NOTE(twilson) This primarily tests implementation details. If a
@@ -363,14 +368,88 @@ class TestPortBindingChassisUpdateEvent(base.BaseTestCase):
         pbtable = fakes.FakeOvsdbTable.create_one_ovsdb_table(
             attrs={'name': 'Port_Binding'})
         ovsdb_row = fakes.FakeOvsdbRow.create_one_ovsdb_row
-        self.driver.nb_ovn.lookup.return_value = ovsdb_row(attrs={'up': True})
+        self.driver.nb_ovn.lookup.return_value = ovsdb_row(
+            attrs={'up': True, 'enabled': True})
+
+        # Port binding change.
         self._test_event(
             self.event.ROW_UPDATE,
             ovsdb_row(attrs={'_table': pbtable, 'chassis': 'one',
-                             'type': '_fake_', 'logical_port': 'foo'}),
+                             'type': '_fake_', 'logical_port': 'foo',
+                             'options': {}}),
             ovsdb_row(attrs={'_table': pbtable, 'chassis': 'two',
                              'type': '_fake_'}))
 
+        # Port binding change because of a live migration in progress.
+        options = {
+            ovn_const.LSP_OPTIONS_REQUESTED_CHASSIS_KEY: 'chassis1,chassis2'}
+        self._test_event(
+            self.event.ROW_UPDATE,
+            ovsdb_row(attrs={'_table': pbtable, 'chassis': 'one',
+                             'type': '_fake_', 'logical_port': 'foo',
+                             'options': options}),
+            ovsdb_row(attrs={'_table': pbtable, 'chassis': 'two',
+                             'type': '_fake_'}))
+
+
+class TestPortBindingUpdateVirtualPortsEvent(base.BaseTestCase):
+    def setUp(self):
+        super().setUp()
+        self.event = ovsdb_monitor.PortBindingUpdateVirtualPortsEvent(None)
+
+        self.pbtable = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'name': 'Port_Binding'})
+        self.ovsdb_row = fakes.FakeOvsdbRow.create_one_ovsdb_row
+
+        self.row = self.ovsdb_row(
+            attrs={'_table': self.pbtable,
+                   'chassis': 'newchassis',
+                   'options': {
+                       'virtual-parents': 'uuid1,uuid2'}})
+
+    def test_delete_event_matches(self):
+        # Delete event (only type virtual).
+        self.assertFalse(self.event.match_fn(
+            self.event.ROW_DELETE,
+            self.ovsdb_row(attrs={'_table': self.pbtable, 'type': '_fake_'}),
+            None))
+        self.assertTrue(self.event.match_fn(
+            self.event.ROW_DELETE,
+            self.ovsdb_row(attrs={'_table': self.pbtable, 'type': 'virtual'}),
+            None))
+
+    def test_event_no_match_no_options(self):
+        # Unrelated portbind change (no options in old, so no virtual parents)
+        self.assertFalse(self.event.match_fn(
+            self.event.ROW_UPDATE, self.row,
+            self.ovsdb_row(attrs={'_table': self.pbtable,
+                                  'name': 'somename'})))
+
+    def test_event_no_match_other_options_change(self):
+        # Non-virtual parent change, no chassis has changed
+        old = self.ovsdb_row(attrs={'_table': self.pbtable,
+                            'options': {
+                                'virtual-parents': 'uuid1,uuid2',
+                                'other-opt': '_fake_'}})
+
+        self.assertFalse(self.event.match_fn(self.event.ROW_UPDATE,
+                                             self.row, old))
+
+    def test_event_match_chassis_change(self):
+        # Port binding change (chassis changed, and marked in old)
+        self.assertTrue(self.event.match_fn(
+            self.event.ROW_UPDATE, self.row,
+            self.ovsdb_row(attrs={'_table': self.pbtable,
+                                  'chassis': 'fakechassis'})))
+
+    def test_event_match_virtual_parent_change(self):
+        # Virtual parent change
+        old = self.ovsdb_row(attrs={'_table': self.pbtable,
+                                    'options': {
+                                        'virtual-parents': 'uuid1,uuid3'}})
+        self.assertTrue(self.event.match_fn(self.event.ROW_UPDATE,
+                                            self.row, old))
+
 
 class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
@@ -405,28 +484,46 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         # Execute the notifications queued
         self.idl.notify_handler.notify_loop()
 
-    def test_lsp_up_create_event(self):
-        row_data = {"up": True, "name": "foo-name"}
+    def test_lsp_create_event(self):
+        row_data = {'name': 'foo'}
+
+        # up and enabled
+        row_data.update({'up': True, 'enabled': True})
         self._test_lsp_helper('create', row_data)
-        self.mech_driver.set_port_status_up.assert_called_once_with("foo-name")
+        self.mech_driver.set_port_status_up.assert_called_once_with('foo')
         self.assertFalse(self.mech_driver.set_port_status_down.called)
+        self.mech_driver.set_port_status_up.reset_mock()
 
-    def test_lsp_down_create_event(self):
-        row_data = {"up": False, "name": "foo-name"}
-        self._test_lsp_helper('create', row_data)
-        self.mech_driver.set_port_status_down.assert_called_once_with(
-            "foo-name")
-        self.assertFalse(self.mech_driver.set_port_status_up.called)
-
-    def test_lsp_up_not_set_event(self):
-        row_data = {"up": ['set', []], "name": "foo-name"}
+        # up and disabled
+        row_data.update({'up': True, 'enabled': False})
         self._test_lsp_helper('create', row_data)
         self.assertFalse(self.mech_driver.set_port_status_up.called)
-        self.assertFalse(self.mech_driver.set_port_status_down.called)
+        self.mech_driver.set_port_status_down.assert_called_once_with('foo')
+        self.mech_driver.set_port_status_down.reset_mock()
+
+        # down and enabled
+        row_data.update({'up': False, 'enabled': True})
+        self._test_lsp_helper('create', row_data)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.mech_driver.set_port_status_down.assert_called_once_with('foo')
+        self.mech_driver.set_port_status_down.reset_mock()
+
+        # down and disabled
+        row_data.update({'up': False, 'enabled': False})
+        self._test_lsp_helper('create', row_data)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.mech_driver.set_port_status_down.assert_called_once_with('foo')
+        self.mech_driver.set_port_status_down.reset_mock()
+
+        # Not set to up
+        row_data.update({'up': ['set', []], 'enabled': True})
+        self._test_lsp_helper('create', row_data)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.mech_driver.set_port_status_down.assert_called_once_with('foo')
 
     def test_unwatch_logical_switch_port_create_events(self):
         self.idl.unwatch_logical_switch_port_create_events()
-        row_data = {"up": True, "name": "foo-name"}
+        row_data = {'up': False, 'name': 'foo-name'}
         self._test_lsp_helper('create', row_data)
         self.assertFalse(self.mech_driver.set_port_status_up.called)
         self.assertFalse(self.mech_driver.set_port_status_down.called)
@@ -438,11 +535,10 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
     def test_post_connect(self):
         self.idl.post_connect()
-        self.assertIsNone(self.idl._lsp_create_up_event)
-        self.assertIsNone(self.idl._lsp_create_down_event)
+        self.assertIsNone(getattr(self.idl, '_lsp_create_event', None))
 
     def test_lsp_up_update_event(self):
-        new_row_json = {"up": True, "name": "foo-name"}
+        new_row_json = {'up': True, 'enabled': True, 'name': 'foo-name'}
         old_row_json = {"up": False}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=old_row_json)
@@ -450,7 +546,7 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_lsp_down_update_event(self):
-        new_row_json = {"up": False, "name": "foo-name"}
+        new_row_json = {'up': False, 'enabled': False, 'name': 'foo-name'}
         old_row_json = {"up": True}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=old_row_json)
@@ -459,7 +555,7 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self.assertFalse(self.mech_driver.set_port_status_up.called)
 
     def test_lsp_up_update_event_no_old_data(self):
-        new_row_json = {"up": True, "name": "foo-name"}
+        new_row_json = {'up': True, 'enabled': True, 'name': 'foo-name'}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=None)
         self.assertFalse(self.mech_driver.set_port_status_up.called)
@@ -522,7 +618,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self.row_json = {
             "name": "fake-name",
             "hostname": "fake-hostname",
-            "external_ids": ['map', [["ovn-bridge-mappings",
+            "other_config": ['map', [["ovn-bridge-mappings",
                                       "fake-phynet1:fake-br1"]]]
         }
         self._mock_hash_ring = mock.patch.object(
@@ -546,14 +642,18 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self.sb_idl.notify_handler.notify_loop()
 
     def test_chassis_create_event(self):
-        self._test_chassis_helper('create', self.row_json)
+        old_row_json = {'other_config': ['map', []]}
+        self._test_chassis_helper('create', self.row_json,
+                                  old_row_json=old_row_json)
         self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
 
     def test_chassis_delete_event(self):
-        self._test_chassis_helper('delete', self.row_json)
+        old_row_json = {'other_config': ['map', []]}
+        self._test_chassis_helper('delete', self.row_json,
+                                  old_row_json=old_row_json)
         self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', [])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
@@ -561,7 +661,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
     def test_chassis_update_event(self):
         old_row_json = copy.deepcopy(self.row_json)
-        old_row_json['external_ids'][1][0][1] = (
+        old_row_json['other_config'][1][0][1] = (
             "fake-phynet2:fake-br2")
         self._test_chassis_helper('update', self.row_json, old_row_json)
         self.mech_driver.update_segment_host_mapping.assert_called_once_with(
@@ -570,9 +670,9 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
             event_from_chassis=None)
 
     def test_chassis_update_event_reschedule_not_needed(self):
-        self.row_json['external_ids'][1].append(['foo_field', 'foo_value_new'])
+        self.row_json['other_config'][1].append(['foo_field', 'foo_value_new'])
         old_row_json = copy.deepcopy(self.row_json)
-        old_row_json['external_ids'][1][1][1] = (
+        old_row_json['other_config'][1][1][1] = (
             "foo_value")
         self._test_chassis_helper('update', self.row_json, old_row_json)
         self.mech_driver.update_segment_host_mapping.assert_not_called()
@@ -580,14 +680,14 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
     def test_chassis_update_event_reschedule_lost_physnet(self):
         old_row_json = copy.deepcopy(self.row_json)
-        self.row_json['external_ids'][1][0][1] = ''
+        self.row_json['other_config'][1][0][1] = ''
         self._test_chassis_helper('update', self.row_json, old_row_json)
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis='fake-name')
 
     def test_chassis_update_event_reschedule_add_physnet(self):
         old_row_json = copy.deepcopy(self.row_json)
-        self.row_json['external_ids'][1][0][1] += ',foo_physnet:foo_br'
+        self.row_json['other_config'][1][0][1] += ',foo_physnet:foo_br'
         self._test_chassis_helper('update', self.row_json, old_row_json)
         self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1', 'foo_physnet'])
@@ -596,7 +696,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
     def test_chassis_update_event_reschedule_add_and_remove_physnet(self):
         old_row_json = copy.deepcopy(self.row_json)
-        self.row_json['external_ids'][1][0][1] = 'foo_physnet:foo_br'
+        self.row_json['other_config'][1][0][1] = 'foo_physnet:foo_br'
         self._test_chassis_helper('update', self.row_json, old_row_json)
         self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['foo_physnet'])
@@ -605,7 +705,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
     def test_chassis_update_empty_no_external_ids(self):
         old_row_json = copy.deepcopy(self.row_json)
-        old_row_json.pop('external_ids')
+        old_row_json.pop('other_config')
         with mock.patch(
             'neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
             'ovsdb_monitor.ChassisEvent.'
@@ -638,10 +738,10 @@ class TestChassisEvent(base.BaseTestCase):
 
     def _test_handle_ha_chassis_group_changes_create(self, event):
         # Chassis
-        ext_ids = {
+        other_config = {
             'ovn-cms-options': 'enable-chassis-as-gw,availability-zones=az-0'}
         row = fakes.FakeOvsdbTable.create_one_ovsdb_table(
-            attrs={'name': 'SpongeBob', 'external_ids': ext_ids})
+            attrs={'name': 'SpongeBob', 'other_config': other_config})
         # HA Chassis
         ch0 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
             attrs={'priority': 10})
@@ -673,10 +773,10 @@ class TestChassisEvent(base.BaseTestCase):
 
     def _test_handle_ha_chassis_group_changes_delete(self, event):
         # Chassis
-        ext_ids = {
+        other_config = {
             'ovn-cms-options': 'enable-chassis-as-gw,availability-zones=az-0'}
         row = fakes.FakeOvsdbTable.create_one_ovsdb_table(
-            attrs={'name': 'SpongeBob', 'external_ids': ext_ids})
+            attrs={'name': 'SpongeBob', 'other_config': other_config})
         # HA Chassis
         ha_ch = fakes.FakeOvsdbTable.create_one_ovsdb_table(
             attrs={'priority': 10})
@@ -700,10 +800,10 @@ class TestChassisEvent(base.BaseTestCase):
     def test_handle_ha_chassis_group_changes_update_no_changes(self):
         # Assert nothing was done because the update didn't
         # change the gateway chassis status or the availability zones
-        ext_ids = {
+        other_config = {
             'ovn-cms-options': 'enable-chassis-as-gw,availability-zones=az-0'}
         new = fakes.FakeOvsdbTable.create_one_ovsdb_table(
-            attrs={'name': 'SpongeBob', 'external_ids': ext_ids})
+            attrs={'name': 'SpongeBob', 'other_config': other_config})
         old = new
         self.assertIsNone(self.event.handle_ha_chassis_group_changes(
             self.event.ROW_UPDATE, new, old))
@@ -723,36 +823,3 @@ class TestChassisEvent(base.BaseTestCase):
         # after it became a Gateway chassis
         self._test_handle_ha_chassis_group_changes_create(
             self.event.ROW_UPDATE)
-
-
-class TestShortLivingOvsdbApi(base.BaseTestCase):
-    def setUp(self):
-        ovn_conf.register_opts()
-        super().setUp()
-
-    def test_context(self):
-        api_class = mock.Mock()
-        idl = mock.Mock()
-        with ovsdb_monitor.short_living_ovsdb_api(api_class, idl) as api:
-            self.assertEqual(api_class.return_value, api)
-        api.ovsdb_connection.stop.assert_called_once_with()
-
-    def test_context_error(self):
-        api_class = mock.Mock()
-        idl = mock.Mock()
-        exc = RuntimeError()
-        try:
-            with ovsdb_monitor.short_living_ovsdb_api(api_class, idl) as api:
-                self.assertEqual(api_class.return_value, api)
-                raise exc
-        except RuntimeError as re:
-            self.assertIs(exc, re)
-        api.ovsdb_connection.stop.assert_called_once_with()
-
-    def test_api_class_error(self):
-        api_class = mock.Mock(side_effect=RuntimeError())
-        idl = mock.Mock()
-        with testtools.ExpectedException(RuntimeError):
-            with ovsdb_monitor.short_living_ovsdb_api(api_class, idl):
-                # Make sure it never enter the api context
-                raise Exception("API class instantiated but it should not")
