@@ -16,6 +16,7 @@ import copy
 from unittest import mock
 
 from neutron_lib.api.definitions import external_net
+from neutron_lib.api.definitions import l3 as l3_def
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib.callbacks import events
@@ -26,12 +27,19 @@ from neutron_lib.exceptions import availability_zone as az_exc
 from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
+from neutron_lib.services.qos import constants as qos_consts
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
+from neutron.api.v2 import router as api_router
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
+from neutron.conf.plugins.ml2.drivers import driver_type as driver_type_conf
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as config
+from neutron.db import extraroute_db
+from neutron import manager as neutron_manager
+from neutron.plugins.ml2 import managers
+from neutron.services.ovn_l3 import exceptions as ovn_l3_exc
 from neutron.services.revisions import revision_plugin
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.extensions import test_extraroute
@@ -58,6 +66,7 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
     def setUp(self):
         mock.patch('neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
                    'impl_idl_ovn.Backend.schema_helper').start()
+        driver_type_conf.register_ml2_drivers_geneve_opts(cfg=cfg.CONF)
         cfg.CONF.set_override('max_header_size', 38, group='ml2_type_geneve')
         super(TestOVNL3RouterPlugin, self).setUp()
         revision_plugin.RevisionPlugin()
@@ -86,7 +95,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
                 ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
                 ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
-                utils.ovn_name(self.fake_network['id'])}}
+                utils.ovn_name(self.fake_network['id']),
+                ovn_const.OVN_ROUTER_IS_EXT_GW: 'False'}}
         self.fake_router_ports = [self.fake_router_port]
         self.fake_subnet = {'id': 'subnet-id',
                             'ip_version': 4,
@@ -142,7 +152,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'ext-subnet-id',
                 ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
                 ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
-                utils.ovn_name(self.fake_network['id'])},
+                utils.ovn_name(self.fake_network['id']),
+                ovn_const.OVN_ROUTER_IS_EXT_GW: 'True'},
             'gateway_chassis': ['hv1'],
             'options': {}}
         self.fake_floating_ip_attrs = {'floating_ip_address': '192.168.0.10',
@@ -296,6 +307,15 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
             'neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.ovn_client.'
             'OVNClient.delete_mac_binding_entries_by_mac',
             return_value=1)
+        self._start_mock(
+            'neutron.common.ovn.utils.is_nat_gateway_port_supported',
+            return_value=False)
+        ext_ids = {
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY: constants.TYPE_GENEVE,
+            ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY: 9000,
+        }
+        self.l3_inst._nb_ovn.ls_get.return_value.execute.return_value = (
+            mock.Mock(external_ids=ext_ids))
 
     def test__plugin_driver(self):
         # No valid mech drivers should raise an exception.
@@ -397,7 +417,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
                 ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
                 ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
-                utils.ovn_name(self.fake_network['id'])})
+                utils.ovn_name(self.fake_network['id']),
+                ovn_const.OVN_ROUTER_IS_EXT_GW: 'False'})
 
     def test_remove_router_interface_router_not_found(self):
         router_id = 'router-id'
@@ -430,8 +451,6 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                                    {'router': updated_data})
         self.l3_inst._nb_ovn.update_lrouter.assert_called_once_with(
             'neutron-router-id', enabled=True, external_ids={
-                ovn_const.OVN_GW_PORT_EXT_ID_KEY: '',
-                ovn_const.OVN_GW_NETWORK_EXT_ID_KEY: '',
                 ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
                 ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'router',
                 ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: ''})
@@ -452,8 +471,6 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
             'neutron-router-id', enabled=False,
             external_ids={ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'test',
                           ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
-                          ovn_const.OVN_GW_PORT_EXT_ID_KEY: '',
-                          ovn_const.OVN_GW_NETWORK_EXT_ID_KEY: '',
                           ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: ''})
 
     @mock.patch.object(utils, 'get_lrouter_non_gw_routes')
@@ -547,8 +564,6 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
 
         external_ids = {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: 'router',
                         ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
-                        ovn_const.OVN_GW_PORT_EXT_ID_KEY: 'gw-port-id',
-                        ovn_const.OVN_GW_NETWORK_EXT_ID_KEY: 'ext-network-id',
                         ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: ''}
         self.l3_inst._nb_ovn.create_lrouter.assert_called_once_with(
             'neutron-router-id', external_ids=external_ids,
@@ -654,6 +669,12 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         fake_network_vlan = self.fake_network
         fake_network_vlan[pnet.NETWORK_TYPE] = constants.TYPE_VLAN
         gn.return_value = fake_network_vlan
+        ext_ids = {
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY: constants.TYPE_VLAN,
+            ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY: 1500,
+        }
+        self.l3_inst._nb_ovn.ls_get.return_value.execute.return_value = (
+            mock.Mock(external_ids=ext_ids))
 
         self.l3_inst.add_router_interface(self.context, router_id,
                                           interface_info)
@@ -1137,6 +1158,58 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
             mock.call('NAT', self.fake_ovn_nat_rule.uuid, 'external_mac'),
             mock.call('NAT', self.fake_ovn_nat_rule.uuid, 'logical_port')])
 
+    def _test_create_floatingip_gateway_port_option(self, is_gw_port):
+        _nb_ovn = self.l3_inst._nb_ovn
+        _nb_ovn.is_col_present.return_value = True
+        self._get_floatingip.return_value = {'floating_port_id': 'fip-port-id'}
+        _nb_ovn.get_lrouter_nat_rules.return_value = [
+            {'external_ip': '192.168.0.10', 'logical_ip': '10.0.0.0/24',
+             'type': 'snat', 'uuid': 'uuid1'}]
+        utils.is_nat_gateway_port_supported.return_value = is_gw_port
+
+        lrp = fake_resources.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'options': {}})
+        _nb_ovn.get_lrouter_port.return_value = lrp
+        self.l3_inst.get_router.return_value = self.fake_router_with_ext_gw
+
+        self.l3_inst.create_floatingip(self.context, 'floatingip')
+        _nb_ovn.set_nat_rule_in_lrouter.assert_not_called()
+
+        expected_ext_ids = {
+            ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip['id'],
+            ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
+            ovn_const.OVN_FIP_PORT_EXT_ID_KEY:
+                self.fake_floating_ip['port_id'],
+            ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
+                self.fake_floating_ip['router_id']),
+            ovn_const.OVN_FIP_EXT_MAC_KEY: 'aa:aa:aa:aa:aa:aa',
+            ovn_const.OVN_FIP_NET_ID:
+                self.fake_floating_ip['floating_network_id']}
+
+        if is_gw_port:
+            _nb_ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+                'neutron-router-id',
+                type='dnat_and_snat',
+                logical_ip='10.0.0.10',
+                external_ip='192.168.0.10',
+                logical_port='port_id',
+                external_ids=expected_ext_ids,
+                gateway_port=lrp.uuid)
+        else:
+            _nb_ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+                'neutron-router-id',
+                type='dnat_and_snat',
+                logical_ip='10.0.0.10',
+                external_ip='192.168.0.10',
+                logical_port='port_id',
+                external_ids=expected_ext_ids)
+
+    def test_create_floatingip_with_gateway_port(self):
+        self._test_create_floatingip_gateway_port_option(True)
+
+    def test_create_floatingip_without_gateway_port(self):
+        self._test_create_floatingip_gateway_port_option(False)
+
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_floatingip')
     def test_delete_floatingip(self, df):
         nb_ovn = self.l3_inst._nb_ovn
@@ -1216,7 +1289,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         uf.return_value = self.fake_floating_ip_new
         nb_ovn.get_floatingip.return_value = (
             self.fake_ovn_nat_rule)
-        self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
+        fip = {l3_def.FLOATINGIP: {'port_id': 'port1'}}
+        self.l3_inst.update_floatingip(self.context, 'id', fip)
         nb_ovn.delete_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
             type='dnat_and_snat',
@@ -1240,13 +1314,30 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
             logical_port='new-port_id',
             external_ids=expected_ext_ids)
 
+    @mock.patch.object(extraroute_db.ExtraRoute_dbonly_mixin,
+                       'update_floatingip')
+    def test_update_floatingip_qos(self, uf):
+        nb_ovn = self.l3_inst._nb_ovn
+        nb_ovn.is_col_present.return_value = True
+        uf.return_value = self.fake_floating_ip_new
+        nb_ovn.get_floatingip.return_value = (
+            self.fake_ovn_nat_rule)
+        fip = {l3_def.FLOATINGIP: {qos_consts.QOS_POLICY_ID: 'qos_id_1'}}
+        with mock.patch.object(self.l3_inst._ovn_client._qos_driver,
+                               'update_floatingip') as ufip:
+            self.l3_inst.update_floatingip(self.context, 'id', fip)
+        nb_ovn.delete_nat_rule_in_lrouter.assert_not_called()
+        nb_ovn.add_nat_rule_in_lrouter.assert_not_called()
+        ufip.assert_called_once_with(mock.ANY, self.fake_floating_ip_new)
+
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
     def test_update_floatingip_associate(self, uf):
         self.l3_inst._nb_ovn.is_col_present.return_value = True
         self.fake_floating_ip.update({'fixed_port_id': None})
         uf.return_value = self.fake_floating_ip_new
-        self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
+        fip = {l3_def.FLOATINGIP: {'port_id': 'port1'}}
+        self.l3_inst.update_floatingip(self.context, 'id', fip)
         self.l3_inst._nb_ovn.delete_nat_rule_in_lrouter.assert_not_called()
         expected_ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip_new['id'],
@@ -1282,7 +1373,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
 
         config.cfg.CONF.set_override(
             'enable_distributed_floating_ip', True, group='ovn')
-        self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
+        fip = {l3_def.FLOATINGIP: {'port_id': 'port1'}}
+        self.l3_inst.update_floatingip(self.context, 'id', fip)
         self.l3_inst._nb_ovn.delete_nat_rule_in_lrouter.assert_not_called()
         expected_ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip_new['id'],
@@ -1310,7 +1402,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         self.fake_floating_ip.update({'fixed_port_id': 'foo'})
         self.fake_floating_ip_new.update({'port_id': 'foo'})
         uf.return_value = self.fake_floating_ip_new
-        self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
+        fip = {l3_def.FLOATINGIP: {'port_id': 'port1'}}
+        self.l3_inst.update_floatingip(self.context, 'id', fip)
         nb_ovn.delete_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
             type='dnat_and_snat',
@@ -1345,7 +1438,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         self.fake_floating_ip_new.update({'port_id': 'port_id',
                                           'fixed_port_id': 'port_id'})
         uf.return_value = self.fake_floating_ip_new
-        self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
+        fip = {l3_def.FLOATINGIP: {'port_id': 'port1'}}
+        self.l3_inst.update_floatingip(self.context, 'id', fip)
 
         nb_ovn.delete_nat_rule_in_lrouter.assert_called_once_with(
             'neutron-router-id',
@@ -1528,12 +1622,10 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                       chassis_physnets=chassis_mappings,
                       cms=chassis, availability_zone_hints=[])] * 3)
         self.mock_schedule.assert_has_calls([
-            mock.call(self.nb_idl(), self.sb_idl(),
-                      'lrp-foo-1', [], ['chassis1', 'chassis2']),
-            mock.call(self.nb_idl(), self.sb_idl(),
-                      'lrp-foo-2', [], ['chassis2']),
-            mock.call(self.nb_idl(), self.sb_idl(),
-                      'lrp-foo-3', [], [])])
+            mock.call(self.nb_idl(), 'lrp-foo-1', [],
+                      ['chassis1', 'chassis2']),
+            mock.call(self.nb_idl(), 'lrp-foo-2', [], ['chassis2']),
+            mock.call(self.nb_idl(), 'lrp-foo-3', [], [])])
         # make sure that for second port primary chassis stays untouched
         self.nb_idl().update_lrouter_port.assert_has_calls([
             mock.call('lrp-foo-1',
@@ -1586,13 +1678,20 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         ari.return_value = self.fake_router_interface_info
         grps.return_value = [interface_info]
         self.get_router.return_value = self.fake_router_with_ext_gw
-        network_attrs = {'id': 'prov-net', 'mtu': 1200}
+        mtu = 1200
+        network_attrs = {'id': 'prov-net', 'mtu': mtu}
         prov_net = fake_resources.FakeNetwork.create_one_network(
                 attrs=network_attrs).info()
         self.fake_router_port['device_owner'] = (
             constants.DEVICE_OWNER_ROUTER_GW)
         gn.return_value = prov_net
         gns.return_value = [self.fake_network]
+        ext_ids = {
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY: constants.TYPE_GENEVE,
+            ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY: mtu,
+        }
+        self.l3_inst._nb_ovn.ls_get.return_value.execute.return_value = (
+            mock.Mock(external_ids=ext_ids))
 
         self.l3_inst.add_router_interface(self.context, router_id,
                                           interface_info)
@@ -1603,6 +1702,8 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         fake_router_port_assert['options'] = {
             ovn_const.OVN_ROUTER_PORT_GW_MTU_OPTION:
             str(prov_net['mtu'])}
+        fake_router_port_assert['external_ids'][
+            ovn_const.OVN_ROUTER_IS_EXT_GW] = 'True'
 
         self.l3_inst._nb_ovn.add_lrouter_port.assert_called_once_with(
             **fake_router_port_assert)
@@ -1650,11 +1751,13 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
         fake_router_port_assert = self.fake_router_port_assert
         fake_router_port_assert['gateway_chassis'] = mock.ANY
         fake_router_port_assert['options'] = {}
+        fake_router_port_assert['external_ids'][
+            ovn_const.OVN_ROUTER_IS_EXT_GW] = 'True'
 
         self.l3_inst._nb_ovn.add_lrouter_port.assert_called_once_with(
             **fake_router_port_assert)
         # Since if_exists = True it will safely return
-        self.l3_inst._nb_ovn.lrp_set_options(
+        self.l3_inst._nb_ovn.update_lrouter_port(
             name='lrp-router-port-id', if_exists=True,
             options=fake_router_port_assert)
         # If no if_exists is provided, it is defaulted to true, so this
@@ -1713,6 +1816,13 @@ class TestOVNL3RouterPlugin(test_mech_driver.Ml2PluginV2TestCase):
                 self.context, 'router', []))
         mock_list_azs.assert_not_called()
 
+    def test_ovn_l3_router_plugin_without_ovn_mech_driver(self):
+        cfg.CONF.set_override('mechanism_drivers', [], group='ml2')
+        neutron_manager.NeutronManager._instance = None
+        with mock.patch.object(managers.TypeManager, 'initialize'):
+            self.assertRaises(ovn_l3_exc.MechanismDriverNotFound,
+                              api_router.APIRouter)
+
 
 class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
                            test_l3.L3NatDBIntTestCase,
@@ -1730,9 +1840,10 @@ class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
         return patch
 
     def setUp(self):
-        plugin = 'neutron.tests.unit.extensions.test_l3.TestNoL3NatPlugin'
+        plugin = 'neutron.tests.unit.extensions.test_l3.TestOVNL3Plugin'
         l3_plugin = ('neutron.services.ovn_l3.plugin.OVNL3RouterPlugin')
         service_plugins = {'l3_plugin_name': l3_plugin}
+        config.register_opts()
         cfg.CONF.set_default('max_routes', 3)
         ext_mgr = test_extraroute.ExtraRouteTestExtensionManager()
         super(test_l3.L3BaseForIntTests, self).setUp(
@@ -1774,6 +1885,12 @@ class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
             'OVNClient.delete_mac_binding_entries_by_mac',
             return_value=1)
         self.setup_notification_driver()
+        ext_ids = {
+            ovn_const.OVN_NETTYPE_EXT_ID_KEY: constants.TYPE_GENEVE,
+            ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY: 9000,
+        }
+        self.l3_inst._nb_ovn.ls_get.return_value.execute.return_value = (
+            mock.Mock(external_ids=ext_ids))
 
     # Note(dongj): According to bug #1657693, status of an unassociated
     # floating IP is set to DOWN. Revise expected_status to DOWN for related
